//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Collections;
using System.Net.Sockets;
using System.Text;

// Class to read/write packets to the mysql server

internal enum gweMysqlPacket_SIZE {
  HEADERLENGTH=4,
  MAXBUFFER=65535,
}

internal interface gweMysqlPacket {

  // Set to -1; packetSequence gets ++'ed before sending
  void resetPacketSequence();

  // Read in next packet; get upset if need be
  void next();

  // Return unsigned byte as int
  int ub(sbyte b);

  // Where are we?
  int currentPos_r();

  // Set position withtin packet data
  void setPos_r(int p);

  // Read byte at current position
  sbyte getCurDataPos_r();

  sbyte readByte();

  sbyte[] readSByteBlock(int l);
  byte[] readUByteBlock(int l);

  int read2Bytes();

  int read3Bytes();

  long read4Bytes();

  // Read n bytes depending
  int readnBytes();

  // read [:length]
  long readLength();

  // Read null-terminated string
  string readString();

  // Read given-length string
  string readLenString();

  // Is this the last packet?
  bool isLastDataPacket_r();

  // Reset position to start of packet data
  void clear_w();

  // Send packet
  void send();

  void writeByte(sbyte b);

  void write2Bytes(int i);

  void write3Bytes(int i);

  void write4Bytes(long i);

  // Write null-terminated string
  void writeString(string s);

  // Write string, with no termination
  void writeStringNoNull(string s);

  // ---------------------------------------------------------------------

  // Send packet
  void sendHeader(int sql_length);

  // Write string, with no termination and Send Buffer
  void writeStringNoNullandSendBuffer(string s);

  // Write string, with no termination and Send Buffer
  void writeStringNoNullandSendBufferEx(string s);

  // Flush Send Buffer
  void flushSendBuffer();
}


internal class gweMysqlPacketNormal : gweMysqlPacket {
  NetworkStream sin;
  NetworkStream sout;
  sbyte packetSequence;
  int bpos_w, bpos_r, packetLength_r;
  public bool TRACING=false;
  sbyte[] buffer_w;
  sbyte[] buffer_r;
  string encoding;
  gweNativeUnicodeMapper mapperclass;

  // Set up given an input and output datastream
  public gweMysqlPacketNormal(NetworkStream i, NetworkStream  o, string enc, gweNativeUnicodeMapper mapper) {
    sin=i;
    sout=o;
    encoding = enc;
    mapperclass = mapper;
    buffer_w=new sbyte[(int)gweMysqlPacket_SIZE.MAXBUFFER];
    buffer_r=new sbyte[(int)gweMysqlPacket_SIZE.MAXBUFFER];
  }

  // Set to -1; packetSequence gets ++'ed before sending
  public void resetPacketSequence() {
    packetSequence=-1;
  }

  // Read in next packet; get upset if need be
  public void next() {
    try {
      // [3: packetLength] [packetSequence]
      int i;
      sbyte[] b3 = new sbyte[3];
      for (i=0; i<3; i++)
        b3[i]=(sbyte)sin.ReadByte();
      packetLength_r=(int)(ub(b3[0])+(256*ub(b3[1]))+(256*256*ub(b3[2])));
      packetSequence=(sbyte)sin.ReadByte();

      if (TRACING) {
        Console.WriteLine("Packet length = "+packetLength_r);
        Console.WriteLine("Packet sequence = "+packetSequence);
      }

      // Get the data (block if necessary - thread higher up the us)
      if( buffer_r.Length < packetLength_r+1 ) // 1=terminator(0)
      {
        buffer_r = new sbyte[packetLength_r+1]; // 1=terminator(0)
      }
      //sin.readFully(buffer_r, 0, packetLength_r);
      for(int r = 0; r < packetLength_r; r++)
        buffer_r[r] = (sbyte)sin.ReadByte();
      buffer_r[packetLength_r]=0;
      bpos_r=0;

      if (TRACING) {
        for (int j=0; j<packetLength_r; j++) {
          Console.WriteLine(j+" : "+ub(buffer_r[j])+"  : ");
          if (buffer_r[j]>30)
            Console.WriteLine((char)buffer_r[j]);
          else
            Console.WriteLine("");
        }
      }

    } catch (Exception e) {
      throw new gweMysqlException("gweMysql.Next: "+e);
    }
  }

  // Return unsigned byte as int
  public int ub(sbyte b) {
    return b < 0 ? (int)(256+b) : b;
  }

  // Where are we?
  public int currentPos_r() {
    return bpos_r;
  }

  // Set position withtin packet data
  public void setPos_r(int p) {
    bpos_r=p;
  }

  // Read byte at current position
  public sbyte getCurDataPos_r() {
    return buffer_r[bpos_r];
  }

  public sbyte readByte() {
    return buffer_r[bpos_r++];
  }

  public sbyte[] readSByteBlock(int l) {
    sbyte[] block = new sbyte[l];
    if( l > 0 )
    {
      //System.arraycopy(buffer_r, bpos_r, block, 0, l);
      for(int i = 0; i < l; i++)
        block[0+i] = buffer_r[bpos_r+i];
      bpos_r += l;
    }
    return block;
  }

  public byte[] readUByteBlock(int l) {
    byte[] block = new byte[l];
    if( l > 0 )
    {
      //System.arraycopy(buffer_r, bpos_r, block, 0, l);
      for(int i = 0; i < l; i++)
        block[0+i] = (byte)buffer_r[bpos_r+i];
      bpos_r += l;
    }
    return block;
  }

  public int read2Bytes() {
    return (int)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++])));
  }

  public int read3Bytes() {
    return (int)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++]))+(256*256*ub(buffer_r[bpos_r++])));
  }

  public long read4Bytes() {
    return (long)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++]))+(256*256*ub(buffer_r[bpos_r++]))+(256*256*256*ub(buffer_r[bpos_r++])));
  }


  // Read n bytes depending
  public int readnBytes() {
    switch(ub(buffer_r[bpos_r++])) {
      case 1 : return ub(buffer_r[bpos_r++]);
      case 2 : return this.read2Bytes();
      case 3 : return this.read3Bytes();
      case 4 : return (int)this.read4Bytes();
      default : return 255;
    }
  }

  // read [:length]
  public long readLength() {
    //Console.WriteLine("flag:" + ub(buffer_r[bpos_r]));
    switch(ub(buffer_r[bpos_r])) {
      case 251 : {bpos_r++; return (long) -1; } // for NULL FIELD!!
      case 252 : {bpos_r++; return (long) read2Bytes();}
      case 253 : {bpos_r++; return (long) read3Bytes();}
      case 254 : {bpos_r++; return (long) read4Bytes();}
      default  : return (long) ub(buffer_r[bpos_r++]);
    }
  }


  // Read null-terminated string
  public string readString() {
    gweMysqlStringBuffer b = new gweMysqlStringBuffer();
    int i=bpos_r;
    while(buffer_r[i]!=0)
      b.append((char)buffer_r[i++]);
    bpos_r=i+1;
    return b.ToString();
  }

  // Read given-length string
  public string readLenString() {
    long len = this.readLength();
    if (len==-1)
      return null; // for NULL FIELD!!
    if (len==0)
      return "";
    gweMysqlStringBuffer b = new gweMysqlStringBuffer((int)len);
    for (int i=0; i<len; i++) {
      if (buffer_r[bpos_r]==0)
        break;
      else
        b.append((char)buffer_r[bpos_r++]);
    }
    return b.ToString();
  }

  // Is this the last packet?
  public bool isLastDataPacket_r() {
    return ((packetLength_r==1)&&(ub(buffer_r[0])==254));
  }

  // Reset position to start of packet data
  public void clear_w() {
    bpos_w=(int)gweMysqlPacket_SIZE.HEADERLENGTH;
  }

  // Send packet
  public void send() {
    try {
      int l=bpos_w;
      bpos_w=0;
      packetSequence++;
      this.write3Bytes(l-(int)gweMysqlPacket_SIZE.HEADERLENGTH);
      this.writeByte(packetSequence);

      if (TRACING) {
        for (int j=0; j<l; j++) {
          Console.WriteLine(j+" : "+ub(buffer_w[j])+"  : ");
          if (buffer_w[j]>30)
            Console.WriteLine((char)buffer_w[j]);
          else
            Console.WriteLine("");
        }
      }

      //sout.write(buffer_w, 0, l);
      //sout.flush();
      for(int w = 0; w < l; w++)
        sout.WriteByte((byte)buffer_w[w]);
    } catch (Exception e) {
      Console.WriteLine("send() Error : "+e);
    }
  }

  public void writeByte(sbyte b) {
    buffer_w[bpos_w++]=b;
  }

  public void write2Bytes(int i) {
    int r;
    r=i%256;
    buffer_w[bpos_w++]=(sbyte)r;
    i=i/256;
    buffer_w[bpos_w++]=(sbyte)i;
  }

  public void write3Bytes(int i) {
    int r;
    r=i%256;
    buffer_w[bpos_w++]=(sbyte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(sbyte)r;
    i=i/256;
    buffer_w[bpos_w++]=(sbyte)i;
  }

  public void write4Bytes(long i) {
    long r;
    r=i%256;
    buffer_w[bpos_w++]=(sbyte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(sbyte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(sbyte)r;
    i=i/256;
    buffer_w[bpos_w++]=(sbyte)i;
  }

  // Write null-terminated string
  public void writeString(string s) {
    int l = s.Length;
    for(int i=0; i<l; i++)
      buffer_w[bpos_w++]=(sbyte)s[i];
    buffer_w[bpos_w++]=0;
  }

  // Write string, with no termination
  public void writeStringNoNull(string s) {
    //try{
    //  sbyte[] temp_bytes = s.getBytes(encoding);
    //  int l = temp_bytes.length;
    //  System.arraycopy(temp_bytes, 0, buffer_w, bpos_w, l);
    //  bpos_w += l;
    //}
    //catch(UnsupportedEncodingException e){
    //  int l = s.length();
    //  for(int i=0; i<l; i++)
    //    buffer_w[bpos_w++]=(sbyte)s.charAt(i);
    //}
    byte[] temp_bytes;
    if( mapperclass != null )
    {
      char[] cbuf = s.ToCharArray();
      temp_bytes = mapperclass.Unicode2Native(cbuf);
    }
    else
    {
      temp_bytes = Encoding.GetEncoding(encoding).GetBytes(s);
    }
    int l = temp_bytes.Length;
    for(int i = 0; i < l; i++)
      buffer_w[bpos_w+i] = (sbyte)temp_bytes[0+i];
    bpos_w += l;
  }

  // ---------------------------------------------------------------------

  // Send packet
  public void sendHeader(int sql_length) {
    bpos_w = 0;
    packetSequence = 0;
    this.write3Bytes(1 + sql_length); // 1=(sbyte)gweMysql.QUERY
    this.writeByte(packetSequence);
    this.writeByte((sbyte)gweMysql.QUERY);
    try {
      //sout.write(buffer_w, 0, gweMysqlPacket_SIZE.HEADERLENGTH + 1); // 1=(sbyte)gweMysql.QUERY
      //sout.flush();
      for(int w = 0; w < (int)gweMysqlPacket_SIZE.HEADERLENGTH + 1; w++)
        sout.WriteByte((byte)buffer_w[w]);
      bpos_w = 0;
    }
    catch (IOException io_e) {
      throw new gweMysqlException("gweMysqlPacket.sendHeader: "+io_e);
    }
  }

  // Write string, with no termination and Send Buffer
  public void writeStringNoNullandSendBuffer(string s) {
    //try{
    //  sbyte[] temp_bytes = s.getBytes(encoding);
    //  int l = temp_bytes.length;
    //  int w = buffer_w.length;
    //  for(int i=0; i<l; i++)
    //  {
    //    buffer_w[bpos_w++] = temp_bytes[i];
    //    if( bpos_w == w )
    //    {
    //      flushSendBuffer();
    //    }
    //  }
    //}
    //catch(UnsupportedEncodingException e){
    //  int l = s.length();
    //  int w = buffer_w.length;
    //  for(int i=0; i<l; i++)
    //  {
    //    buffer_w[bpos_w++] = (sbyte)s.charAt(i);
    //    if( bpos_w == w )
    //    {
    //      flushSendBuffer();
    //    }
    //  }
    //}
    byte[] temp_bytes;
    if( mapperclass != null )
    {
      char[] cbuf = s.ToCharArray();
      temp_bytes = mapperclass.Unicode2Native(cbuf);
    }
    else
    {
      temp_bytes = Encoding.GetEncoding(encoding).GetBytes(s);
    }
    int l = temp_bytes.Length;
    int w = buffer_w.Length;
    for(int i=0; i<l; i++)
    {
      buffer_w[bpos_w++] = (sbyte)temp_bytes[i];
      if( bpos_w == w )
      {
        flushSendBuffer();
      }
    }
  }

  // Write string, with no termination and Send Buffer
  public void writeStringNoNullandSendBufferEx(string s) {
    int l = s.Length;
    int w = buffer_w.Length;
    for(int i=0; i<l; i++)
    {
      buffer_w[bpos_w++] = (sbyte)s[i];
      if( bpos_w == w )
      {
        flushSendBuffer();
      }
    }
  }

  // Flush Send Buffer
  public void flushSendBuffer() {
    try {
      //sout.write(buffer_w, 0, bpos_w);
      //sout.flush();
      for(int w = 0; w < bpos_w; w++)
        sout.WriteByte((byte)buffer_w[w]);
      bpos_w = 0;
    }
    catch(IOException io_e) {
      throw new gweMysqlException("gweMysqlPacket.writeStringNoNullandSendBuffer: "+io_e);
    }
  }
}


public class gweMysql {

  // Constants defined from mysql
  public const int SLEEP=0;
  public const int QUIT=1;
  public const int INIT_DB=2;
  public const int QUERY=3;
  public const int FIELD_LIST=4;
  public const int CREATE_DB=5;
  public const int DROP_DB=6;
  public const int RELOAD=7;
  public const int SHUTDOWN=8;
  public const int STATISTICS=9;
  public const int PROCESS_INFO=10;
  public const int CONNECT=11;

  TcpClient sock;
  //SocketChannel socketchannel;  // for New I/O
  bool useSocketChannel;
  gweMysqlPacket packet;
  string host;
  int port;
  string database;
  string user;
  string password;
  sbyte protocolVersion;
  string serverVersion;
  internal int serverMajorVersion;
  internal int serverMinorVersion;
  internal int serverSubminorVersion;
  long threadId;
  string cryptSeed;
  long columnCount;
  gweMysqlColumn[] columnResult;
  internal Hashtable columnName;
  internal Hashtable columnFullName;
  int[] dataStart;
  string encoding;
  gweNativeUnicodeMapper mapperclass;

  public gweMysql(string enc, gweNativeUnicodeMapper mapper, bool useChannel) {
    encoding = enc;
    mapperclass = mapper;
    useSocketChannel = useChannel;
  }

  // Connect to mysql host
  public void Connect(string h, int p, string d, string u, string pw) {
    lock(this) {
      host=h;
      port=p;
      database=d;
      user=u;
      password=pw;
      if( user.Equals("") )
        user="nobody";

      try {
        // Open TCP/IP socket
        if( useSocketChannel )
        {
          //// for New I/O
          //socketchannel = SocketChannel.open();
          //socketchannel.connect(new InetSocketAddress(host, port));
          //// blocking mode!!
          //ReadableByteChannel readchannel = socketchannel;
          //WritableByteChannel writechannel = socketchannel;
          //
          //// Create new packet handler
          //packet = new gweMysqlPacketChannel(readchannel,writechannel,encoding);
          packet = null;
        }
        else
        {
          sock = new TcpClient(host, port);
          NetworkStream sin = sock.GetStream();
          NetworkStream sout = sock.GetStream();

          // Create new packet handler
          packet = new gweMysqlPacketNormal(sin,sout,encoding,mapperclass);
        }

//      packet.TRACING=true;

        // Get the first packet from mysql
        packet.next();
        protocolVersion = packet.readByte();
        serverVersion = packet.readString(); // major.minor.subminor-??

        parseServerVersion(serverVersion);

        long threadId = packet.read4Bytes();
        cryptSeed = packet.readString();

        // Send who we are
        packet.clear_w();
        packet.write2Bytes(0);
        packet.write3Bytes((int)gweMysqlPacket_SIZE.MAXBUFFER);
        packet.writeString(user);
        packet.writeString(cryptPassword(password,cryptSeed));
        packet.send();

        // Check if all is ok
        packet.next();
        byte statusCode = (byte)packet.readByte();
        if (statusCode==0xff) {
          string errorString = packet.readString();
          throw new gweMysqlException(errorString);
        } else if (statusCode==0x00) {
          long affectedRows = packet.readLength();
          long uniqueId = packet.readLength();
        } else {
          throw new gweMysqlException("Unknown status code '"+statusCode+"'");
        }

        // Ok, we're authenticated, so select our database
        packet.clear_w();
        packet.resetPacketSequence();
        packet.writeByte((sbyte)gweMysql.INIT_DB);
        packet.writeStringNoNull(database);
        packet.send();

        // Check if all is well
        packet.next();
        statusCode = (byte)packet.readByte();
        if (statusCode==0xff) {
          string errorString = packet.readString();
          throw new gweMysqlException(errorString);
        } else if (statusCode==0x00) {
          long affectedRows = packet.readLength();
          long uniqueId = packet.readLength();
        } else {
          throw new gweMysqlException("Unknown status code '"+statusCode+"'");
        }
      } catch (Exception e) {
        throw new gweMysqlException("gweMysql.Connect: "+e);
      }
    }
  }

  // major.minor.subminor-??
  void parseServerVersion(string verstr) {
    serverMajorVersion    = -1;
    serverMinorVersion    = -1;
    serverSubminorVersion = -1;
    string temp_verstr;
    int pidx = verstr.IndexOf('.');
    if( pidx >= 1 )
    {
      try {
        serverMajorVersion = Int32.Parse(verstr.Substring(0, pidx-0));
      }
      catch(FormatException ex) {
        throw new gweMysqlException("Illegal Server Version : "+verstr);
      }
    }
    else
    {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }

    temp_verstr = verstr.Substring(pidx+1, verstr.Length-(pidx+1));
    pidx = temp_verstr.IndexOf('.');
    if( pidx >= 1 )
    {
      try {
        serverMinorVersion = Int32.Parse(temp_verstr.Substring(0, pidx-0));
      }
      catch(FormatException ex) {
        throw new gweMysqlException("Illegal Server Version : "+verstr);
      }
    }
    else
    {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }

    temp_verstr = temp_verstr.Substring(pidx+1, temp_verstr.Length-(pidx+1));
    int lastidx;
    for(lastidx = 0; lastidx < temp_verstr.Length; lastidx++)
    {
      char c = temp_verstr[lastidx];
      if( c < '0' || c > '9' )
        break;
    }
    try {
      serverSubminorVersion = Int32.Parse(temp_verstr.Substring(0, lastidx-0));
    }
    catch(FormatException ex) {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }
  }

  // return    x > 0 or 0 or x < 0
  internal int judgeServerVersion(int maj, int min, int sub) {
    if( serverMajorVersion > maj )
      return 1;
    else if( serverMajorVersion < maj )
      return -1;

    if( serverMinorVersion > min )
      return 1;
    else if( serverMinorVersion < min )
      return -1;

    if( serverSubminorVersion > sub )
      return 1;
    else if( serverSubminorVersion < sub )
      return -1;

    return 0;
  }

  internal string getServerVersion() {
    return serverVersion;
  }
  int getServerMajorVersion() {
    return serverMajorVersion;
  }
  int getServerMinorVersion() {
    return serverMinorVersion;
  }
  int getServerSubminorVersion() {
    return serverSubminorVersion;
  }

  // Return hash for given string
  long hashPassword(string p) {
    long  nr=1345345333;
    long  nr2=7;
    long tmp;
    for(int i=0; i<p.Length; i++) {
      if( (p[i]==' ') || (p[i]=='\t') )
        continue;
      tmp=(long)p[i];
      nr^= (((nr & 63)+nr2)*tmp)+ (nr << 8);
      nr2+=tmp;
    }
    return nr & (((long) 1L << 31) -1L);
  }

  // Return encrypted response, given clear password and seed message
  string cryptPassword(string p, string m) {
    if( p == null || p.Length == 0 )
      return p;

    long hp, hm, s1, s2;
    long max_value= 0x01FFFFFF;
    double d;
    sbyte b;

    hp=this.hashPassword(p);
    hm=this.hashPassword(m);
    long nr=hp^hm;
    nr%=max_value;
    s1=nr;
    s2=nr/2;

    char[] cr = new char[m.Length];
    for (int i=0; i<m.Length; i++) {
      s1=(s1*3+s2)%max_value;
      s2=(s1+s2+33)%max_value;
      d = (double)s1/max_value;
      b = (sbyte)Math.Floor((d*31)+64);
      cr[i]=(char)b;
    }
    return new string(cr);
  }

  public gweMysql sqlQuery(string q) {

    // Send query command and sql query string
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((sbyte)gweMysql.QUERY);
    packet.writeStringNoNull(q);
    packet.send();

    // Check if we're ok
    packet.next();
    columnCount = packet.readLength();
    if (columnCount==255 || columnCount==-1) { // for NULL FIELD!!
      string errorString = packet.readString();
      throw new gweMysqlException(errorString);
    } else if (columnCount==0) {
      //throw new gweMysqlException("No columns returned.");
      return null;
    } else {
      columnResult = new gweMysqlColumn[(int)columnCount];
      columnName=new Hashtable((int)columnCount);
      columnFullName=new Hashtable((int)columnCount);
      dataStart = new int[(int)columnCount];

      // Read in the column information
      for (int i=0;i<columnCount; i++) {
        packet.next();
        string tableName=packet.readLenString();
        string colName=packet.readLenString();
        int colLength = packet.readnBytes();
        int colType=packet.readnBytes();
        packet.readByte(); // We know it's currently 2
        short colFlag = (short)packet.ub(packet.readByte());
        int colDecimals = packet.ub(packet.readByte());

        columnResult[i] = new gweMysqlColumn(tableName, colName, colLength, colType, colFlag, colDecimals);
        columnName.Add(colName, i);
        string fullName = String.Concat(tableName, ".", colName);
        columnFullName.Add(fullName, i);
      }
      packet.next(); //Redundant - we know how many, this is end of section marker packet
    }
    return this;
  }


  public long[] sqlUpdate(string q) {

    // Send query command and sql update
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((sbyte)gweMysql.QUERY);
    packet.writeStringNoNull(q);
    packet.send();

    // check if we're ok
    packet.next();
    int updateStatus = packet.ub(packet.readByte());
    if (updateStatus==255) {
      string errorString = packet.readString();
      throw new gweMysqlException(errorString);
    }
    long affectedRows = packet.readLength();
    long updateId = packet.readLength();
    long[] count_and_id = new long[2];
    count_and_id[0] = affectedRows;
    count_and_id[1] = updateId;
    return count_and_id;
  }


  public void sqlQuit() {

    // Send quit command to mysql
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((sbyte)gweMysql.QUIT);
    packet.send();

  }



  public bool nextResult() {
    int i, p;

    // Get the next incoming packet
    packet.next();
    // It it's not the last one, note start positions of data
    if(!packet.isLastDataPacket_r()) {
      for (i=0; i<columnCount; i++) {
        p=packet.currentPos_r();
        dataStart[i]=p;
        switch(packet.ub(packet.getCurDataPos_r()))
        {
          case 252: p += 2; break;
          case 253: p += 3; break;
          case 254: p += 4; break;
          default: break;
        }
        long next_len = packet.readLength();
        if( next_len == -1 )                 // for NULL FIELD!!
          next_len = 0;                      // for NULL FIELD!!
        packet.setPos_r(1+p+(int)next_len);
      }
      return true;
    }
    return false;
  }

  public int getColumnCount() {
    return (int) columnCount;
  }

  public gweMysqlColumn getColumn(int c) {
    c--;  // Columnd go from 1..columnCount to 0..columnCount-1
    if ((c<columnCount)&&(c>=0)) {
      return columnResult[c];
    } else {
      throw new gweMysqlException("Invalid column number : "+c);
    }
  }

  public bool isNull(int c) {
    //Console.WriteLine("isNull:" + c);
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      return (l==-1); // for NULL FIELD!!
    } else {
      throw new gweMysqlException("Invalid column number : "+c);
    }
  }

  public int getColumnNum(string cn) {
    object nI = columnName[cn];
    if (nI==null)
      nI = columnFullName[cn];
    if (nI!=null)
      return ((int)nI)+1;
    throw new gweMysqlException("Invalid column name : "+cn);
  }


  public string getString(int c) {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      return packet.readLenString();
    } else {
      throw new gweMysqlException("Invalid column number : "+c);
    }
  }

  public string getString(string cn) {
    object nI = columnName[cn];
    if (nI==null)
      nI = columnFullName[cn];
    if (nI!=null) {
      int c = ((int)nI)+1;
      if (c<columnCount) {
        packet.setPos_r(dataStart[c]);
        return packet.readLenString();
      }
    }
    throw new gweMysqlException("Invalid column string : "+cn);
  }

  public sbyte[] getSBytes(int c) {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      if( l == -1 )                   // for NULL FIELD!!
        return null;                  // for NULL FIELD!!
      //sbyte[] r = new sbyte[(int)l];
      //for (int i=0; i<(int)l; i++) {
      //  r[i]=packet.readByte();
      //}
      sbyte[] r = packet.readSByteBlock((int)l);
      return r;
    }
    throw new gweMysqlException("Invalid column number : "+c);
  }

  public byte[] getUBytes(int c) {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      if( l == -1 )                   // for NULL FIELD!!
        return null;                  // for NULL FIELD!!
      //sbyte[] r = new sbyte[(int)l];
      //for (int i=0; i<(int)l; i++) {
      //  r[i]=packet.readByte();
      //}
      byte[] r = packet.readUByteBlock((int)l);
      return r;
    }
    throw new gweMysqlException("Invalid column number : "+c);
  }

  public void reConnect() {
    try {
      sqlQuit();
      sock.Close();
      this.Connect(host, port, database, user, password);
    } catch (Exception e) {
      throw new gweMysqlException("gweMysql.reConnect: "+e);
    }
  }


  public void Close() {
    try {
      sqlQuit();
      sock.Close();
    } catch (Exception e) {
      throw new gweMysqlException("gweMysql.Close: "+e);
    }
  }

  internal gweMysqlPacket getPacket(){
    return (packet);
  }

  internal string getEncoding() {
    return encoding;
  }

  internal gweNativeUnicodeMapper getNativeUnicodeMapper() {
    return mapperclass;
  }
}

}
