//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Collections;
using System.Reflection;
using System.Threading;
using net.sql;
using netx.sql;

[Serializable]
public class gweMysqlConnectionPoolDataSource : ConnectionPoolDataSource {

  private static gwe.sql.gweMysqlDriver driver = null;
  private static readonly String factoryName = "gwe.sql.gweMysqlConnectionPoolDataSourceFactory";
  private int seconds = 0;
  private string networkProtocol = "jdbc:mysql:";
  private string serverName = "";
  private int portNumber = 3306;
  private string databaseName = "";
  private string dataSourceName = null;
  private string description = "";
  private string user = null;
  private string password = null;
  private string characterEncoding = "shift-jis";
  private string nativeUnicodeMapper = null; // nativeUnicodeMapper support!!
  private bool add5CAfter5C = false;
  private int maxPoolSize = 0;
  private bool useNewIO = false;
  private bool debugSQL = false;

  private ArrayList pool = new ArrayList();
  private ArrayList busy = new ArrayList();

  [NonSerialized]
  private TextWriter logger = null;

  static gweMysqlConnectionPoolDataSource() {
    try {
      Type callerC = Type.GetType("gwe.sql.gweMysqlDriver", true);
      ConstructorInfo ci = callerC.GetConstructor(new Type[0]);
      driver = (gwe.sql.gweMysqlDriver)ci.Invoke(new object[0]);
    }
    catch(Exception e) {
      throw new SystemException("Could not Load Driver gwe.sql.gweMysqlDriver class");
    }
  }

  public PooledConnection getPooledConnection() {
    return getPooledConnection(getUser(), getPassword());
  }

  public PooledConnection getPooledConnection(string username, string password) {
    Hashtable info = new Hashtable();
    info.Add("user", username);
    info.Add("password", password);

    if( characterEncoding != null )
      info.Add("characterEncoding", characterEncoding);
    if( nativeUnicodeMapper != null )
      info.Add("nativeUnicodeMapper", nativeUnicodeMapper);

    info.Add("add5CAfter5C", add5CAfter5C.ToString());
    info.Add("useNewIO", useNewIO.ToString());
    info.Add("debugSQL", debugSQL.ToString());

    string url = networkProtocol + "//" +  serverName + ":" + portNumber + "/"
                 + databaseName;

    return getPooledConnectionSub(url, info);
  }

  /*
  public Reference getReference() throws NamingException {
    Reference ref_obj = new Reference(getClass().getName(), factoryName, null);
    ref_obj.add(new StringRefAddr("networkProtocol", getNetworkProtocol()));
    ref_obj.add(new StringRefAddr("serverName",      getServerName()));
    ref_obj.add(new StringRefAddr("portNumber",      getPortNumber().toString()));
    ref_obj.add(new StringRefAddr("databaseName",    getDatabaseName()));
    ref_obj.add(new StringRefAddr("user",            getUser()));
    ref_obj.add(new StringRefAddr("password",        getPassword()));
    ref_obj.add(new StringRefAddr("characterEncoding",
                                                     getCharacterEncoding()));
    ref_obj.add(new StringRefAddr("nativeUnicodeMapper",
                                                     getNativeUnicodeMapper()));
    ref_obj.add(new StringRefAddr("add5CAfter5C",    isAdd5CAfter5C().toString()));
    ref_obj.add(new StringRefAddr("maxPoolSize",     getMaxPoolSize().toString()));
    ref_obj.add(new StringRefAddr("useNewIO",        isUseNewIO().toString()));
    ref_obj.add(new StringRefAddr("debugSQL",        isDebugSQL().toString()));
    return ref_obj;
  }
  */

  public PooledConnection getPooledConnectionSub(string url, Hashtable info) {
    lock(pool) {
      while(true)
      {
        if( pool.Count <= 0 )
        {
          if( maxPoolSize <= 0 || busy.Count < maxPoolSize )
            break;
          
          while( pool.Count <= 0 )
          {
            try {
              Monitor.Wait(pool);
            }
            catch(System.Threading.ThreadInterruptedException itre) {
              throw new SQLException(itre.Message);
            }
          }
        }

        gweMysqlPooledConnection pooled_conn =
                                  (gweMysqlPooledConnection)pool[0];
        pool.RemoveAt(0);
        Connection org_conn = pooled_conn.getOrgConnection();
        try {
          org_conn.setAutoCommit(false);
          org_conn.setAutoCommit(true);
          busy.Add(pooled_conn);
          return pooled_conn;
        }
        catch(SQLException ex) {
          Console.WriteLine(ex.Message);
        }
        try {
          org_conn.close();
        }
        catch(SQLException ex) {
          ;
        }
      }

      gweMysqlPooledConnection ret_conn =
               new gweMysqlPooledConnection(this, driver.connect(url, info));
      busy.Add(ret_conn);
      return ret_conn;
    }
  }

  public int getPooledCount() {
    lock(pool) {
      return pool.Count;
    }
  }

  public void sweepPooledConnections() {
    lock(pool) {
      for(int i = 0; i < pool.Count; i++)
      {
        gweMysqlPooledConnection pooled_conn =
                                 (gweMysqlPooledConnection)pool[i];
        Connection org_conn = pooled_conn.getOrgConnection();
        try {
          org_conn.close();
        }
        catch(SQLException ex) {
          ;
        }
      }
      pool.Clear();
    }
  }

  internal void storeConnection(gweMysqlPooledConnection conn) {
    lock(pool) {
      int i = busy.IndexOf(conn);
      if( i >= 0 )
      {
        busy.RemoveAt(i);
        pool.Add(conn);
      }
      Monitor.PulseAll(pool);
    }
  }

  /**
   * Prints a message to the current JDBC Log writer.
   */
  public void println(string mes) {
    lock(typeof(gweMysqlConnectionPoolDataSource)) {
      if( logger != null )
      {
        logger.WriteLine(mes);
        logger.Flush();
      }
    }
  }

  /**
   *  Set the log writer.
   *
   *  @see javax.sql.DataSource
   */
  public void setLogWriter(TextWriter logger) {
    this.logger = logger;
  }
  public TextWriter getLogWriter() {
    return logger;
  }

  /**
   *  Set the login timeout (in seconds).  If this value is set to
   *  something other than zero, any method that gets a connection
   *  is guaranteed to return (or throw a <tt>SQLException</tt>) within the
   *  specified amount of time.  If this value is set to zero,
   *  any method that gets a connection will block indefinately
   *  until a connection is available.  The default value is zero.
   *
   *  @see javax.sql.DataSource
   */
  public void setLoginTimeout(int seconds) {
    this.seconds = seconds;
  }
  public int getLoginTimeout() {
    return seconds;
  }

  // An optional database protocol to use when communicating with InterServer.
  // This can only be jdbc:mysql: for MySQL.
  public void setNetworkProtocol(string networkProtocol) {
    this.networkProtocol = networkProtocol;
  }
  public String getNetworkProtocol() {
    return networkProtocol;
  }

  // The server name where MySQL is running.
  public void setServerName(string serverName) {
    this.serverName = serverName;
  }
  public string getServerName() {
    return serverName;
  }

  // The TCP/IP port number where MySQL listens
  // for connection requests on serverName.
  public void setPortNumber(int portNumber) {
    this.portNumber = portNumber;
  }
  public int getPortNumber() {
    return portNumber;
  }

  // Absolute path to a database on a server.
  public void setDatabaseName(string databaseName) {
    this.databaseName = databaseName;
  }
  public string getDatabaseName() {
    return databaseName;
  }

  // An optional data source name; used to name an underlying XADataSource,
  // or ConnectionPoolDataSource when pooling of connections is done.
  public void setDataSourceName(string dataSourceName) {
    this.dataSourceName = dataSourceName;
  }
  public string getDataSourceName() {
    return dataSourceName;
  }

  // An optional text description of the data source.
  public void setDescription(string description) {
    this.description = description;
  }
  public string getDescription() {
    return description;
  }

  // The user name for the connection.
  public void setUser(string user) {
    this.user = user;
  }
  public string getUser() {
    return user;
  }

  // The user's password for the connection.
  public void setPassword(string password) {
    this.password = password;
  }
  public string getPassword() {
    return password;
  }

  // The character encoding for the connection.
  public void setCharacterEncoding(string characterEncoding) {
    this.characterEncoding = characterEncoding;
  }
  public string getCharacterEncoding() {
    return characterEncoding;
  }

  // The nativeUnicodeMapper for the connection.
  public void setNativeUnicodeMapper(string nativeUnicodeMapper) {
    this.nativeUnicodeMapper = nativeUnicodeMapper;
  }
  public string getNativeUnicodeMapper() {
    return nativeUnicodeMapper;
  }

  // Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString.
  public void setAdd5CAfter5C(bool add5CAfter5C) {
    this.add5CAfter5C = add5CAfter5C;
  }
  public bool isAdd5CAfter5C() {
    return add5CAfter5C;
  }

  // Max Size of Pooled Connection.
  public void setMaxPoolSize(int maxPoolSize) {
    this.maxPoolSize = maxPoolSize;
  }
  public int getMaxPoolSize() {
    return maxPoolSize;
  }

  // Use SocketChannel and DirectBuffer of JDK1.4's New I/O API
  public void setUseNewIO(bool useNewIO) {
    this.useNewIO = useNewIO;
  }
  public bool isUseNewIO() {
    return useNewIO;
  }

  // Trace SQL Statement with LogWriter.
  public void setDebugSQL(bool debugSQL) {
    this.debugSQL = debugSQL;
  }
  public bool isDebugSQL() {
    return debugSQL;
  }
}

}
