//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//en
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.Collections;
using net.sql;

public class gweMysqlDatabaseMetaData : DatabaseMetaData {

  private gweMysqlConnection conn;
  private const string ProductName = "MySQL";
  private const string DriverName = "gweMysql JDBC Driver";
  private const string DriverVersion = "0.9.2";
  private bool add_quote = false;

  public gweMysqlDatabaseMetaData(gweMysqlConnection org_conn) {
    this.conn = org_conn;

    //if( conn.judgeServerVersion(3, 23, 6) >= 0 )
    //  add_quote = true;
    add_quote = false;
  }

  string trimQuote(string quoted_str) {
    int idx1 = quoted_str.IndexOf('`');
    int idx2 = quoted_str.LastIndexOf('`');
    if( idx1 != -1 && idx2 != -1 && idx1 != idx2 )
      return quoted_str.Substring(idx1+1, idx2-(idx1+1));
    return quoted_str;
  }

  /**
   * Can all the procedures returned by getProcedures be called by the
   * current user?
   *
   * @return true if so
   */
  public bool allProceduresAreCallable() {
    return false;
  }

  /**
   * Can all the tables returned by getTable be SELECTed by the
   * current user?
   *
   * @return true if so
   */
  public bool allTablesAreSelectable() {
    return false;
  }

  /**
   * What's the url for this database?
   *
   * @return the url or null if it can't be generated
   */
  public string getURL() {
    return conn.getUrl();
  }

  /**
   * What's our user name as known to the database?
   *
   * @return our database user name
   */
  public string getUserName() {
    return conn.getUsername();
  }

  /**
   * Is the database in read-only mode?
   *
   * @return true if so
   */
  public bool isReadOnly() {
    return false;
  }

  /**
   * Are NULL values sorted high?
   *
   * @return true if so
   */
  public bool nullsAreSortedHigh() {
    return false;
  }

  /**
   * Are NULL values sorted low?
   *
   * @return true if so
   */
  public bool nullsAreSortedLow() {
    return true;
  }

  /**
   * Are NULL values sorted at the start regardless of sort order?
   *
   * @return true if so
   */
  public bool nullsAreSortedAtStart() {
    return false;
  }

  /**
   * Are NULL values sorted at the end regardless of sort order?
   *
   * @return true if so
   */
  public bool nullsAreSortedAtEnd() {
    return false;
  }

  /**
   * What's the name of this database product?
   *
   * @return database product name
   */
  public string getDatabaseProductName() {
    return ProductName;
  }

  /**
   * What's the version of this database product?
   *
   * @return database version
   */
  public string getDatabaseProductVersion() {
    return conn.getServerVersion();
  }

  /**
   * What's the name of this JDBC driver?
   *
   * @return JDBC driver name
   */
  public string getDriverName() {
    return DriverName;
  }

  /**
   * What's the version of this JDBC driver?
   *
   * @return JDBC driver version
   */
  public string getDriverVersion() {
    return DriverVersion;
  }

  /**
   * What's this JDBC driver's major version number?
   *
   * @return JDBC driver major version
   */
  public int getDriverMajorVersion() {
    return conn.getDriverMajorVersion();
  }

  /**
   * What's this JDBC driver's minor version number?
   *
   * @return JDBC driver minor version number
   */
  public int getDriverMinorVersion() {
    return conn.getDriverMinorVersion();
  }

  /**
   * Does the database store tables in a local file?
   *
   * @return true if so
   */
  public bool usesLocalFiles() {
    return false;
  }

  /**
   * Does the database use a file for each table?
   *
   * @return true if the database uses a local file for each table
   */
  public bool usesLocalFilePerTable() {
    return false;
  }

  /**
   * Does the database support mixed case unquoted SQL identifiers?
   *
   * @return true if so
   */
  public bool supportsMixedCaseIdentifiers() {
    return false;
  }

  /**
   * Does the database store mixed case unquoted SQL identifiers in
   * upper case?
   *
   * @return true if so
   */
  public bool storesUpperCaseIdentifiers() {
    return false;
  }

  /**
   * Does the database store mixed case unquoted SQL identifiers in
   * lower case?
   *
   * @return true if so
   */
  public bool storesLowerCaseIdentifiers() {
    return false;
  }

  /**
   * Does the database store mixed case unquoted SQL identifiers in
   * mixed case?
   *
   * @return true if so
   */
  public bool storesMixedCaseIdentifiers() {
    return false;
  }

  /**
   * Does the database support mixed case quoted SQL identifiers?
   *
   * A JDBC compliant driver will always return true.
   *
   * @return true if so
   */
  public bool supportsMixedCaseQuotedIdentifiers() {
    return false;
  }

  /**
   * Does the database store mixed case quoted SQL identifiers in
   * upper case?
   *
   * A JDBC compliant driver will always return true.
   *
   * @return true if so
   */
  public bool storesUpperCaseQuotedIdentifiers() {
    return false;
  }

  /**
   * Does the database store mixed case quoted SQL identifiers in
   * lower case?
   *
   * A JDBC compliant driver will always return false.
   *
   * @return true if so
   */
  public bool storesLowerCaseQuotedIdentifiers() {
    return false;
  }

  /**
   * Does the database store mixed case quoted SQL identifiers in
   * mixed case?
   *
   * A JDBC compliant driver will always return false.
   *
   * @return true if so
   */
  public bool storesMixedCaseQuotedIdentifiers() {
    return false;
  }

  /**
   * What's the string used to quote SQL identifiers?
   * This returns a space " " if identifier quoting isn't supported.
   *
   * A JDBC compliant driver always uses a double quote character.
   *
   * @return the quoting string
   */
  public string getIdentifierQuoteString() {
    //return " ";
    return "";
  }

  /**
   * Get a comma separated list of all a database's SQL keywords
   * that are NOT also SQL92 keywords.
   *
   * @return the list
   */
  public string getSQLKeywords() {
    string SQLKeywords = "AUTO_INCREMENT,BINARY,BLOB,ENUM,INFILE,LOAD,MEDIUMINT,OPTION,OUTFILE,REPLACE,SET,TEXT,UNSIGNED,ZEROFILL";
    return SQLKeywords;
  }

  /**
   * Get a comma separated list of math functions.
   *
   * @return the list
   */
  public string getNumericFunctions() {
    string NumericFunctions = "ABS,ACOS,ASIN,ATAN,ATAN2,BIT_COUNT,CEILING,COS,COT,DEGREES,EXP,FLOOR,LOG,LOG10,MAX,MIN,MOD,PI,POW,POWER,RADIANS,RAND,ROUND,SIN,SQRT,TAN,TRUNCATE";
    return NumericFunctions;
  }

  /**
   * Get a comma separated list of string functions.
   *
   * @return the list
   */
  public string getStringFunctions() {
    string StringFunctions = "ACII,CHAR,CHAR_LENGTH,CHARACTER_LENGTH,CONCAT,ELT,FIELD,FIND_IN_SET,INSERT,INSTR,INTERVAL,LCASE,LEFT,LENGTH,LOCATE,LOWER,LTRIM,MID,POSITION,OCTET_LENGTH,REPEAT,REPLACE,REVERSE,RIGHT,RTRIM,SPACE,SOUNDEX,SUBSTRING,SUBSTRING_INDEX,TRIM,UCASE,UPPER";
    return StringFunctions;
  }

  /**
   * Get a comma separated list of system functions.
   *
   * @return the list
   */
  public string getSystemFunctions() {
    string SystemFunctions = "DATABASE,USER,SYSTEM_USER,SESSION_USER,PASSWORD,ENCRYPT,LAST_INSERT_ID,VERSION";
    return SystemFunctions;
  }

  /**
   * Get a comma separated list of time and date functions.
   *
   * @return the list
   */
  public string getTimeDateFunctions() {
    string TimeDateFunctions = "DAYOFWEEK,WEEKDAY,DAYOFMONTH,DAYOFYEAR,MONTH,DAYNAME,MONTHNAME,QUARTER,WEEK,YEAR,HOUR,MINUTE,SECOND,PERIOD_ADD,PERIOD_DIFF,TO_DAYS,FROM_DAYS,DATE_FORMAT,TIME_FORMAT,CURDATE,CURRENT_DATE,CURTIME,CURRENT_TIME,NOW,SYSDATE,CURRENT_TIMESTAMP,UNIX_TIMESTAMP,FROM_UNIXTIME,SEC_TO_TIME,TIME_TO_SEC";
    return TimeDateFunctions;
  }

  /**
   * This is the string that can be used to escape '_' or '%' in
   * the string pattern style catalog search parameters.
   *
   * The '_' character represents any single character.
   * The '%' character represents any sequence of zero or
   * more characters.
   * @return the string used to escape wildcard characters
   */
  public string getSearchStringEscape() {
    string SearchStringEscape = "\\";
    return SearchStringEscape;
  }

  /**
   * Get all the "extra" characters that can be used in unquoted
   * identifier names (those beyond a-z, 0-9 and _).
   *
   * @return the string containing the extra characters
   */
  public string getExtraNameCharacters() {
    string ExtraNameCharacters = "";
    return ExtraNameCharacters;
  }

  /**
   * Is "ALTER TABLE" with add column supported?
   *
   * @return true if so
   */
  public bool supportsAlterTableWithAddColumn() {
    return true;
  }

  /**
   * Is "ALTER TABLE" with drop column supported?
   *
   * @return true if so
   */
  public bool supportsAlterTableWithDropColumn() {
    return true;
  }

  /**
   * Is column aliasing supported?
   *
   * If so, the SQL AS clause can be used to provide names for
   * computed columns or to provide alias names for columns as
   * required.
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsColumnAliasing() {
    return true;
  }

  /**
   * Are concatenations between NULL and non-NULL values NULL?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool nullPlusNonNullIsNull() {
    return true;
  }

  /**
   * Is the CONVERT function between SQL types supported?
   *
   * @return true if so
   */
  public bool supportsConvert() {
    return false;
  }

  /**
   * Is CONVERT between the given SQL types supported?
   *
   * @param fromType the type to convert from
   * @param toType the type to convert to
   * @return true if so
   * @see Types
   */
  public bool supportsConvert(int fromType, int toType) {
    bool retval = false;
    switch(fromType)
    {
      case net.sql.Types.CHAR:
      case net.sql.Types.VARCHAR:
      case net.sql.Types.LONGVARCHAR:
      case net.sql.Types.BINARY:
      case net.sql.Types.VARBINARY:
      case net.sql.Types.LONGVARBINARY:
        switch(toType)
        {
          case net.sql.Types.DECIMAL:
          case net.sql.Types.NUMERIC:
          case net.sql.Types.REAL:
          case net.sql.Types.TINYINT:
          case net.sql.Types.SMALLINT:
          case net.sql.Types.INTEGER:
          case net.sql.Types.BIGINT:
          case net.sql.Types.FLOAT:
          case net.sql.Types.DOUBLE:
          case net.sql.Types.CHAR:
          case net.sql.Types.VARCHAR:
          case net.sql.Types.LONGVARCHAR:
          case net.sql.Types.BINARY:
          case net.sql.Types.VARBINARY:
          case net.sql.Types.LONGVARBINARY:
          case net.sql.Types.OTHER:
          case net.sql.Types.DATE:
          case net.sql.Types.TIME:
          case net.sql.Types.TIMESTAMP:
            retval = true;
            break;
          default:
            break;
        }
        break;
      case net.sql.Types.BIT:
        break;
      case net.sql.Types.DECIMAL:
      case net.sql.Types.NUMERIC:
      case net.sql.Types.REAL:
      case net.sql.Types.TINYINT:
      case net.sql.Types.SMALLINT:
      case net.sql.Types.INTEGER:
      case net.sql.Types.BIGINT:
      case net.sql.Types.FLOAT:
      case net.sql.Types.DOUBLE:
        switch(toType)
        {
          case net.sql.Types.DECIMAL:
          case net.sql.Types.NUMERIC:
          case net.sql.Types.REAL:
          case net.sql.Types.TINYINT:
          case net.sql.Types.SMALLINT:
          case net.sql.Types.INTEGER:
          case net.sql.Types.BIGINT:
          case net.sql.Types.FLOAT:
          case net.sql.Types.DOUBLE:
          case net.sql.Types.CHAR:
          case net.sql.Types.VARCHAR:
          case net.sql.Types.LONGVARCHAR:
          case net.sql.Types.BINARY:
          case net.sql.Types.VARBINARY:
          case net.sql.Types.LONGVARBINARY:
            retval = true;
            break;
          default:
            break;
        }
        break;
      case net.sql.Types.DATE:
      case net.sql.Types.TIME:
        switch(toType)
        {
          case net.sql.Types.CHAR:
          case net.sql.Types.VARCHAR:
          case net.sql.Types.LONGVARCHAR:
          case net.sql.Types.BINARY:
          case net.sql.Types.VARBINARY:
          case net.sql.Types.LONGVARBINARY:
            retval = true;
            break;
          default:
            break;
        }
        break;
      case net.sql.Types.TIMESTAMP:
        switch(toType)
        {
          case net.sql.Types.CHAR:
          case net.sql.Types.VARCHAR:
          case net.sql.Types.LONGVARCHAR:
          case net.sql.Types.BINARY:
          case net.sql.Types.VARBINARY:
          case net.sql.Types.LONGVARBINARY:
          case net.sql.Types.TIME:
          case net.sql.Types.DATE:
            retval = true;
            break;
          default:
            break;
        }
        break;
      case net.sql.Types.NULL:
        break;
      case net.sql.Types.OTHER:
        switch(toType)
        {
          case net.sql.Types.CHAR:
          case net.sql.Types.VARCHAR:
          case net.sql.Types.LONGVARCHAR:
          case net.sql.Types.BINARY:
          case net.sql.Types.VARBINARY:
          case net.sql.Types.LONGVARBINARY:
            retval = true;
            break;
          default:
            break;
        }
        break;
      default:
        break;
    }
    return retval;
  }

  /**
   * Are table correlation names supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsTableCorrelationNames() {
    return true;
  }

  /**
   * If table correlation names are supported, are they restricted
   * to be different from the names of the tables?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsDifferentTableCorrelationNames() {
    return true;
  }

  /**
   * Are expressions in "ORDER BY" lists supported?
   *
   * @return true if so
   */
  public bool supportsExpressionsInOrderBy() {
    return true;
  }

  /**
   * Can an "ORDER BY" clause use columns not in the SELECT?
   *
   * @return true if so
   */
  public bool supportsOrderByUnrelated() {
    return false;
  }

  /**
   * Is some form of "GROUP BY" clause supported?
   *
   * @return true if so
   */
  public bool supportsGroupBy() {
    return true;
  }

  /**
   * Can a "GROUP BY" clause use columns not in the SELECT?
   *
   * @return true if so
   */
  public bool supportsGroupByUnrelated() {
    return false;
  }

  /**
   * Can a "GROUP BY" clause add columns not in the SELECT
   * provided it specifies all the columns in the SELECT?
   *
   * @return true if so
   */
  public bool supportsGroupByBeyondSelect() {
    return true;
  }

  /**
   * Is the escape character in "LIKE" clauses supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsLikeEscapeClause() {
    return true;
  }

  /**
   * Are multiple ResultSets from a single execute supported?
   *
   * @return true if so
   */
  public bool supportsMultipleResultSets() {
    return false;
  }

  /**
   * Can we have multiple transactions open at once (on different
   * connections)?
   *
   * @return true if so
   */
  public bool supportsMultipleTransactions() {
    return true;
  }

  /**
   * Can columns be defined as non-nullable?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsNonNullableColumns() {
    return true;
  }

  /**
   * Is the ODBC Minimum SQL grammar supported?
   *
   * All JDBC compliant drivers must return true.
   *
   * @return true if so
   */
  public bool supportsMinimumSQLGrammar() {
    return true;
  }

  /**
   * Is the ODBC Core SQL grammar supported?
   *
   * @return true if so
   */
  public bool supportsCoreSQLGrammar() {
    return true;
  }

  /**
   * Is the ODBC Extended SQL grammar supported?
   *
   * @return true if so
   */
  public bool supportsExtendedSQLGrammar() {
    return false;
  }

  /**
   * Is the ANSI92 entry level SQL grammar supported?
   *
   * All JDBC compliant drivers must return true.
   *
   * @return true if so
   */
  public bool supportsANSI92EntryLevelSQL() {
    return true;
  }

  /**
   * Is the ANSI92 intermediate SQL grammar supported?
   *
   * @return true if so
   */
  public bool supportsANSI92IntermediateSQL() {
    return false;
  }

  /**
   * Is the ANSI92 full SQL grammar supported?
   *
   * @return true if so
   */
  public bool supportsANSI92FullSQL() {
    return false;
  }

  /**
   * Is the SQL Integrity Enhancement Facility supported?
   *
   * @return true if so
   */
  public bool supportsIntegrityEnhancementFacility() {
    return false;
  }

  /**
   * Is some form of outer join supported?
   *
   * @return true if so
   */
  public bool supportsOuterJoins() {
    return true;
  }

  /**
   * Are full nested outer joins supported?
   *
   * @return true if so
   */
  public bool supportsFullOuterJoins() {
    return false;
  }

  /**
   * Is there limited support for outer joins?  (This will be true
   * if supportFullOuterJoins is true.)
   *
   * @return true if so
   */
  public bool supportsLimitedOuterJoins() {
    return true;
  }

  /**
   * What's the database vendor's preferred term for "schema"?
   *
   * @return the vendor term
   */
  public string getSchemaTerm() {
    string SchemaTerm = "";
    return SchemaTerm;
  }

  /**
   * What's the database vendor's preferred term for "procedure"?
   *
   * @return the vendor term
   */
  public string getProcedureTerm() {
    string ProcedureTerm = "";
    return ProcedureTerm;
  }

  /**
   * What's the database vendor's preferred term for "catalog"?
   *
   * @return the vendor term
   */
  public string getCatalogTerm() {
    string CatalogTerm = "database";
    return CatalogTerm;
  }

  /**
   * Does a catalog appear at the start of a qualified table name?
   * (Otherwise it appears at the end)
   *
   * @return true if it appears at the start
   */
  public bool isCatalogAtStart() {
    return true;
  }

  /**
   * What's the separator between catalog and table name?
   *
   * @return the separator string
   */
  public string getCatalogSeparator() {
    string CatalogSeparator = ".";
    return CatalogSeparator;
  }

  /**
   * Can a schema name be used in a data manipulation statement?
   *
   * @return true if so
   */
  public bool supportsSchemasInDataManipulation() {
    return false;
  }

  /**
   * Can a schema name be used in a procedure call statement?
   *
   * @return true if so
   */
  public bool supportsSchemasInProcedureCalls() {
    return false;
  }

  /**
   * Can a schema name be used in a table definition statement?
   *
   * @return true if so
   */
  public bool supportsSchemasInTableDefinitions() {
    return false;
  }

  /**
   * Can a schema name be used in an index definition statement?
   *
   * @return true if so
   */
  public bool supportsSchemasInIndexDefinitions() {
    return false;
  }

  /**
   * Can a schema name be used in a privilege definition statement?
   *
   * @return true if so
   */
  public bool supportsSchemasInPrivilegeDefinitions() {
    return false;
  }

  /**
   * Can a catalog name be used in a data manipulation statement?
   *
   * @return true if so
   */
  public bool supportsCatalogsInDataManipulation() {
    // mysql older than 3.22 does not support this facility.
    if( conn.judgeServerVersion(3, 22, 0) >= 0 )
      return true;
    return false;
  }

  /**
   * Can a catalog name be used in a procedure call statement?
   *
   * @return true if so
   */
  public bool supportsCatalogsInProcedureCalls() {
    return false;
  }

  /**
   * Can a catalog name be used in a table definition statement?
   *
   * @return true if so
   */
  public bool supportsCatalogsInTableDefinitions() {
    return false;
  }

  /**
   * Can a catalog name be used in a index definition statement?
   *
   * @return true if so
   */
  public bool supportsCatalogsInIndexDefinitions() {
    return false;
  }

  /**
   * Can a catalog name be used in a privilege definition statement?
   *
   * @return true if so
   */
  public bool supportsCatalogsInPrivilegeDefinitions() {
    return false;
  }

  /**
   * Is positioned DELETE supported?
   *
   * @return true if so
   */
  public bool supportsPositionedDelete() {
    return false;
  }

  /**
   * Is positioned UPDATE supported?
   *
   * @return true if so
   */
  public bool supportsPositionedUpdate() {
    return false;
  }

  /**
   * Is SELECT for UPDATE supported?
   *
   * @return true if so
   */
  public bool supportsSelectForUpdate() {
    return false;
  }

  /**
   * Are stored procedure calls using the stored procedure escape
   * syntax supported?
   *
   * @return true if so
   */
  public bool supportsStoredProcedures() {
    return false;
  }

  /**
   * Are subqueries in comparison expressions supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsSubqueriesInComparisons() {
    return true;
  }

  /**
   * Are subqueries in exists expressions supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsSubqueriesInExists() {
    return false;
  }

  /**
   * Are subqueries in "in" statements supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsSubqueriesInIns() {
    return false;
  }

  /**
   * Are subqueries in quantified expressions supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsSubqueriesInQuantifieds() {
    return false;
  }

  /**
   * Are correlated subqueries supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsCorrelatedSubqueries() {
    return false;
  }

  /**
   * Is SQL UNION supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsUnion() {
    return false;
  }

  /**
   * Is SQL UNION ALL supported?
   *
   * A JDBC compliant driver always returns true.
   *
   * @return true if so
   */
  public bool supportsUnionAll() {
    return false;
  }

  /**
   * Can cursors remain open across commits?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public bool supportsOpenCursorsAcrossCommit() {
    return false;
  }

  /**
   * Can cursors remain open across rollbacks?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public bool supportsOpenCursorsAcrossRollback() {
    return false;
  }

  /**
   * Can statements remain open across commits?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public bool supportsOpenStatementsAcrossCommit() {
    return false;
  }

  /**
   * Can statements remain open across rollbacks?
   *
   * @return true if so
   * @see Connection#disableAutoClose
   */
  public bool supportsOpenStatementsAcrossRollback() {
    return false;
  }

  /**
   * How many hex characters can you have in an inline binary literal?
   *
   * @return max literal length
   */
  public int getMaxBinaryLiteralLength() {
    return 16777208;
  }

  /**
   * What's the max length for a character literal?
   *
   * @return max literal length
   */
  public int getMaxCharLiteralLength() {
    return 16777208;
  }

  /**
   * What's the limit on column name length?
   *
   * @return max literal length
   */
  public int getMaxColumnNameLength() {
    return 32;
  }

  /**
   * What's the maximum number of columns in a "GROUP BY" clause?
   *
   * @return max number of columns
   */
  public int getMaxColumnsInGroupBy() {
    return 16;
  }

  /**
   * What's the maximum number of columns allowed in an index?
   *
   * @return max columns
   */
  public int getMaxColumnsInIndex() {
    return 16;
  }

  /**
   * What's the maximum number of columns in an "ORDER BY" clause?
   *
   * @return max columns
   */
  public int getMaxColumnsInOrderBy() {
    return 16;
  }

  /**
   * What's the maximum number of columns in a "SELECT" list?
   *
   * @return max columns
   */
  public int getMaxColumnsInSelect() {
    return 256;
  }

  /**
   * What's maximum number of columns in a table?
   *
   * @return max columns
   */
  public int getMaxColumnsInTable() {
    return 512;
  }

  /**
   * How many active connections can we have at a time to this database?
   *
   * @return max number of active connections;
   *   a result of zero means that there is no limit or the limit is not known
   */
  public int getMaxConnections() {
    return 0;
  }

  /**
   * What's the maximum cursor name length?
   *
   * @return max cursor name length in bytes
   */
  public int getMaxCursorNameLength() {
    return 64;
  }

  /**
   * What's the maximum length of an index (in bytes)?
   *
   * @return max index length in bytes
   */
  public int getMaxIndexLength() {
    return 128;
  }

  /**
   * What's the maximum length allowed for a schema name?
   *
   * @return max name length in bytes
   */
  public int getMaxSchemaNameLength() {
    return 0;
  }

  /**
   * What's the maximum length of a procedure name?
   *
   * @return max name length in bytes
   */
  public int getMaxProcedureNameLength() {
    return 0;
  }

  /**
   * What's the maximum length of a catalog name?
   *
   * @return max name length in bytes
   */
  public int getMaxCatalogNameLength() {
    return 32;
  }

  /**
   * What's the maximum length of a single row?
   *
   * @return max row size in bytes
   */
  public int getMaxRowSize() {
    return Int32.MaxValue - 8; // Max buffer size - HEADER
  }

  /**
   * Did getMaxRowSize() include LONGVARCHAR and LONGVARBINARY
   * blobs?
   *
   * @return true if so
   */
  public bool doesMaxRowSizeIncludeBlobs() {
    return true;
  }

  /**
   * What's the maximum length of a SQL statement?
   *
   * @return max length in bytes
   */
  public int getMaxStatementLength() {
    return (256*256*256) + (256*256) + 256 - 3 - 1;
  }

  /**
   * How many active statements can we have open at one time to this
   * database?
   *
   * @return the maximum
    */
  public int getMaxStatements() {
    return 0;
  }

  /**
   * What's the maximum length of a table name?
   *
   * @return max name length in bytes
   */
  public int getMaxTableNameLength() {
    return 32;
  }

  /**
   * What's the maximum number of tables in a SELECT?
   *
   * @return the maximum
   */
  public int getMaxTablesInSelect() {
    return 256;
  }

  /**
   * What's the maximum length of a user name?
   *
   * @return max name length  in bytes
   */
  public int getMaxUserNameLength() {
    return 16;
  }

  /**
   * What's the database's default transaction isolation level?  The
   * values are defined in java.sql.Connection.
   *
   * @return the default isolation level
   * @see Connection
   */
  public int getDefaultTransactionIsolation() {
    return (int)net.sql.Connection_TRANSACTION.SERIALIZABLE;
  }

  /**
   * Are transactions supported? If not, commit is a noop and the
   * isolation level is TRANSACTION_NONE.
   *
   * @return true if transactions are supported
   */
  public bool supportsTransactions() {
    if( conn.judgeServerVersion(3, 23, 15) >= 0 )
      return true;
    return false;
  }

  /**
   * Does the database support the given transaction isolation level?
   *
   * @param level the values are defined in java.sql.Connection
   * @return true if so
   * @see Connection
   */
  public bool supportsTransactionIsolationLevel(int level) {
    bool retval = false;
    if(level == (int)net.sql.Connection_TRANSACTION.NONE             ||
       level == (int)net.sql.Connection_TRANSACTION.READ_COMMITTED   ||
       level == (int)net.sql.Connection_TRANSACTION.READ_UNCOMMITTED ||
       level == (int)net.sql.Connection_TRANSACTION.REPEATABLE_READ  ||
       level == (int)net.sql.Connection_TRANSACTION.SERIALIZABLE     )
    {
      retval = true;
    }
    return retval;
  }

  /**
   * Are both data definition and data manipulation statements
   * within a transaction supported?
   *
   * @return true if so
   */
  public bool supportsDataDefinitionAndDataManipulationTransactions()
                                                        {
    return false;
  }

  /**
   * Are only data manipulation statements within a transaction
   * supported?
   *
   * @return true if so
   */
  public bool supportsDataManipulationTransactionsOnly()
                                                        {
    return false;
  }

  /**
   * Does a data definition statement within a transaction force the
   * transaction to commit?
   *
   * @return true if so
   */
  public bool dataDefinitionCausesTransactionCommit() {
    return false;
  }

  /**
   * Is a data definition statement within a transaction ignored?
   *
   * @return true if so
   */
  public bool dataDefinitionIgnoredInTransactions() {
    return false;
  }

  /**
   * Get a description of stored procedures available in a
   * catalog.
   *
   * <P>Only procedure descriptions matching the schema and
   * procedure name criteria are returned.  They are ordered by
   * PROCEDURE_SCHEM, and PROCEDURE_NAME.
   *
   * <P>Each procedure description has the the following columns:
   *  <OL>
   *    <LI><B>PROCEDURE_CAT</B> string => procedure catalog (may be null)
   *    <LI><B>PROCEDURE_SCHEM</B> string => procedure schema (may be null)
   *    <LI><B>PROCEDURE_NAME</B> string => procedure name
   *  <LI> reserved for future use
   *  <LI> reserved for future use
   *  <LI> reserved for future use
   *    <LI><B>REMARKS</B> string => explanatory comment on the procedure
   *    <LI><B>PROCEDURE_TYPE</B> short => kind of procedure:
   *      <UL>
   *      <LI> procedureResultUnknown - May return a result
   *      <LI> procedureNoResult - Does not return a result
   *      <LI> procedureReturnsResult - Returns a result
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param procedureNamePattern a procedure name pattern
   * @return ResultSet each row is a procedure description
   * @see #getSearchStringEscape
   */
  public ResultSet getProcedures(string catalog, string schemaPattern,
                                 string procedureNamePattern) {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "PROCEDURE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PROCEDURE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PROCEDURE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "reserved", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "reserved", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "reserved", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "PROCEDURE_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Get a description of a catalog's stored procedure parameters
   * and result columns.
   *
   * <P>Only descriptions matching the schema, procedure and
   * parameter name criteria are returned.  They are ordered by
   * PROCEDURE_SCHEM and PROCEDURE_NAME. Within this, the return value,
   * if any, is first. Next are the parameter descriptions in call
   * order. The column descriptions follow in column number order.
   *
   * <P>Each row in the ResultSet is a parameter desription or
   * column description with the following fields:
   *  <OL>
   *    <LI><B>PROCEDURE_CAT</B> string => procedure catalog (may be null)
   *    <LI><B>PROCEDURE_SCHEM</B> string => procedure schema (may be null)
   *    <LI><B>PROCEDURE_NAME</B> string => procedure name
   *    <LI><B>COLUMN_NAME</B> string => column/parameter name
   *    <LI><B>COLUMN_TYPE</B> Short => kind of column/parameter:
   *      <UL>
   *      <LI> procedureColumnUnknown - nobody knows
   *      <LI> procedureColumnIn - IN parameter
   *      <LI> procedureColumnInOut - INOUT parameter
   *      <LI> procedureColumnOut - OUT parameter
   *      <LI> procedureColumnReturn - procedure return value
   *      <LI> procedureColumnResult - result column in ResultSet
   *      </UL>
   *  <LI><B>DATA_TYPE</B> short => SQL type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> string => SQL type name
   *    <LI><B>PRECISION</B> int => precision
   *    <LI><B>LENGTH</B> int => length in bytes of data
   *    <LI><B>SCALE</B> short => scale
   *    <LI><B>RADIX</B> short => radix
   *    <LI><B>NULLABLE</B> short => can it contain NULL?
   *      <UL>
   *      <LI> procedureNoNulls - does not allow NULL values
   *      <LI> procedureNullable - allows NULL values
   *      <LI> procedureNullableUnknown - nullability unknown
   *      </UL>
   *    <LI><B>REMARKS</B> string => comment describing parameter/column
   *  </OL>
   *
   * <P><B>Note:</B> Some databases may not return the column
   * descriptions for a procedure. Additional columns beyond
   * REMARKS can be defined by the database.
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param procedureNamePattern a procedure name pattern
   * @param columnNamePattern a column name pattern
   * @return ResultSet each row is a stored procedure parameter or
   *      column description
   * @see #getSearchStringEscape
   */
  public ResultSet getProcedureColumns(string catalog, string schemaPattern,
                        string procedureNamePattern, string columnNamePattern)
                                                        {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PROCEDURE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PROCEDURE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PROCEDURE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "COLUMN_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "PRECISION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "RADIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Get a description of tables available in a catalog.
   *
   * <P>Only table descriptions matching the catalog, schema, table
   * name and type criteria are returned.  They are ordered by
   * TABLE_TYPE, TABLE_SCHEM and TABLE_NAME.
   *
   * <P>Each table description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> string => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> string => table name
   *    <LI><B>TABLE_TYPE</B> string => table type.  Typical types are "TABLE",
   *                    "VIEW", "SYSTEM TABLE", "GLOBAL TEMPORARY",
   *                    "LOCAL TEMPORARY", "ALIAS", "SYNONYM".
   *    <LI><B>REMARKS</B> string => explanatory comment on the table
   *  </OL>
   *
   * <P><B>Note:</B> Some databases may not return information for
   * all tables.
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param tableNamePattern a table name pattern
   * @param types a list of table types to include; null returns all types
   * @return ResultSet each row is a table description
   * @see #getSearchStringEscape
   */
  public ResultSet getTables(string catalog, string schemaPattern,
                             string tableNamePattern, String[] types)
                                                        {
    gweMysqlColumn[] cols = new gweMysqlColumn[5];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "TABLE_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    string temp_catalog;
    string sqlstr = "SHOW TABLES";
    if( catalog != null && !catalog.Equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    sqlstr += " FROM " + temp_catalog;
    if( tableNamePattern != null && !tableNamePattern.Equals("") )
      sqlstr += " LIKE '" + trimQuote(tableNamePattern) + "'";
    sqlstr += ";";

    ArrayList rows = new ArrayList();
    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        string tablename = rs.getString(1);

        ArrayList rowdata = new ArrayList();
        rowdata.Add(temp_catalog);  // TABLE_CAT
        rowdata.Add("");
        if( add_quote == true )     // TABLE_NAME
          rowdata.Add("`" + tablename + "`");
        else
          rowdata.Add(tablename);
        rowdata.Add("TABLE");       // TABLE_TYPE
        rowdata.Add("");
        rows.Add(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get the schema names available in this database.  The results
   * are ordered by schema name.
   *
   * <P>The schema column is:
   *  <OL>
   *    <LI><B>TABLE_SCHEM</B> string => schema name
   *  </OL>
   *
   * @return ResultSet each row has a single string column that is a
   * schema name
   */
  public ResultSet getSchemas() {
    gweMysqlColumn[] cols = new gweMysqlColumn[1];
    cols[0] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Get the catalog names available in this database.  The results
   * are ordered by catalog name.
   *
   * <P>The catalog column is:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => catalog name
   *  </OL>
   *
   * @return ResultSet each row has a single string column that is a
   * catalog name
   */
  public ResultSet getCatalogs() {
    gweMysqlColumn[] cols = new gweMysqlColumn[1];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    string sqlstr = "SHOW DATABASES;";
    ArrayList rows = new ArrayList();
    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        string dbname = rs.getString(1);
        ArrayList rowdata = new ArrayList();
        rowdata.Add(dbname);  // TABLE_CAT
        rows.Add(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get the table types available in this database.  The results
   * are ordered by table type.
   *
   * <P>The table type is:
   *  <OL>
   *    <LI><B>TABLE_TYPE</B> string => table type.  Typical types are "TABLE",
   *                    "VIEW", "SYSTEM TABLE", "GLOBAL TEMPORARY",
   *                    "LOCAL TEMPORARY", "ALIAS", "SYNONYM".
   *  </OL>
   *
   * @return ResultSet each row has a single string column that is a
   * table type
   */
  public ResultSet getTableTypes() {
    gweMysqlColumn[] cols = new gweMysqlColumn[1];
    cols[0] = new gweMysqlColumn("", "TABLE_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    ArrayList rows = new ArrayList();
    ArrayList rowdata = new ArrayList();
    rowdata.Add("TABLE");  // TABLE_TYPE
    rows.Add(rowdata);

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get a description of table columns available in a catalog.
   *
   * <P>Only column descriptions matching the catalog, schema, table
   * and column name criteria are returned.  They are ordered by
   * TABLE_SCHEM, TABLE_NAME and ORDINAL_POSITION.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> string => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> string => table name
   *    <LI><B>COLUMN_NAME</B> string => column name
   *    <LI><B>DATA_TYPE</B> short => SQL type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> string => Data source dependent type name
   *    <LI><B>COLUMN_SIZE</B> int => column size.  For char or date
   *        types this is the maximum number of characters, for numeric or
   *        decimal types this is precision.
   *    <LI><B>BUFFER_LENGTH</B> is not used.
   *    <LI><B>DECIMAL_DIGITS</B> int => the number of fractional digits
   *    <LI><B>NUM_PREC_RADIX</B> int => Radix (typically either 10 or 2)
   *    <LI><B>NULLABLE</B> int => is NULL allowed?
   *      <UL>
   *      <LI> columnNoNulls - might not allow NULL values
   *      <LI> columnNullable - definitely allows NULL values
   *      <LI> columnNullableUnknown - nullability unknown
   *      </UL>
   *    <LI><B>REMARKS</B> string => comment describing column (may be null)
   *    <LI><B>COLUMN_DEF</B> string => default value (may be null)
   *    <LI><B>SQL_DATA_TYPE</B> int => unused
   *    <LI><B>SQL_DATETIME_SUB</B> int => unused
   *    <LI><B>CHAR_OCTET_LENGTH</B> int => for char types the
   *       maximum number of bytes in the column
   *    <LI><B>ORDINAL_POSITION</B> int => index of column in table
   *      (starting at 1)
   *    <LI><B>IS_NULLABLE</B> string => "NO" means column definitely
   *      does not allow NULL values; "YES" means the column might
   *      allow NULL values.  An empty string means nobody knows.
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param tableNamePattern a table name pattern
   * @param columnNamePattern a column name pattern
   * @return ResultSet each row is a column description
   * @see #getSearchStringEscape
   */
  public ResultSet getColumns(string catalog, string schemaPattern,
                           string tableNamePattern, string columnNamePattern)
                                                         {
    ResultSet tables = null;
    ArrayList tablenames = new ArrayList();
    try {
      tables = getTables(catalog, schemaPattern, tableNamePattern, null);
      while(tables.next())
      {
        string tablename = tables.getString("TABLE_NAME");
        tablenames.Add(tablename);
      }
    }
    finally {
      if( tables != null )
      {
        try { tables.close(); }
        catch(Exception e) { }
      }
      tables = null;
    }

    gweMysqlColumn[] cols = new gweMysqlColumn[18];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "COLUMN_SIZE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "BUFFER_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "DECIMAL_DIGITS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "NUM_PREC_RADIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "REMARKS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "COLUMN_DEF", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "SQL_DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[14]= new gweMysqlColumn("", "SQL_DATETIME_SUB", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[15]= new gweMysqlColumn("", "CHAR_OCTET_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[16]= new gweMysqlColumn("", "ORDINAL_POSITION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[17]= new gweMysqlColumn("", "IS_NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    ArrayList rows = new ArrayList();
    string temp_catalog;
    if( catalog != null && !catalog.Equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    for(int i = 0; i < tablenames.Count; i++)
    {
      string sqlstr = "SHOW COLUMNS";
      string tablename = (string)tablenames[i];
      sqlstr += " FROM " + temp_catalog + "." + tablename;
      if( columnNamePattern != null && !columnNamePattern.Equals("") )
        sqlstr += " LIKE '" + trimQuote(columnNamePattern) + "'";
      sqlstr += ";";

      Statement stmt = null;
      ResultSet rs = null;
      try {
        stmt = conn.createStatement();
        rs = stmt.executeQuery(sqlstr);
        int ordinal_position = 0;
        while(rs.next())
        {
          string mysql_field = rs.getString("Field");
          string mysql_type  = rs.getString("Type");
          int jdbc_col_type = getJDBCColumnType(mysql_type);
          string native_col_type = getNativeColumnType(mysql_type);
          int col_size = getColumnSize(mysql_type);
          int dig_size = getDeciamlDigits(mysql_type);
          string null_str = rs.getString("Null");
          bool allow_null = false;
          if( null_str != null && null_str.Equals("YES") )
            allow_null = true;
          string mysql_extra = "";
          try { mysql_extra = rs.getString("Extra"); }
          catch(Exception e) { mysql_extra = ""; }
          string mysql_default = "";
          try { mysql_default = rs.getString("Default"); }
          catch(Exception e) { mysql_default = ""; }
          ordinal_position++;

          ArrayList rowdata = new ArrayList();
          rowdata.Add(temp_catalog);                    // TABLE_CAT
          rowdata.Add("");
          if( add_quote == true )                       // TABLE_NAME
            rowdata.Add("`" + tablename + "`");
          else
            rowdata.Add(tablename);
          if( add_quote == true )                       // COLUMN_NAME
            rowdata.Add("`" + mysql_field + "`");
          else
            rowdata.Add(mysql_field);
          rowdata.Add((short)jdbc_col_type);            // DATA_TYPE
          rowdata.Add(native_col_type);                 // TYPE_NAME
          rowdata.Add((int)col_size);                   // COLUMN_SIZE
          rowdata.Add((int)-1);
          rowdata.Add((int)dig_size);                   // DECIMAL_DIGITS
          rowdata.Add((int)10);                         // NUM_PREC_RADIX
          if( allow_null == true )                      // NULLABLE
            rowdata.Add((int)DatabaseMetaData_column.Nullable);
          else
            rowdata.Add((int)DatabaseMetaData_column.NoNulls);
          rowdata.Add(mysql_extra);                     // REMARKS
          rowdata.Add(mysql_default);                   // COLUMN_DEF
          rowdata.Add((int)-1);
          rowdata.Add((int)-1);
          rowdata.Add((int)col_size);                   // CHAR_OCTET_LENGTH
          rowdata.Add((int)ordinal_position);           // ORDINAL_POSITION
          if( allow_null == true )                      // IS_NULLABLE
            rowdata.Add("YES");
          else
            rowdata.Add("NO");
          rows.Add(rowdata);
        }
      }
      finally {
        if( rs != null )
        {
          try { rs.close(); }
          catch(Exception ex) { }
        }
        rs = null;
        if( stmt != null )
        {
          try { stmt.close(); }
          catch(Exception ex) { }
        }
        stmt = null;
      }
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  // mysql_type = varchar(32) etc...
  private string getNativeColumnType(string mysql_type) {
    string native_col_type;
    int sep_idx = mysql_type.IndexOf('(');
    if( sep_idx != -1 )
      native_col_type = mysql_type.Substring(0, sep_idx-0);
    else
      native_col_type = mysql_type;
    return native_col_type.ToUpper();
  }

  // mysql_type = varchar(32) etc...
  private int getJDBCColumnType(string mysql_type) {
    string temp_type = getNativeColumnType(mysql_type);

    if( temp_type.Equals("TINYINT") )
        return net.sql.Types.TINYINT;
    else if( temp_type.Equals("SMALLINT") )
      return net.sql.Types.SMALLINT;
    else if( temp_type.Equals("MEDIUMINT") )
      return net.sql.Types.SMALLINT;
    else if( temp_type.Equals("INT") )
      return net.sql.Types.INTEGER;
    else if( temp_type.Equals("INTEGER") )
      return net.sql.Types.INTEGER;
    else if( temp_type.Equals("BIGINT") )
      return net.sql.Types.BIGINT;
    else if( temp_type.Equals("INT24") )
      return net.sql.Types.BIGINT;
    else if( temp_type.Equals("REAL") )
      return net.sql.Types.REAL;
    else if( temp_type.Equals("FLOAT") )
      return net.sql.Types.FLOAT;
    else if( temp_type.Equals("DECIMAL") )
      return net.sql.Types.DECIMAL;
    else if( temp_type.Equals("NUMERIC") )
      return net.sql.Types.NUMERIC;
    else if( temp_type.Equals("DOUBLE") )
      return net.sql.Types.DOUBLE;
    else if( temp_type.Equals("CHAR") )
      return net.sql.Types.CHAR;
    else if( temp_type.Equals("VARCHAR") )
      return net.sql.Types.VARCHAR;
    else if( temp_type.Equals("DATE") )
      return net.sql.Types.DATE;
    else if( temp_type.Equals("TIME") )
      return net.sql.Types.TIME;
    else if( temp_type.Equals("TIMESTAMP") )
      return net.sql.Types.TIMESTAMP;
    else if( temp_type.Equals("DATETIME") )
      return net.sql.Types.TIMESTAMP;
    else if( temp_type.Equals("TINYBLOB") )
      return net.sql.Types.VARBINARY;
    else if( temp_type.Equals("BLOB") )
      return net.sql.Types.LONGVARBINARY;
    else if( temp_type.Equals("MEDIUMBLOB") )
      return net.sql.Types.LONGVARBINARY;
    else if( temp_type.Equals("LONGBLOB") )
      return net.sql.Types.LONGVARBINARY;
    else if( temp_type.Equals("TINYTEXT") )
      return net.sql.Types.VARCHAR;
    else if( temp_type.Equals("TEXT") )
      return net.sql.Types.LONGVARCHAR;
    else if( temp_type.Equals("MEDIUMTEXT") )
      return net.sql.Types.LONGVARCHAR;
    else if( temp_type.Equals("LONGTEXT") ) // Thanks for INDEN-san
      return net.sql.Types.LONGVARCHAR;
    else if( temp_type.Equals("ENUM") )
      return net.sql.Types.CHAR;
    else if( temp_type.Equals("SET") )
      return net.sql.Types.CHAR;
    else if( temp_type.Equals("YEAR") ) // YEAR support!!
      return net.sql.Types.SMALLINT;
    throw new SQLException("Unknown data type.");
  }

  // mysql_type = NUMERIC(10,2) etc...
  private int getColumnSize(string mysql_type) {
    int start_idx = mysql_type.IndexOf('(');
    int sep_idx   = mysql_type.IndexOf(',');
    int last_idx  = mysql_type.LastIndexOf(')');
    string temp_type = getNativeColumnType(mysql_type);
    try {
      if( temp_type.Equals("ENUM") || temp_type.Equals("SET") )
      {
        if( start_idx == -1 )
          return 255;

        string value_str = mysql_type.Substring(start_idx+1, last_idx-(start_idx+1));
        StringTokenizer st = new StringTokenizer(value_str, ",", false);
        int retval = 0;
        while(st.hasMoreTokens())
        {
          string token = st.nextToken();
          int temp_size = token.Length - 2; // 'xxxxxx'
          if( temp_size > retval )
            retval = temp_size;
        }
        return retval;
      }
      else if( sep_idx != -1 ) // NUMERIC(10,2)
      {
        string precision_str = mysql_type.Substring(start_idx+1, sep_idx-(start_idx+1));
        return Int32.Parse(precision_str);
      }
      else if( start_idx != -1 ) // VARCHAR(32)
      {
        string size_str = mysql_type.Substring(start_idx+1, last_idx-(start_idx+1));
        return Int32.Parse(size_str);
      }
      else
      {
        int retval = 0;
        if( temp_type.Equals("TINYINT") )
          retval = 1;
        else if( temp_type.Equals("SMALLINT") )
          retval = 6;
        else if( temp_type.Equals("MEDIUMINT") )
          retval = 6;
        else if( temp_type.Equals("INT") )
          retval = 11;
        else if( temp_type.Equals("INTEGER") )
          retval = 11;
        else if( temp_type.Equals("BIGINT") )
          retval = 25;
        else if( temp_type.Equals("INT24") )
          retval = 25;
        else if( temp_type.Equals("REAL") )
          retval = 12;
        else if( temp_type.Equals("FLOAT") )
          retval = 12;
        else if( temp_type.Equals("DECIMAL") )
          retval = 12;
        else if( temp_type.Equals("NUMERIC") )
          retval = 12;
        else if( temp_type.Equals("DOUBLE") )
          retval = 22;
        else if( temp_type.Equals("CHAR") )
          retval = 1;
        else if( temp_type.Equals("VARCHAR") )
          retval = 255;
        else if( temp_type.Equals("DATE") )
          retval = 10;
        else if( temp_type.Equals("TIME") )
          retval = 8;
        else if( temp_type.Equals("TIMESTAMP") )
          retval = 19;
        else if( temp_type.Equals("DATETIME") )
          retval = 19;
        else if( temp_type.Equals("TINYBLOB") )
          retval = 255;
        else if( temp_type.Equals("BLOB") )
          retval = 65535;
        else if( temp_type.Equals("MEDIUMBLOB") )
          retval = 16277215;
        else if( temp_type.Equals("LONGBLOB") )
          retval = 2147483647;
        else if( temp_type.Equals("TINYTEXT") )
          retval = 255;
        else if( temp_type.Equals("TEXT") )
          retval = 65535;
        else if( temp_type.Equals("MEDIUMTEXT") )
          retval = 16277215;
        else if( temp_type.Equals("LONGTEXT") ) // Thanks for INDEN-san
          retval = 2147483647;
        else if( temp_type.Equals("YEAR") ) // YEAR support!!
          retval = 4;
        return retval;
      }
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  // mysql_type = NUMERIC(10,2) etc...
  private int getDeciamlDigits(string mysql_type) {
    int sep_idx = mysql_type.IndexOf(',');
    int last_idx = mysql_type.LastIndexOf(')');
    string temp_type = getNativeColumnType(mysql_type);
    try {
      if( temp_type.Equals("ENUM") || temp_type.Equals("SET") )
      {
        return 0;
      }
      if( sep_idx != -1 && last_idx != -1 ) // NUMERIC(10,2)
      {
         string fractional_str = mysql_type.Substring(sep_idx+1, last_idx-(sep_idx+1));
         return Int32.Parse(fractional_str);
      }
      return 0;
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  /**
   * Get a description of the access rights for a table's columns.
   *
   * <P>Only privileges matching the column name criteria are
   * returned.  They are ordered by COLUMN_NAME and PRIVILEGE.
   *
   * <P>Each privilige description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> string => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> string => table name
   *    <LI><B>COLUMN_NAME</B> string => column name
   *    <LI><B>GRANTOR</B> => grantor of access (may be null)
   *    <LI><B>GRANTEE</B> string => grantee of access
   *    <LI><B>PRIVILEGE</B> string => name of access (SELECT,
   *      INSERT, UPDATE, REFRENCES, ...)
   *    <LI><B>IS_GRANTABLE</B> string => "YES" if grantee is permitted
   *      to grant to others; "NO" if not; null if unknown
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @param columnNamePattern a column name pattern
   * @return ResultSet each row is a column privilege description
   * @see #getSearchStringEscape
   */
  public ResultSet getColumnPrivileges(string catalog, string schema,
                                       string table, string columnNamePattern)
                                                          {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "GRANTOR", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "GRANTEE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "PRIVILEGE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "IS_GRANTABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Get a description of the access rights for each table available
   * in a catalog.
   *
   * <P>Only privileges matching the schema and table name
   * criteria are returned.  They are ordered by TABLE_SCHEM,
   * TABLE_NAME, and PRIVILEGE.
   *
   * <P>Each privilige description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> string => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> string => table name
   *    <LI><B>COLUMN_NAME</B> string => column name
   *    <LI><B>GRANTOR</B> => grantor of access (may be null)
   *    <LI><B>GRANTEE</B> string => grantee of access
   *    <LI><B>PRIVILEGE</B> string => name of access (SELECT,
   *      INSERT, UPDATE, REFRENCES, ...)
   *    <LI><B>IS_GRANTABLE</B> string => "YES" if grantee is permitted
   *      to grant to others; "NO" if not; null if unknown
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param tableNamePattern a table name pattern
   * @return ResultSet each row is a table privilege description
   * @see #getSearchStringEscape
   */
  public ResultSet getTablePrivileges(string catalog, string schemaPattern,
                                 string tableNamePattern) {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "GRANTOR", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "GRANTEE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "PRIVILEGE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "IS_GRANTABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Get a description of a table's optimal set of columns that
   * uniquely identifies a row. They are ordered by SCOPE.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>SCOPE</B> short => actual scope of result
   *      <UL>
   *      <LI> bestRowTemporary - very temporary, while using row
   *      <LI> bestRowTransaction - valid for remainder of current transaction
   *      <LI> bestRowSession - valid for remainder of current session
   *      </UL>
   *    <LI><B>COLUMN_NAME</B> string => column name
   *    <LI><B>DATA_TYPE</B> short => SQL data type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> string => Data source dependent type name
   *    <LI><B>COLUMN_SIZE</B> int => precision
   *    <LI><B>BUFFER_LENGTH</B> int => not used
   *    <LI><B>DECIMAL_DIGITS</B> short  => scale
   *    <LI><B>PSEUDO_COLUMN</B> short => is this a pseudo column
   *      like an Oracle ROWID
   *      <UL>
   *      <LI> bestRowUnknown - may or may not be pseudo column
   *      <LI> bestRowNotPseudo - is NOT a pseudo column
   *      <LI> bestRowPseudo - is a pseudo column
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @param scope the scope of interest; use same values as SCOPE
   * @param nullable include columns that are nullable?
   * @return ResultSet each row is a column description
   */
  public ResultSet getBestRowIdentifier(string catalog, string schema,
                                     string table, int scope, bool nullable) {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "SCOPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "COLUMN_SIZE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "BUFFER_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "DECIMAL_DIGITS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "PSEUDO_COLUMN", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    ArrayList rows = new ArrayList();
    string temp_catalog;
    if( catalog != null && !catalog.Equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    string sqlstr = "SHOW COLUMNS FROM " + temp_catalog + "." + table + ";";

    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        string mysql_field = rs.getString("Field");
        string mysql_type  = rs.getString("Type");
        int jdbc_col_type = getJDBCColumnType(mysql_type);
        string native_col_type = getNativeColumnType(mysql_type);
        int col_size = getColumnSize(mysql_type);
        int dig_size = getDeciamlDigits(mysql_type);
        string null_str = rs.getString("Null");
        bool allow_null = false;
        if( null_str != null && null_str.Equals("YES") )
          allow_null = true;
        string mysql_key = rs.getString("Key");

        if( !mysql_key.Equals("PRI") )
          continue;
        if( nullable == false && allow_null == true )
          continue;

        ArrayList rowdata = new ArrayList();
        rowdata.Add((short)DatabaseMetaData_bestRow.Session);// SCOPE
        if( add_quote == true )                          // COLUMN_NAME
          rowdata.Add("`" + mysql_field + "`");
        else
          rowdata.Add(mysql_field);
        rowdata.Add((short)jdbc_col_type);               // DATA_TYPE
        rowdata.Add(native_col_type);                    // TYPE_NAME
        rowdata.Add((int)col_size);                      // COLUMN_SIZE
        rowdata.Add((int)-1);
        rowdata.Add((short)dig_size);                   // DECIMAL_DIGITS
        rowdata.Add((short)DatabaseMetaData_bestRow2.NotPseudo);
                                                         // PSEUDO_COLUMN
        rows.Add(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get a description of a table's columns that are automatically
   * updated when any value in a row is updated.  They are
   * unordered.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>SCOPE</B> short => is not used
   *    <LI><B>COLUMN_NAME</B> string => column name
   *    <LI><B>DATA_TYPE</B> short => SQL data type from java.sql.Types
   *    <LI><B>TYPE_NAME</B> string => Data source dependent type name
   *    <LI><B>COLUMN_SIZE</B> int => precision
   *    <LI><B>BUFFER_LENGTH</B> int => length of column value in bytes
   *    <LI><B>DECIMAL_DIGITS</B> short  => scale
   *    <LI><B>PSEUDO_COLUMN</B> short => is this a pseudo column
   *      like an Oracle ROWID
   *      <UL>
   *      <LI> versionColumnUnknown - may or may not be pseudo column
   *      <LI> versionColumnNotPseudo - is NOT a pseudo column
   *      <LI> versionColumnPseudo - is a pseudo column
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @return ResultSet each row is a column description
   */
  public ResultSet getVersionColumns(string catalog, string schema,
                                          string table) {
    gweMysqlColumn[] cols = new gweMysqlColumn[8];
    cols[0] = new gweMysqlColumn("", "SCOPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "COLUMN_SIZE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "BUFFER_LENGTH", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "DECIMAL_DIGITS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "PSEUDO_COLUMN", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Get a description of a table's primary key columns.  They
   * are ordered by COLUMN_NAME.
   *
   * <P>Each column description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> string => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> string => table name
   *    <LI><B>COLUMN_NAME</B> string => column name
   *    <LI><B>KEY_SEQ</B> short => sequence number within primary key
   *    <LI><B>PK_NAME</B> string => primary key name (may be null)
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name pattern; "" retrieves those
   * without a schema
   * @param table a table name
   * @return ResultSet each row is a primary key column description
   */
  public ResultSet getPrimaryKeys(string catalog, string schema,
                                  string table) {
    gweMysqlColumn[] cols = new gweMysqlColumn[6];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    ArrayList rows = new ArrayList();
    string temp_catalog;
    if( catalog != null && !catalog.Equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    string sqlstr = "SHOW KEYS FROM " + temp_catalog + "." + table + ";";

    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        string mysql_tbl = rs.getString("Table");
        string mysql_key = rs.getString("Key_name");
        string mysql_col = rs.getString("Column_name");
        string mysql_seq = rs.getString("Seq_in_index");

        if( !mysql_key.Equals("PRIMARY") )
          continue;

        ArrayList rowdata = new ArrayList();
        rowdata.Add(temp_catalog);             // TABLE_CAT
        rowdata.Add("");                       // TABLE_SCHEM
        if( add_quote == true )                // TABLE_NAME
          rowdata.Add("`" + mysql_tbl + "`");
        else
          rowdata.Add(mysql_tbl);
        if( add_quote == true )                // COLUMN_NAME
          rowdata.Add("`" + mysql_col + "`");
        else
          rowdata.Add(mysql_col);
        try {
          rowdata.Add(Int16.Parse(mysql_seq)); // KEY_SEQ
        }
        catch(Exception e) {
          throw new SQLException(e.Message);
        }
        rowdata.Add(mysql_col);                // PK_NAME
        rows.Add(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Get a description of the primary key columns that are
   * referenced by a table's foreign key columns (the primary keys
   * imported by a table).  They are ordered by PKTABLE_CAT,
   * PKTABLE_SCHEM, PKTABLE_NAME, and KEY_SEQ.
   *
   * <P>Each primary key column description has the following columns:
   *  <OL>
   *    <LI><B>PKTABLE_CAT</B> string => primary key table catalog
   *      being imported (may be null)
   *    <LI><B>PKTABLE_SCHEM</B> string => primary key table schema
   *      being imported (may be null)
   *    <LI><B>PKTABLE_NAME</B> string => primary key table name
   *      being imported
   *    <LI><B>PKCOLUMN_NAME</B> string => primary key column name
   *      being imported
   *    <LI><B>FKTABLE_CAT</B> string => foreign key table catalog (may be null)
   *    <LI><B>FKTABLE_SCHEM</B> string => foreign key table schema (may be null)
   *    <LI><B>FKTABLE_NAME</B> string => foreign key table name
   *    <LI><B>FKCOLUMN_NAME</B> string => foreign key column name
   *    <LI><B>KEY_SEQ</B> short => sequence number within foreign key
   *    <LI><B>UPDATE_RULE</B> short => What happens to
   *       foreign key when primary is updated:
   *      <UL>
   *      <LI> importedKeyCascade - change imported key to agree
   *               with primary key update
   *      <LI> importedKeyRestrict - do not allow update of primary
   *               key if it has been imported
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been updated
   *      </UL>
   *    <LI><B>DELETE_RULE</B> short => What happens to
   *      the foreign key when primary is deleted.
   *      <UL>
   *      <LI> importedKeyCascade - delete rows that import a deleted key
   *      <LI> importedKeyRestrict - do not allow delete of primary
   *               key if it has been imported
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been deleted
   *      </UL>
   *    <LI><B>FK_NAME</B> string => foreign key name (may be null)
   *    <LI><B>PK_NAME</B> string => primary key name (may be null)
   *    <LI><B>DEFERRABILITY</B> short => can the evaluation of foreign key
   *      constraints be deferred until commit
   *      <UL>
   *      <LI> importedKeyInitiallyDeferred - see SQL92 for definition
   *      <LI> importedKeyInitiallyImmediate - see SQL92 for definition
   *      <LI> importedKeyNotDeferrable - see SQL92 for definition
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a catalog
   * @param schema a schema name pattern; "" retrieves those
   * without a schema
   * @param table a table name
   * @return ResultSet each row is a primary key column description
   * @see #getExportedKeys
   */
  public ResultSet getImportedKeys(string catalog, string schema,
                                   string table) {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "PKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "FKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "FKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "FKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "FKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UPDATE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "DELETE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "FK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "DEFERRABILITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Gets a description of the foreign key columns that reference a
   * table's primary key columns (the foreign keys exported by a
   * table).  They are ordered by FKTABLE_CAT, FKTABLE_SCHEM,
   * FKTABLE_NAME, and KEY_SEQ.
   *
   * <P>Each foreign key column description has the following columns:
   *  <OL>
   *    <LI><B>PKTABLE_CAT</B> string => primary key table catalog (may be null)
   *    <LI><B>PKTABLE_SCHEM</B> string => primary key table schema (may be null)
   *    <LI><B>PKTABLE_NAME</B> string => primary key table name
   *    <LI><B>PKCOLUMN_NAME</B> string => primary key column name
   *    <LI><B>FKTABLE_CAT</B> string => foreign key table catalog (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_SCHEM</B> string => foreign key table schema (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_NAME</B> string => foreign key table name
   *      being exported
   *    <LI><B>FKCOLUMN_NAME</B> string => foreign key column name
   *      being exported
   *    <LI><B>KEY_SEQ</B> short => sequence number within foreign key
   *    <LI><B>UPDATE_RULE</B> short => What happens to
   *       foreign key when primary is updated:
   *      <UL>
   *      <LI> importedNoAction - do not allow update of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - change imported key to agree
   *               with primary key update
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been updated
   *      <LI> importedKeySetDefault - change imported key to default values
   *               if its primary key has been updated
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      </UL>
   *    <LI><B>DELETE_RULE</B> short => What happens to
   *      the foreign key when primary is deleted.
   *      <UL>
   *      <LI> importedKeyNoAction - do not allow delete of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - delete rows that import a deleted key
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been deleted
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      <LI> importedKeySetDefault - change imported key to default if
   *               its primary key has been deleted
   *      </UL>
   *    <LI><B>FK_NAME</B> string => foreign key name (may be null)
   *    <LI><B>PK_NAME</B> string => primary key name (may be null)
   *    <LI><B>DEFERRABILITY</B> short => can the evaluation of foreign key
   *      constraints be deferred until commit
   *      <UL>
   *      <LI> importedKeyInitiallyDeferred - see SQL92 for definition
   *      <LI> importedKeyInitiallyImmediate - see SQL92 for definition
   *      <LI> importedKeyNotDeferrable - see SQL92 for definition
   *      </UL>
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param schema a schema name; "" retrieves those
   * without a schema
   * @param table a table name
   * @return <code>ResultSet</code> - each row is a foreign key column description
   * @exception SQLException if a database access error occurs
   * @see #getImportedKeys
   */
  public ResultSet getExportedKeys(string catalog, string schema,
                                   string table) {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "PKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "FKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "FKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "FKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "FKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UPDATE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "DELETE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "FK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "DEFERRABILITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Gets a description of the foreign key columns in the foreign key
   * table that reference the primary key columns of the primary key
   * table (describe how one table imports another's key.) This
   * should normally return a single foreign key/primary key pair
   * (most tables only import a foreign key from a table once.)  They
   * are ordered by FKTABLE_CAT, FKTABLE_SCHEM, FKTABLE_NAME, and
   * KEY_SEQ.
   *
   * <P>Each foreign key column description has the following columns:
   *  <OL>
   *    <LI><B>PKTABLE_CAT</B> string => primary key table catalog (may be null)
   *    <LI><B>PKTABLE_SCHEM</B> string => primary key table schema (may be null)
   *    <LI><B>PKTABLE_NAME</B> string => primary key table name
   *    <LI><B>PKCOLUMN_NAME</B> string => primary key column name
   *    <LI><B>FKTABLE_CAT</B> string => foreign key table catalog (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_SCHEM</B> string => foreign key table schema (may be null)
   *      being exported (may be null)
   *    <LI><B>FKTABLE_NAME</B> string => foreign key table name
   *      being exported
   *    <LI><B>FKCOLUMN_NAME</B> string => foreign key column name
   *      being exported
   *    <LI><B>KEY_SEQ</B> short => sequence number within foreign key
   *    <LI><B>UPDATE_RULE</B> short => What happens to
   *       foreign key when primary is updated:
   *      <UL>
   *      <LI> importedNoAction - do not allow update of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - change imported key to agree
   *               with primary key update
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been updated
   *      <LI> importedKeySetDefault - change imported key to default values
   *               if its primary key has been updated
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      </UL>
   *    <LI><B>DELETE_RULE</B> short => What happens to
   *      the foreign key when primary is deleted.
   *      <UL>
   *      <LI> importedKeyNoAction - do not allow delete of primary
   *               key if it has been imported
   *      <LI> importedKeyCascade - delete rows that import a deleted key
   *      <LI> importedKeySetNull - change imported key to NULL if
   *               its primary key has been deleted
   *      <LI> importedKeyRestrict - same as importedKeyNoAction
   *                                 (for ODBC 2.x compatibility)
   *      <LI> importedKeySetDefault - change imported key to default if
   *               its primary key has been deleted
   *      </UL>
   *    <LI><B>FK_NAME</B> string => foreign key name (may be null)
   *    <LI><B>PK_NAME</B> string => primary key name (may be null)
   *    <LI><B>DEFERRABILITY</B> short => can the evaluation of foreign key
   *      constraints be deferred until commit
   *      <UL>
   *      <LI> importedKeyInitiallyDeferred - see SQL92 for definition
   *      <LI> importedKeyInitiallyImmediate - see SQL92 for definition
   *      <LI> importedKeyNotDeferrable - see SQL92 for definition
   *      </UL>
   *  </OL>
   *
   * @param primaryCatalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param primarySchema a schema name; "" retrieves those
   * without a schema
   * @param primaryTable the table name that exports the key
   * @param foreignCatalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param foreignSchema a schema name; "" retrieves those
   * without a schema
   * @param foreignTable the table name that imports the key
   * @return <code>ResultSet</code> - each row is a foreign key column description
   * @exception SQLException if a database access error occurs
   * @see #getImportedKeys
   */
  public ResultSet getCrossReference(string primaryCatalog,
                               string primarySchema, string primaryTable,
                               string foreignCatalog, string foreignSchema,
                                    string foreignTable) {
    gweMysqlColumn[] cols = new gweMysqlColumn[14];
    cols[0] = new gweMysqlColumn("", "PKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "PKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "PKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "FKTABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "FKTABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "FKTABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "FKCOLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "KEY_SEQ", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UPDATE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "DELETE_RULE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "FK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "PK_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "DEFERRABILITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);

    return new gweMysqlVectorResultSet(conn, cols, new ArrayList());
  }

  /**
   * Gets a description of all the standard SQL types supported by
   * this database. They are ordered by DATA_TYPE and then by how
   * closely the data type maps to the corresponding JDBC SQL type.
   *
   * <P>Each type description has the following columns:
   *  <OL>
   *    <LI><B>TYPE_NAME</B> string => Type name
   *    <LI><B>DATA_TYPE</B> short => SQL data type from java.sql.Types
   *    <LI><B>PRECISION</B> int => maximum precision
   *    <LI><B>LITERAL_PREFIX</B> string => prefix used to quote a literal
   *      (may be null)
   *    <LI><B>LITERAL_SUFFIX</B> string => suffix used to quote a literal
            (may be null)
   *    <LI><B>CREATE_PARAMS</B> string => parameters used in creating
   *      the type (may be null)
   *    <LI><B>NULLABLE</B> short => can you use NULL for this type?
   *      <UL>
   *      <LI> typeNoNulls - does not allow NULL values
   *      <LI> typeNullable - allows NULL values
   *      <LI> typeNullableUnknown - nullability unknown
   *      </UL>
   *    <LI><B>CASE_SENSITIVE</B> boolean=> is it case sensitive?
   *    <LI><B>SEARCHABLE</B> short => can you use "WHERE" based on this type:
   *      <UL>
   *      <LI> typePredNone - No support
   *      <LI> typePredChar - Only supported with WHERE .. LIKE
   *      <LI> typePredBasic - Supported except for WHERE .. LIKE
   *      <LI> typeSearchable - Supported for all WHERE ..
   *      </UL>
   *    <LI><B>UNSIGNED_ATTRIBUTE</B> => is it unsigned?
   *    <LI><B>FIXED_PREC_SCALE</B> => can it be a money value?
   *    <LI><B>AUTO_INCREMENT</B> => can it be used for an
   *      auto-increment value?
   *    <LI><B>LOCAL_TYPE_NAME</B> string => localized version of type name
   *      (may be null)
   *    <LI><B>MINIMUM_SCALE</B> short => minimum scale supported
   *    <LI><B>MAXIMUM_SCALE</B> short => maximum scale supported
   *    <LI><B>SQL_DATA_TYPE</B> int => unused
   *    <LI><B>SQL_DATETIME_SUB</B> int => unused
   *    <LI><B>NUM_PREC_RADIX</B> int => usually 2 or 10
   *  </OL>
   *
   * @return <code>ResultSet</code> - each row is an SQL type description
   * @exception SQLException if a database access error occurs
   */
  public ResultSet getTypeInfo() {
    gweMysqlColumn[] cols = new gweMysqlColumn[18];
    cols[0] = new gweMysqlColumn("", "TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "PRECISION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "LITERAL_PREFIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "LITERAL_SUFFIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "CREATE_PARAMS", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "NULLABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "CASE_SENSITIVE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "SEARCHABLE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "UNSIGNED_ATTRIBUTE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "FIXED_PREC_SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "AUTO_INCREMENT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "LOCAL_TYPE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[13]= new gweMysqlColumn("", "MINIMUM_SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[14]= new gweMysqlColumn("", "MAXIMUM_SCALE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[15]= new gweMysqlColumn("", "SQL_DATA_TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[16]= new gweMysqlColumn("", "SQL_DATETIME_SUB", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[17]= new gweMysqlColumn("", "NUM_PREC_RADIX", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);

    ArrayList rows = new ArrayList();
    ArrayList rowdata;

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TINY
    // JDBC  : Types.TINYINT
    rowdata = new ArrayList();
    rowdata.Add("TINYINT");                            // TYPE_NAME
    rowdata.Add((short)Types.TINYINT);                 // DATA_TYPE
    rowdata.Add((int)3);                               // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("True");                               // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("TINYINT");                            // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_SHORT
    // JDBC  : Types.SMALLINT
    rowdata = new ArrayList();
    rowdata.Add("SMALLINT");                           // TYPE_NAME
    rowdata.Add((short)Types.SMALLINT);                // DATA_TYPE
    rowdata.Add((int)5);                               // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("True");                               // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("SMALLINT");                           // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_INT24
    // JDBC  : Types.INTEGER
    rowdata = new ArrayList();
    rowdata.Add("MEDIUMINT");                          // TYPE_NAME
    rowdata.Add((short)Types.INTEGER);                 // DATA_TYPE
    rowdata.Add((int)7);                               // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("True");                               // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("MEDIUMINT");                          // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONG
    // JDBC  : Types.INTEGER
    rowdata = new ArrayList();
    rowdata.Add("INTEGER");                            // TYPE_NAME
    rowdata.Add((short)Types.INTEGER);                 // DATA_TYPE
    rowdata.Add((int)10);                              // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("True");                               // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("INTEGER");                            // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONGLONG
    // JDBC  : Types.BIGINT
    rowdata = new ArrayList();
    rowdata.Add("BIGINT");                             // TYPE_NAME
    rowdata.Add((short)Types.BIGINT);                  // DATA_TYPE
    rowdata.Add((int)19);                              // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("True");                               // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("BIGINT");                             // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_FLOAT
    // JDBC  : Types.FLOAT
    rowdata = new ArrayList();
    rowdata.Add("FLOAT");                              // TYPE_NAME
    rowdata.Add((short)Types.FLOAT);                   // DATA_TYPE
    rowdata.Add((int)10);                              // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M,D) ZEROFILL");                     // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("FLOAT");                              // LOCAL_TYPE_NAME
    rowdata.Add((short)-38);                           // MINIMUM_SCALE
    rowdata.Add((short)38);                            // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DOUBLE
    // JDBC  : Types.DOUBLE
    rowdata = new ArrayList();
    rowdata.Add("DOUBLE");                             // TYPE_NAME
    rowdata.Add((short)Types.DOUBLE);                  // DATA_TYPE
    rowdata.Add((int)17);                              // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M,D) ZEROFILL");                     // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("DOUBLE");                             // LOCAL_TYPE_NAME
    rowdata.Add((short)-308);                          // MINIMUM_SCALE
    rowdata.Add((short)308);                           // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DECIMAL
    // JDBC  : Types.DECIMAL
    rowdata = new ArrayList();
    rowdata.Add("DECIMAL");                            // TYPE_NAME
    rowdata.Add((short)Types.DECIMAL);                 // DATA_TYPE
    rowdata.Add((int)17);                              // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M,D) ZEROFILL");                     // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("True");                               // AUTO_INCREMENT
    rowdata.Add("DECIMAL");                            // LOCAL_TYPE_NAME
    rowdata.Add((short)-308);                          // MINIMUM_SCALE
    rowdata.Add((short)308);                           // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DATE
    // JDBC  : Types.DATE
    rowdata = new ArrayList();
    rowdata.Add("DATE");                               // TYPE_NAME
    rowdata.Add((short)Types.DATE);                    // DATA_TYPE
    rowdata.Add((int)0);                               // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("DATE");                               // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_DATETIME
    // JDBC  : Types.TIMESTAMP
    rowdata = new ArrayList();
    rowdata.Add("DATETIME");                           // TYPE_NAME
    rowdata.Add((short)Types.TIMESTAMP);               // DATA_TYPE
    rowdata.Add((int)0);                               // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("DATETIME");                           // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TIMESTAMP
    // JDBC  : Types.TIMESTAMP
    rowdata = new ArrayList();
    rowdata.Add("TIMESTAMP");                          // TYPE_NAME
    rowdata.Add((short)Types.TIMESTAMP);               // DATA_TYPE
    rowdata.Add((int)0);                               // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("(M)");                                // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("TIMESTAMP");                          // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TIME
    // JDBC  : Types.TIME
    rowdata = new ArrayList();
    rowdata.Add("TIME");                               // TYPE_NAME
    rowdata.Add((short)Types.TIME);                    // DATA_TYPE
    rowdata.Add((int)0);                               // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("TIME");                               // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_STRING
    // JDBC  : Types.CHAR
    rowdata = new ArrayList();
    rowdata.Add("CHAR");                               // TYPE_NAME
    rowdata.Add((short)Types.CHAR);                    // DATA_TYPE
    rowdata.Add((int)255);                             // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("(M) [BINARY]");                       // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("CHAR");                               // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_VAR_STRING
    // JDBC  : Types.VARCHAR
    rowdata = new ArrayList();
    rowdata.Add("VARCHAR");                            // TYPE_NAME
    rowdata.Add((short)Types.VARCHAR);                 // DATA_TYPE
    rowdata.Add((int)255);                             // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("(M) [BINARY]");                       // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("VARCHAR");                            // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TINY_BLOB
    // JDBC  : Types.VARBINARY
    rowdata = new ArrayList();
    rowdata.Add("TINYBLOB");                           // TYPE_NAME
    rowdata.Add((short)Types.VARBINARY);               // DATA_TYPE
    rowdata.Add((int)255);                             // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("TINYBLOB");                           // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_BLOB
    // JDBC  : Types.LONGVARBINARY
    rowdata = new ArrayList();
    rowdata.Add("BLOB");                               // TYPE_NAME
    rowdata.Add((short)Types.LONGVARBINARY);           // DATA_TYPE
    rowdata.Add((int)65535);                           // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("BLOB");                               // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_MEDIUM_BLOB
    // JDBC  : Types.LONGVARBINARY
    rowdata = new ArrayList();
    rowdata.Add("MEDIUMBLOB");                         // TYPE_NAME
    rowdata.Add((short)Types.LONGVARBINARY);           // DATA_TYPE
    rowdata.Add((int)16777215);                        // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("MEDIUMBLOB");                         // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONG_BLOB
    // JDBC  : Types.LONGVARBINARY
    rowdata = new ArrayList();
    rowdata.Add("LONGBLOB");                           // TYPE_NAME
    rowdata.Add((short)Types.LONGVARBINARY);           // DATA_TYPE
    rowdata.Add((int)0x7fffffff);         //4294967295 // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("LONGBLOB");                           // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_TINY_BLOB (TEXT)
    // JDBC  : Types.VARCHAR
    rowdata = new ArrayList();
    rowdata.Add("TINYTEXT");                           // TYPE_NAME
    rowdata.Add((short)Types.VARCHAR);                 // DATA_TYPE
    rowdata.Add((int)255);                             // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("TINYTEXT");                           // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_BLOB (TEXT)
    // JDBC  : Types.LONGVARCHAR
    rowdata = new ArrayList();
    rowdata.Add("TEXT");                               // TYPE_NAME
    rowdata.Add((short)Types.LONGVARCHAR);             // DATA_TYPE
    rowdata.Add((int)65535);                           // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("TEXT");                               // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_MEDIUM_BLOB (TEXT)
    // JDBC  : Types.LONGVARCHAR
    rowdata = new ArrayList();
    rowdata.Add("MEDIUMTEXT");                         // TYPE_NAME
    rowdata.Add((short)Types.LONGVARCHAR);             // DATA_TYPE
    rowdata.Add((int)16777215);                        // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("MEDIUMTEXT");                         // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

// Thanks for INDEN-san
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_LONG_BLOB (TEXT)
    // JDBC  : Types.LONGVARCHAR
    rowdata = new ArrayList();
    rowdata.Add("LONGTEXT");                           // TYPE_NAME
    rowdata.Add((short)Types.LONGVARCHAR);             // DATA_TYPE
    rowdata.Add((int)0x7fffffff);         //4294967295 // PRECISION
    rowdata.Add("'");                                  // LITERAL_PREFIX
    rowdata.Add("'");                                  // LITERAL_SUFFIX
    rowdata.Add("");                                   // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("True");                               // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("False");                              // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("LONGTEXT");                           // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

// YEAR support!!
    //////////////////////////////////////////////////////////////////////
    // mysql : FIELD_TYPE_YEAR
    // JDBC  : Types.SMALLINT
    rowdata = new ArrayList();
    rowdata.Add("YEAR");                               // TYPE_NAME
    rowdata.Add((short)Types.SMALLINT);                // DATA_TYPE
    rowdata.Add((int)4);                               // PRECISION
    rowdata.Add("");                                   // LITERAL_PREFIX
    rowdata.Add("");                                   // LITERAL_SUFFIX
    rowdata.Add("(M) UNSIGNED ZEROFILL");              // CREATE_PARAMS
    rowdata.Add((short)DatabaseMetaData_typeNull.Nullable); // NULLABLE
    rowdata.Add("False");                              // CASE_SENSITIVE
    rowdata.Add((short)DatabaseMetaData_type.Searchable); // SEARCHABLE
    rowdata.Add("True");                               // UNSIGNED_ATTRIBUTE
    rowdata.Add("False");                              // FIXED_PREC_SCALE
    rowdata.Add("False");                              // AUTO_INCREMENT
    rowdata.Add("YEAR");                               // LOCAL_TYPE_NAME
    rowdata.Add((short)0);                             // MINIMUM_SCALE
    rowdata.Add((short)0);                             // MAXIMUM_SCALE
    rowdata.Add((int)-1);
    rowdata.Add((int)-1);
    rowdata.Add((int)10);                              // NUM_PREC_RADIX
    rows.Add(rowdata);

    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  /**
   * Gets a description of a table's indices and statistics. They are
   * ordered by NON_UNIQUE, TYPE, INDEX_NAME, and ORDINAL_POSITION.
   *
   * <P>Each index column description has the following columns:
   *  <OL>
   *    <LI><B>TABLE_CAT</B> string => table catalog (may be null)
   *    <LI><B>TABLE_SCHEM</B> string => table schema (may be null)
   *    <LI><B>TABLE_NAME</B> string => table name
   *    <LI><B>NON_UNIQUE</B> => Can index values be non-unique?
   *      false when TYPE is tableIndexStatistic
   *    <LI><B>INDEX_QUALIFIER</B> string => index catalog (may be null);
   *      null when TYPE is tableIndexStatistic
   *    <LI><B>INDEX_NAME</B> string => index name; null when TYPE is
   *      tableIndexStatistic
   *    <LI><B>TYPE</B> short => index type:
   *      <UL>
   *      <LI> tableIndexStatistic - this identifies table statistics that are
   *           returned in conjuction with a table's index descriptions
   *      <LI> tableIndexClustered - this is a clustered index
   *      <LI> tableIndexHashed - this is a hashed index
   *      <LI> tableIndexOther - this is some other style of index
   *      </UL>
   *    <LI><B>ORDINAL_POSITION</B> short => column sequence number
   *      within index; zero when TYPE is tableIndexStatistic
   *    <LI><B>COLUMN_NAME</B> string => column name; null when TYPE is
   *      tableIndexStatistic
   *    <LI><B>ASC_OR_DESC</B> string => column sort sequence, "A" => ascending,
   *      "D" => descending, may be null if sort sequence is not supported;
   *      null when TYPE is tableIndexStatistic
   *    <LI><B>CARDINALITY</B> int => When TYPE is tableIndexStatistic, then
   *      this is the number of rows in the table; otherwise, it is the
   *      number of unique values in the index.
   *    <LI><B>PAGES</B> int => When TYPE is  tableIndexStatisic then
   *      this is the number of pages used for the table, otherwise it
   *      is the number of pages used for the current index.
   *    <LI><B>FILTER_CONDITION</B> string => Filter condition, if any.
   *      (may be null)
   *  </OL>
   *
   * @param catalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param schema a schema name; "" retrieves those without a schema
   * @param table a table name
   * @param unique when true, return only indices for unique values;
   *     when false, return indices regardless of whether unique or not
   * @param approximate when true, result is allowed to reflect approximate
   *     or out of data values; when false, results are requested to be
   *     accurate
   * @return <code>ResultSet</code> - each row is an index column description
   * @exception SQLException if a database access error occurs
   */
  public ResultSet getIndexInfo(string catalog, string schema, string table,
                                bool unique, bool approximate) {
    gweMysqlColumn[] cols = new gweMysqlColumn[13];
    cols[0] = new gweMysqlColumn("", "TABLE_CAT", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[1] = new gweMysqlColumn("", "TABLE_SCHEM", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[2] = new gweMysqlColumn("", "TABLE_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[3] = new gweMysqlColumn("", "NON_UNIQUE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[4] = new gweMysqlColumn("", "INDEX_QUALIFIER", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[5] = new gweMysqlColumn("", "INDEX_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[6] = new gweMysqlColumn("", "TYPE", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[7] = new gweMysqlColumn("", "ORDINAL_POSITION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_SHORT, (short)-1, -1);
    cols[8] = new gweMysqlColumn("", "COLUMN_NAME", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[9] = new gweMysqlColumn("", "ASC_OR_DESC", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);
    cols[10]= new gweMysqlColumn("", "CARDINALITY", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[11]= new gweMysqlColumn("", "PAGES", -1,
                      gweMysqlMetaResult.FIELD_TYPE_LONG, (short)-1, -1);
    cols[12]= new gweMysqlColumn("", "FILTER_CONDITION", -1,
                      gweMysqlMetaResult.FIELD_TYPE_VAR_STRING, (short)-1, -1);

    ArrayList rows = new ArrayList();
    string temp_catalog;
    if( catalog != null && !catalog.Equals("") )
      temp_catalog = catalog;
    else
      temp_catalog = conn.getCatalog();
    string sqlstr = "SHOW INDEX FROM " + temp_catalog + "." + table + ";";

    Statement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.createStatement();
      rs = stmt.executeQuery(sqlstr);
      while(rs.next())
      {
        string  tablename   = rs.getString("Table");
        bool    non_unique  = rs.getBoolean("Non_unique");
        string  indexname   = rs.getString("Key_name");
        short   seq_index   = rs.getShort("Seq_in_index");
        string  columnname  = rs.getString("Column_name");
        string  collation   = rs.getString("Collation");
        int     cardinality = rs.getInt("Cardinality");

        if( unique == true && non_unique == true )
          continue;

        ArrayList rowdata = new ArrayList();
        rowdata.Add(temp_catalog);             // TABLE_CAT
        rowdata.Add("");
        if( add_quote == true )                // TABLE_NAME
          rowdata.Add("`" + tablename + "`");
        else
          rowdata.Add(tablename);
        if( non_unique == true )               // NON_UNIQUE
          rowdata.Add("True");
        else
          rowdata.Add("False");
        rowdata.Add("");                       // INDEX_QUALIFIER
        rowdata.Add(indexname);                // INDEX_NAME
        rowdata.Add((short)DatabaseMetaData_table.IndexOther); // TYPE
        rowdata.Add((short)seq_index);         // ORDINAL_POSITION
        if( add_quote == true )                // COLUMN_NAME
          rowdata.Add("`" + columnname + "`");
        else
          rowdata.Add(columnname);
        rowdata.Add(collation);                // ASC_OR_DESC
        rowdata.Add((int)cardinality);         // CARDINALITY
        rowdata.Add((int)0);                   // PAGES
        rowdata.Add("");                       // FILTER_CONDITION
        rows.Add(rowdata);
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
      }
      rs = null;
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
      }
      stmt = null;
    }
    return new gweMysqlVectorResultSet(conn, cols, rows);
  }

  //--------------------------JDBC 2.0-----------------------------

  /**
   * Does the database support the given result set type?
   *
   * @param type defined in java.sql.ResultSet
   * @return true if so; false otherwise
   * @exception SQLException if a database access error occurs
   * @see Connection
   * @since 1.2
   */
  public bool supportsResultSetType(int type) {
    bool retval = false;
    if( type == (int)ResultSet_TYPE.FORWARD_ONLY       ||
        type == (int)ResultSet_TYPE.SCROLL_INSENSITIVE )
    {
      retval = true;
    }
    return retval;
  }

  /**
   * Does the database support the concurrency type in combination
   * with the given result set type?
   *
   * @param type defined in java.sql.ResultSet
   * @param concurrency type defined in java.sql.ResultSet
   * @return true if so; false otherwise
   * @exception SQLException if a database access error occurs
   * @see Connection
   * @since 1.2
   */
  public bool supportsResultSetConcurrency(int type, int concurrency) {
    bool retval = false;
    if( type == (int)ResultSet_TYPE.FORWARD_ONLY      &&
        concurrency == (int)ResultSet_CONCUR.READ_ONLY )
    {
      retval = true;
    }
    else if( type == (int)ResultSet_TYPE.SCROLL_INSENSITIVE &&
             concurrency == (int)ResultSet_CONCUR.READ_ONLY )
    {
      retval = true;
    }
    else if( (type == (int)ResultSet_TYPE.FORWARD_ONLY ||
              type == (int)ResultSet_TYPE.SCROLL_INSENSITIVE) &&
             concurrency == (int)ResultSet_CONCUR.UPDATABLE )
    {
      retval = true;
    }
    return retval;
  }

  /**
   * Determine whether a result set's own changes visible.
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are visible for the result set type
   * @exception SQLException if a database-access error occurs.
   */
  public bool ownUpdatesAreVisible(int type) {
    return true;
  }
  public bool ownDeletesAreVisible(int type) {
    return true;
  }
  public bool ownInsertsAreVisible(int type) {
    return false;
  }

  /**
   * Determine whether a result set's own changes visible.
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are visible for the result set type
   * @exception SQLException if a database-access error occurs.
   */
  public bool othersUpdatesAreVisible(int type) {
    return false;
  }
  public bool othersDeletesAreVisible(int type) {
    return false;
  }
  public bool othersInsertsAreVisible(int type) {
    return false;
  }

  /**
   * Determine whether or not a visible row update can be detected by
   * calling ResultSet.rowUpdated().
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are detected by the resultset type
   * @exception SQLException if a database-access error occurs.
   */
  public bool updatesAreDetected(int type) {
    return false;
  }

  /**
   * Determine whether or not a visible row delete can be detected by
   * calling ResultSet.rowDeleted().  If deletesAreDetected()
   * returns false, then deleted rows are removed from the result set.
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are detected by the resultset type
   * @exception SQLException if a database-access error occurs.
   */
  public bool deletesAreDetected(int type) {
    return true;
  }

  /**
   * Determine whether or not a visible row insert can be detected
   * by calling ResultSet.rowInserted().
   *
   * @param result set type, i.e. ResultSet.TYPE_XXX
   * @return true if changes are detected by the resultset type
   * @exception SQLException if a database-access error occurs.
   */
  public bool insertsAreDetected(int type) {
    return false;
  }

  /**
   * Indicates whether the driver supports batch updates.
   * @return true if the driver supports batch updates; false otherwise
   * @since 1.2
   */
  public bool supportsBatchUpdates() {
    return true;
  }

  /**
   * Get a description of the user-defined types defined in a particular
   * schema.  Schema specific UDTs may have type JAVA_OBJECT, STRUCT,
   * or DISTINCT.
   *
   * <P>Only types matching the catalog, schema, type name and type
   * criteria are returned.  They are ordered by DATA_TYPE, TYPE_SCHEM
   * and TYPE_NAME.  The type name parameter may be a fully qualified
   * name.  In this case, the catalog and schemaPattern parameters are
   * ignored.
   *
   * <P>Each type description has the following columns:
   *  <OL>
   *	<LI><B>TYPE_CAT</B> string => the type's catalog (may be null)
   *	<LI><B>TYPE_SCHEM</B> string => type's schema (may be null)
   *	<LI><B>TYPE_NAME</B> string => type name
   *  <LI><B>CLASS_NAME</B> string => Java class name
   *	<LI><B>DATA_TYPE</B> string => type value defined in java.sql.Types.
   *  One of JAVA_OBJECT, STRUCT, or DISTINCT
   *    <LI><B>REMARKS</B> string => explanatory comment on the type
   *  </OL>
   *
   * <P><B>Note:</B> If the driver does not support UDTs then an empty
   * result set is returned.
   *
   * @param catalog a catalog name; "" retrieves those without a
   * catalog; null means drop catalog name from the selection criteria
   * @param schemaPattern a schema name pattern; "" retrieves those
   * without a schema
   * @param typeNamePattern a type name pattern; may be a fully qualified
   * name
   * @param types a list of user-named types to include (JAVA_OBJECT,
   * STRUCT, or DISTINCT); null returns all types
   * @return ResultSet - each row is a type description
   * @exception SQLException if a database-access error occurs.
   */
  public ResultSet getUDTs(string catalog, string schemaPattern,
                   string typeNamePattern, int[] types) {
    throw new SQLException("mysql does not support getUDTs(string, String, String, int[]).");
  }

  /**
   * Return the connection that produced this metadata object.
   */
  public Connection getConnection() {
    return conn;
  }

  //--------------------------JDBC 3.0-----------------------------

  public bool supportsSavepoints() {
    bool retval = false;
    int judge40 = conn.judgeServerVersion(4, 0, 14);
    int judge41 = conn.judgeServerVersion(4, 1, 1);
    if( judge40 >= 0 || judge41 >= 0 )
    {
      retval = true;
    }
    return retval;
  }
  public bool supportsNamedParameters() {
    return false;
  }
  public bool supportsMultipleOpenResults() {
    return false;
  }
  public bool supportsGetGeneratedKeys() {
    return true;
  }
  public ResultSet getSuperTypes(string catalog, string schemaPattern,
                                 string typeNamePattern) {
    throw new SQLException("mysql does not support getSuperTypes(string, String, String).");
  }
  public ResultSet getSuperTables(string catalog, string schemaPattern,
                                  string tableNamePattern) {
    throw new SQLException("mysql does not support getSuperTables(string, String, String).");
  }
  public ResultSet getAttributes(string catalog, string schemaPattern,
                                 string typeNamePattern, string attributeNamePattern) {
    throw new SQLException("mysql does not support getAttributes(string, String, String, String).");
  }
  public bool supportsResultSetHoldability(int holdability) {
    return false;
  }
  public int getResultSetHoldability() {
    throw new SQLException("mysql does not support getResultSetHoldability().");
  }
  public int getDatabaseMajorVersion() {
    return conn.getServerMajorVersion();
  }
  public int getDatabaseMinorVersion() {
    return conn.getServerMinorVersion();
  }
  public int getJDBCMajorVersion() {
    return 3;
  }
  public int getJDBCMinorVersion() {
    return 0;
  }
  public int getSQLStateType() {
    throw new SQLException("mysql does not support getSQLStateType().");
  }
  public bool locatorsUpdateCopy() {
    throw new SQLException("mysql does not support locatorsUpdateCopy().");
  }
  public bool supportsStatementPooling() {
    return false;
  }
}

}
