//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Text;
using System.Collections;
using System.Globalization;
using net.sql;

public class gweMysqlResultSetBuffer : ResultSet {
  private gweMysqlResultSetCore org_rs;
  private ArrayList rows = new ArrayList();
  private int index = 0;
  private bool last_null = false;

  private bool onetable = true;
  private ArrayList pk_cols = null;
  private bool add_quote = false;
  private object[] param_obj_array = null;
  private int[] param_type_array = null;
  private bool insertmode = false;

  public gweMysqlResultSetBuffer(gweMysqlResultSetCore org_rs) {
    this.org_rs = org_rs;

    try {
      try {
        int maxRows = getStatement().getMaxRows();
        int rowcount = 0;
        int colnum = org_rs.getMetaData().getColumnCount();
        while( org_rs.next() )
        {
          ArrayList vecrow = new ArrayList(colnum);
          for(int i = 1; i <= colnum; i++)
          {
            object obj = org_rs.getObject(i);
            if( org_rs.wasNull() )
              obj = null;
            vecrow.Add(obj);
          }
          rows.Add(vecrow);
          rowcount++;
          if( maxRows != 0 && rowcount >= maxRows )
          {
            while( org_rs.next() )
              ;
            break;
          }
        }
      }
      finally {
        org_rs.close();
      }
    }
    catch(SQLException meta_e) {
      Console.WriteLine(meta_e.Message);
      Console.WriteLine(meta_e.StackTrace);
      Console.WriteLine("mysql can not get ResultSet and MetaData!! : " +
meta_e.Message);
    }

    try {
      if( org_rs.getConcurrency() == (int)ResultSet_CONCUR.UPDATABLE )
      {
        ResultSetMetaData meta = org_rs.getMetaData();
        int colnum = meta.getColumnCount();
        if( colnum >= 2 )
        {
          string tablename = meta.getTableName(1);
          for(int i = 2; i <= colnum; i++)
          {
            if( !tablename.Equals(meta.getTableName(i)) )
            {
              onetable = false;
              break;
            }
          }
        }
        gweMysqlConnection conn = (gweMysqlConnection)org_rs.getStatement().getConnection();
        if( colnum >= 1 )
        {
          ResultSet primarykeys = conn.getMetaData().getPrimaryKeys("", "", meta.getTableName(1));
          pk_cols = new ArrayList();
          try {
            while(primarykeys.next())
            {
              string colname = primarykeys.getString("COLUMN_NAME");
              for(int i = 1; i <= colnum; i++)
              {
                string temp_colname = meta.getColumnName(i);
                if( string.Compare(colname, temp_colname, true) == 0 )
                {
                  pk_cols.Add(temp_colname);
                  break;
                }
              }
            }
          }
          finally {
            if( primarykeys != null )
            {
              try { primarykeys.close(); }
              catch (Exception ex) { }
              primarykeys = null;
            }
          }
        }
        if( conn.judgeServerVersion(3, 23, 6) >= 0 )
          add_quote = true;
        param_obj_array  = new object[colnum];
        param_type_array = new int[colnum];
      }
    }
    catch(SQLException meta_e) {
      Console.WriteLine(meta_e.Message);
      Console.WriteLine(meta_e.StackTrace);
      Console.WriteLine("mysql can not get ResultSet and MetaData!! : " +
meta_e.Message);
    }
  }

  public bool next() {
    int row_count = rows.Count;
    if( index + 1 <= row_count )
    {
      index++;
      return true;
    }
    return false;
  }

  public void close() {
    org_rs = null;
    rows = null;
  }

  public bool wasNull() {
    return last_null;
  }

  public string getString(int columnIndex) {
    object retval = getObject(columnIndex);
    if( retval == null )
      return null;
    if( retval is byte[] )
    {
      byte[] blob = (byte[])retval;
      int blob_len = blob.Length;
      try {
        gweNativeUnicodeMapper mapperclass = org_rs.getNativeUnicodeMapper();
        if( mapperclass != null )
          return new string(mapperclass.Native2Unicode(blob));
        else
          return Encoding.GetEncoding(org_rs.getEncoding()).GetString(blob, 0, blob_len);
      }
      catch(Exception enc_e){
        throw new SQLException(enc_e.Message);
      }
    }
    return retval.ToString();
  }

  public bool getBoolean(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Boolean.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return false;
    }
  }

  public sbyte getByte(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return (sbyte)Int32.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }

  public short getShort(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return (short)Int32.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }

  public int getInt(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Int32.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }

  public long getLong(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Int64.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }

  public float getFloat(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Single.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0.0f;
    }
  }

  public double getDouble(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Double.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0.0;
    }
  }

  public System.Decimal getBigDecimal(int columnIndex, int scale) {
    return (System.Decimal)getObject(columnIndex);
  }

  public sbyte[] getSBytes(int columnIndex) {
    byte[] buff = (byte[])getObject(columnIndex);
    sbyte[] retval = new sbyte[buff.Length];
    for(int i = 0; i < buff.Length; i++)
      retval[i] = (sbyte)buff[i];
    return retval;
  }
  public byte[] getUBytes(int columnIndex) {
    return (byte[])getObject(columnIndex);
  }

  public System.DateTime getDate(int columnIndex) {
    return (System.DateTime)getObject(columnIndex);
  }

  public System.DateTime getTime(int columnIndex) {
    return (System.DateTime)getObject(columnIndex);
  }

  public System.DateTime getTimestamp(int columnIndex) {
    return (System.DateTime)getObject(columnIndex);
  }

  public System.IO.BinaryReader getAsciiStream(int columnIndex) {
    throw new SQLException("mysql does not support getAsciiStream(int).");
  }

  public System.IO.BinaryReader getUnicodeStream(int columnIndex) {
    throw new SQLException("mysql does not support getUnicodeStream(int).");
  }

  public System.IO.BinaryReader getBinaryStream(int columnIndex) {
    byte[] bin_buf = getUBytes(columnIndex);
    if( bin_buf == null )
      return null;
    return new System.IO.BinaryReader(new System.IO.MemoryStream(bin_buf));
  }

  public string getString(string columnName) {
    return getString(findColumn(columnName));
  }

  public bool getBoolean(string columnName) {
    return getBoolean(findColumn(columnName));
  }

  public sbyte getByte(string columnName) {
    return getByte(findColumn(columnName));
  }

  public short getShort(string columnName) {
    return getShort(findColumn(columnName));
  }

  public int getInt(string columnName) {
    return getInt(findColumn(columnName));
  }

  public long getLong(string columnName) {
    return getLong(findColumn(columnName));
  }

  public float getFloat(string columnName) {
    return getFloat(findColumn(columnName));
  }

  public double getDouble(string columnName) {
    return getDouble(findColumn(columnName));
  }

  public System.Decimal getBigDecimal(string columnName, int scale) {
    return getBigDecimal(findColumn(columnName));
  }

  public sbyte[] getSBytes(string columnName) {
    return getSBytes(findColumn(columnName));
  }
  public byte[] getUBytes(string columnName) {
    return getUBytes(findColumn(columnName));
  }

  public System.DateTime getDate(string columnName) {
    return getDate(findColumn(columnName));
  }

  public System.DateTime getTime(string columnName) {
    return getTime(findColumn(columnName));
  }

  public System.DateTime getTimestamp(string columnName) {
    return getTimestamp(findColumn(columnName));
  }

  public System.IO.BinaryReader getAsciiStream(string columnName) {
    return getAsciiStream(findColumn(columnName));
  }

  public System.IO.BinaryReader getUnicodeStream(string columnName) {
    return getUnicodeStream(findColumn(columnName));
  }

  public System.IO.BinaryReader getBinaryStream(string columnName) {
    return getBinaryStream(findColumn(columnName));
  }

  public SQLWarning getWarnings() {
    return null;
  }

  public void clearWarnings() {
  }

  public string getCursorName() {
    return "";
  }

  public ResultSetMetaData getMetaData() {
    return org_rs.getMetaData();
  }

  public object getObject(int columnIndex) {
    try {
      ArrayList rowdata = (ArrayList)rows[index-1];
      object data = rowdata[columnIndex-1];
      last_null = data == null ? true : false;
      return data;
    }
    catch(Exception e) {
      return null;
    }
  }

  public object getObject(string columnName) {
    return getObject(findColumn(columnName));
  }

  public int findColumn(string columnName) {
    return org_rs.findColumn(columnName);
  }

  public System.IO.StreamReader getCharacterStream(int columnIndex) {
    return null;
  }

  public System.IO.StreamReader getCharacterStream(string columnName) {
    return getCharacterStream(findColumn(columnName));
  }

  public System.Decimal getBigDecimal(int columnIndex) {
    return getBigDecimal(columnIndex, 10);
  }

  public System.Decimal getBigDecimal(string columnName) {
    return getBigDecimal(findColumn(columnName));
  }

  //---------------------------------------------------------------------
  // Traversal/Positioning
  //---------------------------------------------------------------------

  /**
   * Indicates whether the cursor is before the first row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is before the first row;
   * <code>false</code> if the cursor is at any other position or the
   * result set contains no rows
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool isBeforeFirst() {
    int row_count = rows.Count;
    if( row_count > 0 && index == 0 )
    {
      return true;
    }
    return false;
  }

  /**
   * Indicates whether the cursor is after the last row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is after the last row;
   * <code>false</code> if the cursor is at any other position or the
   * result set contains no rows
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool isAfterLast() {
    int row_count = rows.Count;
    if( row_count > 0 && index == row_count + 1 )
    {
      return true;
    }
    return false;
  }

  /**
   * Indicates whether the cursor is on the first row of
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is on the first row;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool isFirst() {
    int row_count = rows.Count;
    if( row_count > 0 && index == 1 )
    {
      return true;
    }
    return false;
  }

  /**
   * Indicates whether the cursor is on the last row of
   * this <code>ResultSet</code> object.
   * Note: Calling the method <code>isLast</code> may be expensive
   * because the JDBC driver
   * might need to fetch ahead one row in order to determine
   * whether the current row is the last row in the result set.
   *
   * @return <code>true</code> if the cursor is on the last row;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool isLast() {
    int row_count = rows.Count;
    if( row_count > 0 && index == row_count )
    {
      return true;
    }
    return false;
  }

  /**
   * Moves the cursor to the front of
   * this <code>ResultSet</code> object, just before the
   * first row. This method has no effect if the result set contains no rows.
   *
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public void beforeFirst() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = 0;
    }
  }

  /**
   * Moves the cursor to the end of
   * this <code>ResultSet</code> object, just after the
   * last row. This method has no effect if the result set contains no rows.
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public void afterLast() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = row_count + 1;
    }
  }

  /**
   * Moves the cursor to the first row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is on a valid row;
   * <code>false</code> if there are no rows in the result set
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool first() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = 1;
      return true;
    }
    return false;
  }

  /**
   * Moves the cursor to the last row in
   * this <code>ResultSet</code> object.
   *
   * @return <code>true</code> if the cursor is on a valid row;
   * <code>false</code> if there are no rows in the result set
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool last() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = row_count;
      return true;
    }
    return false;
  }

  /**
   * Retrieves the current row number.  The first row is number 1, the
   * second number 2, and so on.
   *
   * @return the current row number; <code>0</code> if there is no current row
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public int getRow() {
    return index;
  }

  /**
   * Moves the cursor to the given row number in
   * this <code>ResultSet</code> object.
   *
   * <p>If the row number is positive, the cursor moves to
   * the given row number with respect to the
   * beginning of the result set.  The first row is row 1, the second
   * is row 2, and so on.
   *
   * <p>If the given row number is negative, the cursor moves to
   * an absolute row position with respect to
   * the end of the result set.  For example, calling the method
   * <code>absolute(-1)</code> positions the
   * cursor on the last row; calling the method <code>absolute(-2)</code>
   * moves the cursor to the next-to-last row, and so on.
   *
   * <p>An attempt to position the cursor beyond the first/last row in
   * the result set leaves the cursor before the first row or after
   * the last row.
   *
   * <p><B>Note:</B> Calling <code>absolute(1)</code> is the same
   * as calling <code>first()</code>. Calling <code>absolute(-1)</code>
   * is the same as calling <code>last()</code>.
   *
   * @return <code>true</code> if the cursor is on the result set;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error
   * occurs, the row is <code>0</code>, or the result set type is
   * <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool absolute(int row) {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    if( row == 0 )
      return false;
    int row_count = rows.Count;
    if( row < 0 )
      row = row_count + 1 + row;
    index = row;
    if( index >= 1 && index <= row_count )
      return true;
    if( index <= 0 )
      beforeFirst();
    else
      afterLast();
    return false;
  }

  /**
   * Moves the cursor a relative number of rows, either positive or negative.
   * Attempting to move beyond the first/last row in the
   * result set positions the cursor before/after the
   * the first/last row. Calling <code>relative(0)</code> is valid, but does
   * not change the cursor position.
   *
   * <p>Note: Calling the method <code>relative(1)</code>
   * is different from calling the method <code>next()</code>
   * because is makes sense to call <code>next()</code> when there
   * is no current row,
   * for example, when the cursor is positioned before the first row
   * or after the last row of the result set.
   *
   * @return <code>true</code> if the cursor is on a row;
   * <code>false</code> otherwise
   * @exception SQLException if a database access error occurs,
   * there is no current row, or the result set type is
   * <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool relative(int row) {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    index += row;
    if( index >= 1 && index <= row_count )
      return true;
    if( index <= 0 )
      beforeFirst();
    else
      afterLast();
    return false;
  }

  /**
   * Moves the cursor to the previous row in this
   * <code>ResultSet</code> object.
   *
   * <p><B>Note:</B> Calling the method <code>previous()</code> is not the same as
   * calling the method <code>relative(-1)</code> because it
   * makes sense to call</code>previous()</code> when there is no current row.
   *
   * @return <code>true</code> if the cursor is on a valid row;
   * <code>false</code> if it is off the result set
   * @exception SQLException if a database access error
   * occurs or the result set type is <code>TYPE_FORWARD_ONLY</code>
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public bool previous() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    if( index - 1 >= 1 )
    {
      index--;
      return true;
    }
    return false;
  }

  public void setFetchDirection(int direction) {
    org_rs.setFetchDirection(direction);
  }

  public int getFetchDirection() {
    return org_rs.getFetchDirection();
  }

  public void setFetchSize(int rows) {
    org_rs.setFetchSize(rows);
  }

  public int getFetchSize() {
    return org_rs.getFetchSize();
  }

  /**
   * Returns the type of this <code>ResultSet</code> object.
   * The type is determined by the <code>Statement</code> object
   * that created the result set.
   *
   * @return <code>TYPE_FORWARD_ONLY</code>,
   * <code>TYPE_SCROLL_INSENSITIVE</code>,
   * or <code>TYPE_SCROLL_SENSITIVE</code>
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public int getType() {
    return org_rs.getType();
  }

  /**
   * Returns the concurrency mode of this <code>ResultSet</code> object.
   * The concurrency used is determined by the
   * <code>Statement</code> object that created the result set.
   *
   * @return the concurrency type, either <code>CONCUR_READ_ONLY</code>
   * or <code>CONCUR_UPDATABLE</code>
   * @exception SQLException if a database access error occurs
   * @since 1.2
   * @see <a href="package-summary.html#2.0 API">What Is in the JDBC
   *      2.0 API</a>
   */
  public int getConcurrency() {
    return org_rs.getConcurrency();
  }

  public bool rowUpdated() {
    throw new SQLException("mysql does not support rowUpdated().");
  }

  public bool rowInserted() {
    throw new SQLException("mysql does not support rowInserted().");
  }

  public bool rowDeleted() {
    try {
      ArrayList rowdata = (ArrayList)rows[index-1];
      return rowdata == null;
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  public void updateNull(int columnIndex) {
    this.updateObject(columnIndex, null);
  }

  public void updateBoolean(int columnIndex, bool x) {
    throw new SQLException("mysql does not support updateBoolean(int, bool).");
  }

  public void updateByte(int columnIndex, sbyte x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.TINYINT);
  }

  public void updateShort(int columnIndex, short x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.SMALLINT);
  }

  public void updateInt(int columnIndex, int x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.INTEGER);
  }

  public void updateLong(int columnIndex, long x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.BIGINT);
  }

  public void updateFloat(int columnIndex, float x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.FLOAT);
  }

  public void updateDouble(int columnIndex, double x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.DOUBLE);
  }

  public void updateBigDecimal(int columnIndex, System.Decimal x) {
    this.updateObjectEx(columnIndex, x, Types.DECIMAL);
  }

  public void updateString(int columnIndex, string x) {
    this.updateObjectEx(columnIndex, x, Types.VARCHAR);
  }

  public void updateBytes(int columnIndex, sbyte[] x) {
    this.updateObjectEx(columnIndex, x, Types.VARBINARY);
  }

  public void updateDate(int columnIndex, System.DateTime x) {
    this.updateObjectEx(columnIndex, x, Types.DATE);
  }

  public void updateTime(int columnIndex, System.DateTime x) {
    this.updateObjectEx(columnIndex, x, Types.TIME);
  }

  public void updateTimestamp(int columnIndex, System.DateTime x) {
    this.updateObjectEx(columnIndex, x, Types.TIMESTAMP);
  }

  public void updateAsciiStream(int columnIndex, System.IO.BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateAsciiStream(int, InputStream, int).");
  }

  public void updateBinaryStream(int columnIndex, System.IO.BinaryReader x, int length) {
    this.updateObjectEx(columnIndex, new gweMysqlPreparedObject(x, length), Types.LONGVARBINARY);
  }

  public void updateCharacterStream(int columnIndex, System.IO.StreamReader x, int length) {
    throw new SQLException("mysql does not support updateCharacterStream(int, Reader, int).");
  }

  public void updateObject(int columnIndex, object x, int scale) {
    this.updateObject(columnIndex, x);
  }

  public void updateObject(int columnIndex, object x) {
    ResultSetMetaData meta = org_rs.getMetaData();
    int targetSqlType = meta.getColumnType(columnIndex);
    updateObjectEx(columnIndex, x, targetSqlType);
  }

  private void updateObjectEx(int columnIndex, Object x, int targetSqlType) {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support updateObject.");

    if( param_obj_array  != null &&
        param_type_array != null )
    {
      if( columnIndex <= param_obj_array.Length  &&
          columnIndex <= param_type_array.Length )
      {
        param_obj_array[columnIndex-1] = x;
        if( (x is sbyte[]) && (targetSqlType != Types.VARBINARY) )
          targetSqlType = Types.VARBINARY;
        if( (x is BinaryReader) && (targetSqlType != Types.LONGVARBINARY) )
          targetSqlType = Types.LONGVARBINARY;
        param_type_array[columnIndex-1] = targetSqlType;
        return;
      }
    }
    throw new SQLException("gweMysqlResultSetBuffer Error : updateObject(int, Object).");
  }

  public void updateNull(string columnName) {
    updateNull(findColumn(columnName));
  }

  public void updateBoolean(string columnName, bool x) {
    throw new SQLException("mysql does not support updateBoolean(String, bool).");
  }

  public void updateByte(string columnName, sbyte x) {
    updateByte(findColumn(columnName), x);
  }

  public void updateShort(string columnName, short x) {
    updateShort(findColumn(columnName), x);
  }

  public void updateInt(string columnName, int x) {
    updateInt(findColumn(columnName), x);
  }

  public void updateLong(string columnName, long x) {
    updateLong(findColumn(columnName), x);
  }

  public void updateFloat(string columnName, float x) {
    updateFloat(findColumn(columnName), x);
  }

  public void updateDouble(string columnName, double x) {
    updateDouble(findColumn(columnName), x);
  }

  public void updateBigDecimal(string columnName, System.Decimal x) {
    updateBigDecimal(findColumn(columnName), x);
  }

  public void updateString(string columnName, string x) {
    updateString(findColumn(columnName), x);
  }

  public void updateBytes(string columnName, sbyte[] x) {
    updateBytes(findColumn(columnName), x);
  }

  public void updateDate(string columnName, System.DateTime x) {
    updateDate(findColumn(columnName), x);
  }

  public void updateTime(string columnName, System.DateTime x) {
    updateTime(findColumn(columnName), x);
  }

  public void updateTimestamp(string columnName, System.DateTime x) {
    updateTimestamp(findColumn(columnName), x);
  }

  public void updateAsciiStream(string columnName, System.IO.BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateAsciiStream(String, InputStream, int).");
  }

  public void updateBinaryStream(string columnName, System.IO.BinaryReader x, int length) {
    updateBinaryStream(findColumn(columnName), x, length);
  }

  public void updateCharacterStream(string columnName, System.IO.StreamReader reader, int length) {
    throw new SQLException("mysql does not support updateCharacterStream(String, Reader, int).");
  }

  public void updateObject(string columnName, object x, int scale) {
    updateObject(findColumn(columnName), x, scale);
  }

  public void updateObject(string columnName, object x) {
    updateObject(findColumn(columnName), x);
  }

  public void insertRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support insertRow.");
    if( !onetable )
      throw new SQLException("ResultSet generated from Multiple Tables does not support insertRow.");
    if( !insertmode )
      throw new SQLException("insertRow() cannot be called when the cursor is not on the insert row.");

    ResultSetMetaData meta = org_rs.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("INSERT INTO ");
    updatesql.append(meta.getTableName(1));
    updatesql.append("(");
    bool first = true;
    for(int i = 1; i <= param_obj_array.Length; i++)
    {
      if( param_type_array[i-1] != Types.NULL )
      {
        if( first )
          first = false;
        else
          updatesql.append(",");
        updatesql.append(meta.getColumnName(i));
      }
    }
    updatesql.append(")");
    updatesql.append(" VALUES(");
    first = true;
    for(int i = 1; i <= param_obj_array.Length; i++)
    {
      if( param_type_array[i-1] != Types.NULL )
      {
        if( first )
          first = false;
        else
          updatesql.append(",");
        updatesql.append("?");
      }
    }
    updatesql.append(");");

    Connection conn = org_rs.getStatement().getConnection();
    PreparedStatement stmt = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 1; i <= param_obj_array.Length; i++)
      {
        int targetSqlType = param_type_array[i-1];
        if( targetSqlType != Types.NULL )
        {
          stmt.setObject(param_index, param_obj_array[i-1], targetSqlType);
          param_index++;
        }
      }
      int count = stmt.executeUpdate();
      if( count == 1 )
      {
        ArrayList vecrow = new ArrayList();
        for(int i = 1; i <= param_obj_array.Length; i++)
        {
          if( param_type_array[i-1] != Types.NULL )
            vecrow.Add(param_obj_array[i-1]);
          else
            vecrow.Add(null);
        }
        rows.Add(vecrow);
      }
      else
      {
        throw new SQLException("Can not insert Row.");
      }
    }
    finally {
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
    }
  }

  public void updateRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support updateRow.");
    if( !onetable )
      throw new SQLException("ResultSet generated from Multiple Tables does not support updateRow.");
    if( insertmode )
      throw new SQLException("updateRow() cannot be called when the cursor is on the insert row.");

    ResultSetMetaData meta = org_rs.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("UPDATE ");
    updatesql.append(meta.getTableName(1));
    updatesql.append(" SET ");
    bool first = true;
    for(int i = 1; i <= param_obj_array.Length; i++)
    {
      if( param_type_array[i-1] != Types.NULL )
      {
        if( first )
          first = false;
        else
          updatesql.append(",");
        updatesql.append(meta.getColumnName(i));
        updatesql.append("=?");
      }
    }
    updatesql.append(" WHERE ");
    for(int i = 0; i < pk_cols.Count; i++)
    {
      if( i != 0 )
        updatesql.append(" AND ");
      string pkname = (string)pk_cols[i];
      updatesql.append(pkname);
      updatesql.append("=?");
    }
    updatesql.append(";");

    Connection conn = org_rs.getStatement().getConnection();
    PreparedStatement stmt = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 1; i <= param_obj_array.Length; i++)
      {
        int targetSqlType = param_type_array[i-1];
        if( targetSqlType != Types.NULL )
        {
          stmt.setObject(param_index, param_obj_array[i-1], targetSqlType);
          param_index++;
        }
      }
      for(int i = 0; i < pk_cols.Count; i++)
      {
        String pkname = (String)pk_cols[i];
        int targetSqlType = meta.getColumnType(findColumn(pkname));
        stmt.setObject(param_index, getObject(pkname), targetSqlType);
        param_index++;
      }
      int count = stmt.executeUpdate();
      if( count == 1 )
      {
        ArrayList rowdata = (ArrayList)rows[index-1];
        for(int i = 1; i <= param_obj_array.Length; i++)
        {
          if( param_type_array[i-1] != Types.NULL )
            rowdata[i-1] = param_obj_array[i-1];
        }
      }
    }
    finally {
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
    }
  }

  public void deleteRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support deleteRow.");
    if( !onetable )
      throw new SQLException("ResultSet generated from Multiple Tables does not support deleteRow.");
    if( insertmode )
      throw new SQLException("deleteRow() cannot be called when the cursor is on the insert row.");

    ResultSetMetaData meta = org_rs.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("DELETE FROM ");
    updatesql.append(meta.getTableName(1));
    updatesql.append(" WHERE ");
    for(int i = 0; i < pk_cols.Count; i++)
    {
      if( i != 0 )
        updatesql.append(" AND ");
      String pkname = (String)pk_cols[i];
      updatesql.append(pkname);
      updatesql.append("=?");
    }
    updatesql.append(";");

    Connection conn = org_rs.getStatement().getConnection();
    PreparedStatement stmt = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 0; i < pk_cols.Count; i++)
      {
        string pkname = (string)pk_cols[i];
        int targetSqlType = meta.getColumnType(findColumn(pkname));
        stmt.setObject(param_index, getObject(pkname), targetSqlType);
        param_index++;
      }
      int count = stmt.executeUpdate();
      if( count == 1 )
        rows[index-1] = null;
      else
        throw new SQLException("Can not delete Row.");
    }
    finally {
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
    }
  }

  public void refreshRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support refreshRow.");
    if( !onetable )
      throw new SQLException("ResultSet generated from Multiple Tables does not support refreshRow.");
    if( insertmode )
      throw new SQLException("refreshRow() cannot be called when the cursor is on the insert row.");

    if( rowDeleted() )
      return;

    ResultSetMetaData meta = org_rs.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("SELECT ");
    int colnum = meta.getColumnCount();
    for(int i = 1; i <= colnum; i++)
    {
      if( i != 1 )
        updatesql.append(",");
      updatesql.append(meta.getColumnName(i));
    }
    updatesql.append(" FROM ");
    updatesql.append(meta.getTableName(1));
    updatesql.append(" WHERE ");
    for(int i = 0; i < pk_cols.Count; i++)
    {
      if( i != 0 )
        updatesql.append(" AND ");
      string pkname = (string)pk_cols[i];
      updatesql.append(pkname);
      updatesql.append("=?");
    }
    updatesql.append(";");

    Connection conn = org_rs.getStatement().getConnection();
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 0; i < pk_cols.Count; i++)
      {
        string pkname = (string)pk_cols[i];
        int targetSqlType = meta.getColumnType(findColumn(pkname));
        stmt.setObject(param_index, getObject(pkname), targetSqlType);
        param_index++;
      }
      rs = stmt.executeQuery();
      if( rs.next() )
      {
        ArrayList rowdata = (ArrayList)rows[index-1];
        for(int i = 1; i <= colnum; i++)
          rowdata[i-1] = rs.getObject(i);
      }
      else
      {
        throw new SQLException("Can not refresh Row.");
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
        rs = null;
      }
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
      clearParameters();
    }
  }

  private void clearParameters() {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      int i;
      for(i = 0; i < param_obj_array.Length; i++)
        param_obj_array[i]  = null;
      for(i = 0; i < param_type_array.Length; i++)
        param_type_array[i] = Types.NULL;
    }
  }

  public void cancelRowUpdates() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support cancelRowUpdates.");

    clearParameters();
  }

  public void moveToInsertRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support moveToInsertRow.");
    if( !onetable )
      throw new SQLException("ResultSet generated from Multiple Tables does not support moveToInsertRow.");

    insertmode = true;
    clearParameters();
  }

  public void moveToCurrentRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY ResultSet does not support moveToCurrentRow.");

    insertmode = false;
    clearParameters();
  }

  public Statement getStatement() {
    return org_rs.getStatement();
  }

  public object getObject(int i, System.Collections.Hashtable map) {
    throw new SQLException("mysql does not support getObject(int, Map).");
  }

  public Ref getRef(int i) {
    throw new SQLException("mysql does not support getRef(int).");
  }

  public Blob getBlob(int i) {
    throw new SQLException("mysql does not support getBlob(int).");
  }

  public Clob getClob(int i) {
    throw new SQLException("mysql does not support getClob(int).");
  }

  public net.sql.Array getArray(int i) {
    throw new SQLException("mysql does not support getArray(int).");
  }

  public object getObject(string colName, System.Collections.Hashtable map) {
    throw new SQLException("mysql does not support getObject(String, Map).");
  }

  public Ref getRef(string colName) {
    throw new SQLException("mysql does not support getRef(String).");
  }

  public Blob getBlob(string colName) {
    throw new SQLException("mysql does not support getBlob(String).");
  }

  public Clob getClob(string colName) {
    throw new SQLException("mysql does not support getClob(String).");
  }

  public net.sql.Array getArray(string colName) {
    throw new SQLException("mysql does not support getArray(String).");
  }

  public System.DateTime getDate(int columnIndex, System.Globalization.Calendar cal) {
    return getDate(columnIndex);
  }

  public System.DateTime getDate(string columnName, System.Globalization.Calendar cal) {
    return getDate(findColumn(columnName));
  }

  public System.DateTime getTime(int columnIndex, System.Globalization.Calendar cal) {
    return getTime(columnIndex);
  }

  public System.DateTime getTime(string columnName, System.Globalization.Calendar cal) {
    return getTime(findColumn(columnName));
  }

  public System.DateTime getTimestamp(int columnIndex, System.Globalization.Calendar cal) {
    return getTimestamp(columnIndex);
  }

  public System.DateTime getTimestamp(string columnName, System.Globalization.Calendar cal) {
    return getTimestamp(findColumn(columnName));
  }

  //-------------------------- JDBC 3.0 ----------------------------------------

  /**
   * Retrieves the value of the designated column in the current row
   * of this <code>ResultSet</code> object as a <code>java.net.URL</code>
   * object in the Java programming language.
   *
   * @param columnIndex the index of the column 1 is the first, 2 is the second,...
   * @return the column value as a <code>java.net.URL</code> object;
   * if the value is SQL <code>NULL</code>,
   * the value returned is <code>null</code> in the Java programming language
   * @exception SQLException if a database access error occurs,
   *            or if a URL is malformed
   * @since 1.4
   */
  public string getURL(int columnIndex) {
    throw new SQLException("mysql does not support getURL(int).");
  }
  /**
   * Retrieves the value of the designated column in the current row
   * of this <code>ResultSet</code> object as a <code>java.net.URL</code>
   * object in the Java programming language.
   *
   * @param columnName the SQL name of the column
   * @return the column value as a <code>java.net.URL</code> object;
   * if the value is SQL <code>NULL</code>,
   * the value returned is <code>null</code> in the Java programming language
   * @exception SQLException if a database access error occurs
   *            or if a URL is malformed
   * @since 1.4
   */
  public string getURL(string columnName) {
    throw new SQLException("mysql does not support getURL(String).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Ref</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateRef(int columnIndex, Ref x) {
    throw new SQLException("mysql does not support updateRef(int, Ref).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Ref</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateRef(string columnName, Ref x) {
    throw new SQLException("mysql does not support updateRef(String, Ref).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Blob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateBlob(int columnIndex, Blob x) {
    throw new SQLException("mysql does not support updateBlob(int, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Blob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateBlob(string columnName, Blob x) {
    throw new SQLException("mysql does not support updateBlob(String, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Clob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateClob(int columnIndex, Clob x) {
    throw new SQLException("mysql does not support updateClob(int, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Clob</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateClob(string columnName, Clob x) {
    throw new SQLException("mysql does not support updateClob(String, Blob).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Array</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnIndex the first column is 1, the second is 2, ...
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateArray(int columnIndex, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(int, Array).");
  }
  /**
   * Updates the designated column with a <code>java.sql.Array</code> value.
   * The updater methods are used to update column values in the
   * current row or the insert row.  The updater methods do not
   * update the underlying database; instead the <code>updateRow</code> or
   * <code>insertRow</code> methods are called to update the database.
   *
   * @param columnName the name of the column
   * @param x the new column value
   * @exception SQLException if a database access error occurs
   * @since 1.4
   */
  public void updateArray(string columnName, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(String, Array).");
  }
}

}
