//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//e
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Collections;
using System.Text;
using net.sql;
using netx.sql;

[Serializable]
public class gweMysqlRowSet : RowSet {
  private const int TRANSACTION_UNKNOWN = -1;
  private object dataSourceObject = null;
  private Hashtable properties = null;
  private string url = null;
  private string username = null;
  private string password = null;
  private int transactionIsolation = gweMysqlRowSet.TRANSACTION_UNKNOWN;
  private int queryTimeout = 0; // seconds
  private int maxFieldSize = 0;
  private int maxRows = 0;
  private bool readOnly = true;
  private Hashtable typemap = null;

  private ArrayList listeners = new ArrayList();

  private bool escapeProcessing = false;
  private string   command = null;
  private object[] param_obj_array = null;
  private int[]    param_type_array = null;

  private ArrayList rows = null;
  private int index = 0;
  private bool last_null = false;
  private int resultSetType = (int)ResultSet_TYPE.FORWARD_ONLY;
  private int resultSetConcurrency = (int)ResultSet_CONCUR.READ_ONLY;
  private int fetchDirection = (int)ResultSet_FETCH.UNKNOWN;
  private int fetchSize = 0;
  private ResultSetMetaData rs_meta = null;
  private Hashtable columnName = null;
  private Hashtable columnFullName = null;

  private bool onetable = true;
  private ArrayList pk_cols = null;
  private bool add_quote = false;
  private object[] update_obj_array = null;
  private int[] update_type_array = null;
  private bool insertmode = false;
  [NonSerialized]
  private Connection updater = null;

  public object getDataSourceObject() {
    return dataSourceObject;
  }
  public void setDataSourceObject(object ds) {
    dataSourceObject = ds;
  }
  public Hashtable getProperties() {
    return properties;
  }
  public void setProperties(Hashtable prop) {
    properties = prop;
  }
  public string getUrl() {
    return url;
  }
  public void setUrl(String url) {
    this.url = url;
  }
  public string getUsername() {
    return username;
  }
  public void setUsername(String name) {
    username = name;
  }
  public string getPassword() {
    return password;
  }
  public void setPassword(String password) {
    this.password = password;
  }
  public int getTransactionIsolation() {
    return transactionIsolation;
  }
  public void setTransactionIsolation(int level) {
    transactionIsolation = level;
  }
  public int getQueryTimeout() {
    return queryTimeout;
  }
  public void setQueryTimeout(int seconds) {
    queryTimeout = seconds;
  }
  public void setConcurrency(int concurrency) {
    resultSetConcurrency = concurrency;
  }
  public void setType(int type) {
    resultSetType = type;
  }
  public int getMaxFieldSize() {
    return maxFieldSize;
  }
  public void setMaxFieldSize(int max) {
    maxFieldSize = max;
  }
  public int getMaxRows() {
    return maxRows;
  }
  public void setMaxRows(int max) {
    maxRows = max;
  }
  public bool isReadOnly() {
    return readOnly;
  }
  public void setReadOnly(bool value) {
    readOnly = value;
  }
  public Hashtable getTypeMap() {
    return typemap;
  }
  public void setTypeMap(Hashtable map) {
    typemap = map;
  }

  public void addRowSetListener(RowSetListener listener) {
    listeners.Add(listener);
  }
  public void removeRowSetListener(RowSetListener listener) {
    listeners.Remove(listener);
  }

  public bool getEscapeProcessing() {
    return escapeProcessing;
  }
  public void setEscapeProcessing(bool enable) {
    if( enable )
      throw new SQLException("mysql does not support setEscapeProcessing(true).");
    escapeProcessing = enable;
  }
  public string getCommand() {
    return command;
  }
  public void setCommand(String cmd) {
    command = cmd;

    int param_count = 0;
    StringTokenizer t_str = new StringTokenizer(command, "?", true);
    while(t_str.hasMoreTokens())
    {
      string token = t_str.nextToken();
      if( token.Equals("?") )
        param_count++;
    }
    param_obj_array  = new object[param_count];
    param_type_array = new int[param_count];
    int i;
    for(i = 0; i < param_obj_array.Length; i++)
      param_obj_array[i]  = null;
    for(i = 0; i < param_type_array.Length; i++)
      param_type_array[i] = Types.NULL;
  }

  public void clearParameters() {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      int i;
      for(i = 0; i < param_obj_array.Length; i++)
        param_obj_array[i]  = null;
      for(i = 0; i < param_type_array.Length; i++)
        param_type_array[i] = Types.NULL;
    }
    else
    {
      throw new SQLException("mysql error : clearParameters(), no parameter.");
    }
  }

  private Connection getConnection() {
    if( dataSourceObject != null )
    {
      try {
        //Context ctx = null;
        //if( properties != null )
        //  ctx = new InitialContext(properties);
        //else
        //  ctx = new InitialContext();
        //Object ref = ctx.lookup(dataSourceName);
        //if( ref == null )
        //  throw new SQLException("Invalid DataSource Object!! : " + "null");
        object ref_ = dataSourceObject;
        if( ref_ is DataSource )
        {
          DataSource ds = (DataSource)ref_;
          if( username != null && password != null )
            return ds.getConnection(username, password);
          else
            return ds.getConnection();
        }
        else if( ref_ is ConnectionPoolDataSource )
        {
          ConnectionPoolDataSource cpds = (ConnectionPoolDataSource)ref_;
          if( username != null && password != null )
            return cpds.getPooledConnection(username, password).getConnection();
          else
            return cpds.getPooledConnection().getConnection();
        }
        //else if( ref instanceof XADataSource )
        //{
        //  XADataSource xads = (XADataSource)ref;
        //  if( username != null && password != null )
        //    return xads.getXAConnection(username, password).getConnection();
        //  else
        //    return xads.getXAConnection().getConnection();
        //}
        else
        {
          throw new SQLException("Invalid DataSource Object!! : " + ref_.ToString());
        }
      }
      catch(Exception ex) {
        throw new SQLException(ex.Message);
      }
    }

    if( url != null )
    {
      if( username != null && password != null )
      {
        return DriverManager.getConnection(url, username, password);
      }
      else
      {
        if( properties != null )
          return DriverManager.getConnection(url, properties);
        else
          return DriverManager.getConnection(url);
      }
    }

    return null;
  }

  private void prepareSQLStatement(PreparedStatement stmt) {
    for(int i = 1; i <= param_obj_array.Length; i++)
    {
      stmt.setObject(i, param_obj_array[i-1], param_type_array[i-1]);
    }
  }

  private void buildResultSet(ResultSet rs) {
    resultSetType        = rs.getType();
    resultSetConcurrency = rs.getConcurrency();

    rs_meta = rs.getMetaData();
    int colnum = rs_meta.getColumnCount();
    columnName = new Hashtable();
    columnFullName = new Hashtable();
    for(int i = 1; i <= colnum; i++)
    {
      string colname = rs_meta.getColumnName(i);
      string colfull = rs_meta.getTableName(i) + "."+ colname;
      object colno = (int)(i-1);
      columnName.Add(colname, colno);
      columnFullName.Add(colfull, colno);
    }

    int rowcount = 0;
    rows = new ArrayList();
    while( rs.next() )
    {
      ArrayList vecrow = new ArrayList(colnum);
      for(int i = 1; i <= colnum; i++)
      {
        object obj = rs.getObject(i);
        if( rs.wasNull() )
          obj = null;
        vecrow.Add(obj);
      }
      rows.Add(vecrow);
      rowcount++;
      if( maxRows != 0 && rowcount >= maxRows )
        break;
    }
  }

  private void initialUpdaters(ResultSet rs) {
    if( rs.getConcurrency() == (int)ResultSet_CONCUR.UPDATABLE )
    {
      ResultSetMetaData meta = rs.getMetaData();
      int colnum = meta.getColumnCount();
      if( colnum >= 2 )
      {
        string tablename = meta.getTableName(1);
        for(int i = 2; i <= colnum; i++)
        {
          if( !tablename.Equals(meta.getTableName(i)) )
          {
            onetable = false;
            break;
          }
        }
      }
      Connection conn = rs.getStatement().getConnection();
      if( colnum >= 1 )
      {
        ResultSet primarykeys = conn.getMetaData().getPrimaryKeys("", "", meta.getTableName(1));
        pk_cols = new ArrayList();
        try {
          while(primarykeys.next())
          {
            string colname = primarykeys.getString("COLUMN_NAME");
            for(int i = 1; i <= colnum; i++)
            {
              string temp_colname = meta.getColumnName(i);
              if( string.Compare(colname, temp_colname, true) == 0 )
              {
                pk_cols.Add(temp_colname);
                break;
              }
            }
          }
        }
        finally {
          if( primarykeys != null )
          {
            try { primarykeys.close(); }
            catch (Exception ex) { }
            primarykeys = null;
          }
        }
      }
      if( conn is gweMysqlConnection )
      {
        if( ((gweMysqlConnection)conn).judgeServerVersion(3, 23, 6) >= 0 )
          add_quote = true;
      }
      update_obj_array  = new object[colnum];
      update_type_array = new int[colnum];
    }
  }

  public void execute() {
    Connection conn = null;
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try {
      conn = getConnection();
      if( transactionIsolation != gweMysqlRowSet.TRANSACTION_UNKNOWN )
        conn.setTransactionIsolation(transactionIsolation);
      conn.setReadOnly(readOnly);
      conn.setTypeMap(typemap);

      stmt = conn.prepareStatement(command, resultSetType, resultSetConcurrency);
//    stmt.setEscapeProcessing(escapeProcessing);
      stmt.setFetchDirection(fetchDirection);
      stmt.setFetchSize(fetchSize);
      stmt.setMaxFieldSize(maxFieldSize);
      stmt.setMaxRows(maxRows);
      stmt.setQueryTimeout(queryTimeout);
      prepareSQLStatement(stmt);
      rs = stmt.executeQuery();

      buildResultSet(rs);
      initialUpdaters(rs);
    }
    finally {
      if( rs != null )
      {
        try {
          rs.close();
        }
        catch(Exception ex){}
      }
      if( stmt != null )
      {
        try {
          stmt.close();
        }
        catch(Exception ex){}
      }
      if( conn != null )
      {
        try {
          conn.close();
        }
        catch(Exception ex){}
      }
    }
  }

  public void setNull(int parameterIndex, int sqlType) {
    setObject(parameterIndex, null, sqlType);
  }
  public void setBoolean(int parameterIndex, bool x) {
    throw new SQLException("mysql does not support setBoolean(int, boolean).");
  }
  public void setByte(int parameterIndex, sbyte x) {
    object obj = x;
    setObject(parameterIndex, obj, Types.TINYINT);
  }
  public void setShort(int parameterIndex, short x) {
    object obj = x;
    setObject(parameterIndex, obj, Types.SMALLINT);
  }
  public void setInt(int parameterIndex, int x) {
    object obj = x;
    setObject(parameterIndex, obj, Types.INTEGER);
  }
  public void setLong(int parameterIndex, long x) {
    object obj = x;
    setObject(parameterIndex, obj, Types.BIGINT);
  }
  public void setFloat(int parameterIndex, float x) {
    object obj = x;
    setObject(parameterIndex, obj, Types.FLOAT);
  }
  public void setDouble(int parameterIndex, double x) {
    object obj = x;
    setObject(parameterIndex, obj, Types.DOUBLE);
  }
  public void setBigDecimal(int parameterIndex, System.Decimal x) {
    setObject(parameterIndex, x, Types.DECIMAL);
  }
  public void setString(int parameterIndex, string x) {
    setObject(parameterIndex, x, Types.VARCHAR);
  }
  public void setBytes(int parameterIndex, sbyte[] x) {
    setObject(parameterIndex, x, Types.VARBINARY);
  }
  public void setDate(int parameterIndex, System.DateTime x) {
    // ex. '1999-02-14'
    setObject(parameterIndex, x, Types.DATE);
  }
  public void setTime(int parameterIndex, System.DateTime x) {
    // ex. '12:34:56'
    setObject(parameterIndex, x, Types.TIME);
  }
  public void setTimestamp(int parameterIndex, System.DateTime x) {
    // ex. '1999-02-14 12:34:56'
    setObject(parameterIndex, x, Types.TIMESTAMP);
  }
  public void setAsciiStream(int parameterIndex, BinaryReader x, int length) {
    throw new SQLException("mysql does not support setAsciiStream(int, InputStream, int).");
  }
  public void setUnicodeStream(int parameterIndex, BinaryReader x, int length) {
    throw new SQLException("mysql does not support setUnicodeStream(int, InputStream, int).");
  }
  public void setBinaryStream(int parameterIndex, BinaryReader x, int length) {
    setObject(parameterIndex, new gweMysqlPreparedObject(x, length), Types.LONGVARBINARY);
  }
  public void setObject(int parameterIndex, object x, int targetSqlType, int scale) {
    setObject(parameterIndex, x, targetSqlType);
  }
  public void setObject(int parameterIndex, object x, int targetSqlType) {
    if( param_obj_array  != null &&
        param_type_array != null )
    {
      if( parameterIndex <= param_obj_array.Length  &&
          parameterIndex <= param_type_array.Length )
      {
        param_obj_array[parameterIndex-1]  = x;
        if( (x is sbyte[]) && (targetSqlType != Types.VARBINARY) )
          targetSqlType = Types.VARBINARY;
        if( (x is BinaryReader) && (targetSqlType != Types.LONGVARBINARY) )
          targetSqlType = Types.LONGVARBINARY;
        param_type_array[parameterIndex-1] = targetSqlType;
        return;
      }
    }
    throw new SQLException("gweMysqlPreparedStatement Error : setObject(int, Object, int).");
  }
  public void setObject(int parameterIndex, object x) {
    throw new SQLException("mysql does not support setObject(int, Object).");
  }

  // for JDBC2.0 java.sql.ResultSet
  public void setCharacterStream(int parameterIndex, StreamReader reader, int length) {
    throw new SQLException("mysql does not support setCharacterStream(int, Reader, int).");
  }
  public void setRef(int i, Ref x) {
    throw new SQLException("mysql does not support setRef(int, Ref).");
  }
  public void setBlob(int i, Blob x) {
    setBinaryStream(i, x.getBinaryStream(), (int)x.length());
  }
  public void setClob(int i, Clob x) {
    throw new SQLException("mysql does not support setClob(int, Clob).");
  }
  public void setArray(int i, net.sql.Array x) {
    throw new SQLException("mysql does not support setArray(int, Array).");
  }
  public void setDate(int parameterIndex, System.DateTime x, System.Globalization.Calendar cal) {
    // ex. '1999-02-14'
    setObject(parameterIndex, x, Types.DATE);
  }
  public void setTime(int parameterIndex, System.DateTime x, System.Globalization.Calendar cal) {
    // ex. '12:34:56'
    setObject(parameterIndex, x, Types.TIME);
  }
  public void setTimestamp(int parameterIndex, System.DateTime x, System.Globalization.Calendar cal) {
    // ex. '1999-02-14 12:34:56'
    setObject(parameterIndex, x, Types.TIMESTAMP);
  }
  public void setNull(int paramIndex, int sqlType, string typeName) {
    throw new SQLException("mysql does not support setNull(int, int, String).");
  }

  // for JDBC2.0 java.sql.ResultSet
  public bool next() {
    int row_count = rows.Count;
    if( index + 1 <= row_count )
    {
      index++;
      notifyCursorMoved();
      return true;
    }
    return false;
  }
  public void close() {
    rows = null;
  }
  public bool wasNull() {
    return last_null;
  }
  public string getString(int columnIndex) {
    object retval = getObject(columnIndex);
    return retval != null ? retval.ToString() : null;
  }
  public bool getBoolean(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Boolean.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return false;
    }
  }
  public sbyte getByte(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return (sbyte)Int32.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }
  public short getShort(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return (short)Int32.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }
  public int getInt(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Int32.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }
  public long getLong(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Int64.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0;
    }
  }
  public float getFloat(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Single.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0.0f;
    }
  }
  public double getDouble(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Double.Parse(s);
    }
    catch(Exception e) {
//    throw new SQLException(e.Message);
      return 0.0;
    }
  }
  public System.Decimal getBigDecimal(int columnIndex, int scale) {
    return (System.Decimal)getObject(columnIndex);
  }
  public sbyte[] getSBytes(int columnIndex) {
    byte[] buff = (byte[])getObject(columnIndex);
    sbyte[] retval = new sbyte[buff.Length];
    for(int i = 0; i < buff.Length; i++)
      retval[i] = (sbyte)buff[i];
    return retval;
  }
  public byte[] getUBytes(int columnIndex) {
    return (byte[])getObject(columnIndex);
  }
  public System.DateTime getDate(int columnIndex) {
    return (System.DateTime)getObject(columnIndex);
  }
  public System.DateTime getTime(int columnIndex) {
    return (System.DateTime)getObject(columnIndex);
  }
  public System.DateTime getTimestamp(int columnIndex) {
    return (System.DateTime)getObject(columnIndex);
  }
  public BinaryReader getAsciiStream(int columnIndex) {
    throw new SQLException("mysql does not support getAsciiStream(int).");
  }
  public BinaryReader getUnicodeStream(int columnIndex) {
    throw new SQLException("mysql does not support getUnicodeStream(int).");
  }
  public BinaryReader getBinaryStream(int columnIndex) {
    byte[] bin_buf = getUBytes(columnIndex);
    if( bin_buf == null )
      return null;
    return new System.IO.BinaryReader(new System.IO.MemoryStream(bin_buf));
  }
  public string getString(string columnName) {
    return getString(findColumn(columnName));
  }
  public bool getBoolean(string columnName) {
    return getBoolean(findColumn(columnName));
  }
  public sbyte getByte(string columnName) {
    return getByte(findColumn(columnName));
  }
  public short getShort(string columnName) {
    return getShort(findColumn(columnName));
  }
  public int getInt(string columnName) {
    return getInt(findColumn(columnName));
  }
  public long getLong(string columnName) {
    return getLong(findColumn(columnName));
  }
  public float getFloat(string columnName) {
    return getFloat(findColumn(columnName));
  }
  public double getDouble(string columnName) {
    return getDouble(findColumn(columnName));
  }
  public System.Decimal getBigDecimal(string columnName, int scale) {
    return getBigDecimal(findColumn(columnName));
  }
  public sbyte[] getSBytes(string columnName) {
    return getSBytes(findColumn(columnName));
  }
  public byte[] getUBytes(string columnName) {
    return getUBytes(findColumn(columnName));
  }
  public System.DateTime getDate(string columnName) {
    return getDate(findColumn(columnName));
  }
  public System.DateTime getTime(string columnName) {
    return getTime(findColumn(columnName));
  }
  public System.DateTime getTimestamp(string columnName) {
    return getTimestamp(findColumn(columnName));
  }
  public BinaryReader getAsciiStream(string columnName) {
    return getAsciiStream(findColumn(columnName));
  }
  public BinaryReader getUnicodeStream(string columnName) {
    return getUnicodeStream(findColumn(columnName));
  }
  public BinaryReader getBinaryStream(string columnName) {
    return getBinaryStream(findColumn(columnName));
  }
  public SQLWarning getWarnings() {
    return null;
  }
  public void clearWarnings() {
  }
  public string getCursorName() {
    return "";
  }
  public ResultSetMetaData getMetaData() {
    return rs_meta;
  }
  public object getObject(int columnIndex) {
    try {
      ArrayList rowdata = (ArrayList)rows[index-1];
      object data = rowdata[columnIndex-1];
      last_null = data == null ? true : false;
      return data;
    }
    catch(Exception e) {
      return null;
    }
  }
  public object getObject(string columnName) {
    return getObject(findColumn(columnName));
  }
  public int findColumn(string name) {
    try {
      object nI = columnName[name];
      if (nI==null)
        nI = columnFullName[name];
      if (nI!=null)
        return ((int)nI)+1;
      throw new SQLException("Invalid column name : "+name);
    }
    catch( Exception e ) {
      throw new SQLException(e.Message);
    }
  }
  public StreamReader getCharacterStream(int columnIndex) {
    return null;
  }
  public StreamReader getCharacterStream(string columnName) {
    return getCharacterStream(findColumn(columnName));
  }
  public System.Decimal getBigDecimal(int columnIndex) {
    return getBigDecimal(columnIndex, 10);
  }
  public System.Decimal getBigDecimal(string columnName) {
    return getBigDecimal(findColumn(columnName));
  }
  public bool isBeforeFirst() {
    int row_count = rows.Count;
    if( row_count > 0 && index == 0 )
    {
      return true;
    }
    return false;
  }
  public bool isAfterLast() {
    int row_count = rows.Count;
    if( row_count > 0 && index == row_count + 1 )
    {
      return true;
    }
    return false;
  }
  public bool isFirst() {
    int row_count = rows.Count;
    if( row_count > 0 && index == 1 )
    {
      return true;
    }
    return false;
  }
  public bool isLast() {
    int row_count = rows.Count;
    if( row_count > 0 && index == row_count )
    {
      return true;
    }
    return false;
  }
  public void beforeFirst() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = 0;
      notifyCursorMoved();
    }
  }
  public void afterLast() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = row_count + 1;
      notifyCursorMoved();
    }
  }
  public bool first() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = 1;
      notifyCursorMoved();
      return true;
    }
    return false;
  }
  public bool last() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    if( row_count > 0 )
    {
      index = row_count;
      notifyCursorMoved();
      return true;
    }
    return false;
  }
  public int getRow() {
    return index;
  }
  public bool absolute(int row) {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    if( row == 0 )
      return false;
    int row_count = rows.Count;
    if( row < 0 )
      row = row_count + 1 + row;
    index = row;
    if( index >= 1 && index <= row_count )
    {
      notifyCursorMoved();
      return true;
    }
    if( index <= 0 )
      beforeFirst();
    else
      afterLast();
    return false;
  }
  public bool relative(int row) {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    int row_count = rows.Count;
    index += row;
    if( index >= 1 && index <= row_count )
    {
      notifyCursorMoved();
      return true;
    }
    if( index <= 0 )
      beforeFirst();
    else
      afterLast();
    return false;
  }
  public bool previous() {
    if( getType() == (int)ResultSet_TYPE.FORWARD_ONLY )
      throw new SQLException("TYPE_FORWARD_ONLY ResultSet does not support previous().");
    if( index - 1 >= 1 )
    {
      index--;
      notifyCursorMoved();
      return true;
    }
    return false;
  }
  public void setFetchDirection(int direction) {
    fetchDirection = direction;
  }
  public int getFetchDirection() {
    return fetchDirection;
  }
  public void setFetchSize(int rows) {
    fetchSize = rows;
  }
  public int getFetchSize() {
    return fetchSize;
  }
  public int getType() {
    return resultSetType;
  }
  public int getConcurrency() {
    return resultSetConcurrency;
  }
  public bool rowUpdated() {
    throw new SQLException("mysql does not support rowUpdated().");
  }
  public bool rowInserted() {
    throw new SQLException("mysql does not support rowInserted().");
  }
  public bool rowDeleted() {
    try {
      ArrayList rowdata = (ArrayList)rows[index-1];
      return rowdata == null;
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }
  public void updateNull(int columnIndex) {
    this.updateObject(columnIndex, null);
  }
  public void updateBoolean(int columnIndex, bool x) {
    throw new SQLException("mysql does not support updateBoolean(int, boolean).");
  }
  public void updateByte(int columnIndex, sbyte x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.TINYINT);
  }
  public void updateShort(int columnIndex, short x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.SMALLINT);
  }
  public void updateInt(int columnIndex, int x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.INTEGER);
  }
  public void updateLong(int columnIndex, long x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.BIGINT);
  }
  public void updateFloat(int columnIndex, float x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.FLOAT);
  }
  public void updateDouble(int columnIndex, double x) {
    object obj = x;
    this.updateObjectEx(columnIndex, obj, Types.DOUBLE);
  }
  public void updateBigDecimal(int columnIndex, System.Decimal x) {
    this.updateObjectEx(columnIndex, x, Types.DECIMAL);
  }
  public void updateString(int columnIndex, string x) {
    this.updateObjectEx(columnIndex, x, Types.VARCHAR);
  }
  public void updateBytes(int columnIndex, sbyte[] x) {
    this.updateObjectEx(columnIndex, x, Types.VARBINARY);
  }
  public void updateDate(int columnIndex, System.DateTime x) {
    this.updateObjectEx(columnIndex, x, Types.DATE);
  }
  public void updateTime(int columnIndex, System.DateTime x) {
    this.updateObjectEx(columnIndex, x, Types.TIME);
  }
  public void updateTimestamp(int columnIndex, System.DateTime x) {
    this.updateObjectEx(columnIndex, x, Types.TIMESTAMP);
  }
  public void updateAsciiStream(int columnIndex, BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateAsciiStream(int, InputStream, int).");
  }
  public void updateBinaryStream(int columnIndex, BinaryReader x, int length) {
    this.updateObjectEx(columnIndex, new gweMysqlPreparedObject(x, length), Types.LONGVARBINARY);
  }
  public void updateCharacterStream(int columnIndex, StreamReader x, int length) {
    throw new SQLException("mysql does not support updateCharacterStream(int, Reader, int).");
  }
  public void updateObject(int columnIndex, object x, int scale) {
    this.updateObject(columnIndex, x);
  }
  public void updateObject(int columnIndex, object x) {
    ResultSetMetaData meta = this.getMetaData();
    int targetSqlType = meta.getColumnType(columnIndex);
    updateObjectEx(columnIndex, x, targetSqlType);
  }
  private void updateObjectEx(int columnIndex, Object x, int targetSqlType) {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support updateObject.");

    if( update_obj_array  != null &&
        update_type_array != null )
    {
      if( columnIndex <= update_obj_array.Length  &&
          columnIndex <= update_type_array.Length )
      {
        update_obj_array[columnIndex-1] = x;
        if( (x is byte[]) && (targetSqlType != Types.VARBINARY) )
          targetSqlType = Types.VARBINARY;
        if( (x is BinaryReader) && (targetSqlType != Types.LONGVARBINARY) )
          targetSqlType = Types.LONGVARBINARY;
        update_type_array[columnIndex-1] = targetSqlType;
        return;
      }
    }
    throw new SQLException("gweMysqlRowSet Error : updateObject(int, Object).");
  }
  public void updateNull(string columnName) {
    updateNull(findColumn(columnName));
  }
  public void updateBoolean(string columnName, bool x) {
    throw new SQLException("mysql does not support updateBoolean(String, boolean).");
  }
  public void updateByte(string columnName, sbyte x) {
    updateByte(findColumn(columnName), x);
  }
  public void updateShort(string columnName, short x) {
    updateShort(findColumn(columnName), x);
  }
  public void updateInt(string columnName, int x) {
    updateInt(findColumn(columnName), x);
  }
  public void updateLong(string columnName, long x) {
    updateLong(findColumn(columnName), x);
  }
  public void updateFloat(string columnName, float x) {
    updateFloat(findColumn(columnName), x);
  }
  public void updateDouble(string columnName, double x) {
    updateDouble(findColumn(columnName), x);
  }
  public void updateBigDecimal(string columnName, System.Decimal x) {
    updateBigDecimal(findColumn(columnName), x);
  }
  public void updateString(string columnName, string x) {
    updateString(findColumn(columnName), x);
  }
  public void updateBytes(string columnName, sbyte[] x) {
    updateBytes(findColumn(columnName), x);
  }
  public void updateDate(string columnName, System.DateTime x) {
    updateDate(findColumn(columnName), x);
  }
  public void updateTime(string columnName, System.DateTime x) {
    updateTime(findColumn(columnName), x);
  }
  public void updateTimestamp(string columnName, System.DateTime x) {
    updateTimestamp(findColumn(columnName), x);
  }
  public void updateAsciiStream(string columnName, BinaryReader x, int length) {
    throw new SQLException("mysql does not support updateAsciiStream(String, InputStream, int).");
  }
  public void updateBinaryStream(string columnName, BinaryReader x, int length) {
    updateBinaryStream(findColumn(columnName), x, length);
  }
  public void updateCharacterStream(string columnName, StreamReader reader, int length) {
    throw new SQLException("mysql does not support updateCharacterStream(String, Reader, int).");
  }
  public void updateObject(string columnName, object x, int scale) {
    updateObject(findColumn(columnName), x, scale);
  }
  public void updateObject(string columnName, object x) {
    updateObject(findColumn(columnName), x);
  }
  private Connection getConnectionForUpdate() {
    if( updater == null )
    {
      updater = getConnection();
      if( transactionIsolation != gweMysqlRowSet.TRANSACTION_UNKNOWN )
        updater.setTransactionIsolation(transactionIsolation);
      updater.setReadOnly(readOnly);
      updater.setTypeMap(typemap);
    }
    return updater;
  }
  public void insertRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support insertRow.");
    if( !onetable )
      throw new SQLException("RowSet generated from Multiple Tables does not support insertRow.");
    if( !insertmode )
      throw new SQLException("insertRow() cannot be called when the cursor is not on the insert row.");

    ResultSetMetaData meta = this.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("INSERT INTO ");
    updatesql.append(meta.getTableName(1));
    updatesql.append("(");
    bool first = true;
    for(int i = 1; i <= update_obj_array.Length; i++)
    {
      if( update_type_array[i-1] != Types.NULL )
      {
        if( first )
          first = false;
        else
          updatesql.append(",");
        updatesql.append(meta.getColumnName(i));
      }
    }
    updatesql.append(")");
    updatesql.append(" VALUES(");
    first = true;
    for(int i = 1; i <= update_obj_array.Length; i++)
    {
      if( update_type_array[i-1] != Types.NULL )
      {
        if( first )
          first = false;
        else
          updatesql.append(",");
        updatesql.append("?");
      }
    }
    updatesql.append(");");

    Connection conn = getConnectionForUpdate();
    PreparedStatement stmt = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 1; i <= update_obj_array.Length; i++)
      {
        int targetSqlType = update_type_array[i-1];
        if( targetSqlType != Types.NULL )
        {
          stmt.setObject(param_index, update_obj_array[i-1], targetSqlType);
          param_index++;
        }
      }
      int count = stmt.executeUpdate();
      if( count == 1 )
      {
        ArrayList vecrow = new ArrayList();
        for(int i = 1; i <= update_obj_array.Length; i++)
        {
          if( update_type_array[i-1] != Types.NULL )
            vecrow.Add(update_obj_array[i-1]);
          else
            vecrow.Add(null);
        }
        rows.Add(vecrow);
        notifyRowChanged();
      }
      else
      {
        throw new SQLException("Can not insert Row.");
      }
    }
    finally {
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
    }
  }
  public void updateRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support updateRow.");
    if( !onetable )
      throw new SQLException("RowSet generated from Multiple Tables does not support updateRow.");
    if( insertmode )
      throw new SQLException("updateRow() cannot be called when the cursor is on the insert row.");

    ResultSetMetaData meta = this.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("UPDATE ");
    updatesql.append(meta.getTableName(1));
    updatesql.append(" SET ");
    bool first = true;
    for(int i = 1; i <= update_obj_array.Length; i++)
    {
      if( update_type_array[i-1] != Types.NULL )
      {
        if( first )
          first = false;
        else
          updatesql.append(",");
        updatesql.append(meta.getColumnName(i));
        updatesql.append("=?");
      }
    }
    updatesql.append(" WHERE ");
    for(int i = 0; i < pk_cols.Count; i++)
    {
      if( i != 0 )
        updatesql.append(" AND ");
      string pkname = (string)pk_cols[i];
      updatesql.append(pkname);
      updatesql.append("=?");
    }
    updatesql.append(";");

    Connection conn = getConnectionForUpdate();
    PreparedStatement stmt = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 1; i <= update_obj_array.Length; i++)
      {
        int targetSqlType = update_type_array[i-1];
        if( targetSqlType != Types.NULL )
        {
          stmt.setObject(param_index, update_obj_array[i-1], targetSqlType);
          param_index++;
        }
      }
      for(int i = 0; i < pk_cols.Count; i++)
      {
        string pkname = (string)pk_cols[i];
        int targetSqlType = meta.getColumnType(findColumn(pkname));
        stmt.setObject(param_index, getObject(pkname), targetSqlType);
        param_index++;
      }
      int count = stmt.executeUpdate();
      if( count == 1 )
      {
        ArrayList rowdata = (ArrayList)rows[index-1];
        for(int i = 1; i <= update_obj_array.Length; i++)
        {
          if( update_type_array[i-1] != Types.NULL )
            rowdata[i-1] = update_obj_array[i-1];
        }
        notifyRowChanged();
      }
    }
    finally {
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
    }
  }
  public void deleteRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support deleteRow.");
    if( !onetable )
      throw new SQLException("RowSet generated from Multiple Tables does not support deleteRow.");
    if( insertmode )
      throw new SQLException("deleteRow() cannot be called when the cursor is on the insert row.");

    ResultSetMetaData meta = this.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("DELETE FROM ");
    updatesql.append(meta.getTableName(1));
    updatesql.append(" WHERE ");
    for(int i = 0; i < pk_cols.Count; i++)
    {
      if( i != 0 )
        updatesql.append(" AND ");
      string pkname = (string)pk_cols[i];
      updatesql.append(pkname);
      updatesql.append("=?");
    }
    updatesql.append(";");

    Connection conn = getConnectionForUpdate();
    PreparedStatement stmt = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 0; i < pk_cols.Count; i++)
      {
        string pkname = (string)pk_cols[i];
        int targetSqlType = meta.getColumnType(findColumn(pkname));
        stmt.setObject(param_index, getObject(pkname), targetSqlType);
        param_index++;
      }
      int count = stmt.executeUpdate();
      if( count == 1 )
      {
        rows[index-1] = null;
        notifyRowChanged();
      }
      else
      {
        throw new SQLException("Can not delete Row.");
      }
    }
    finally {
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
    }
  }
  public void refreshRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support refreshRow.");
    if( !onetable )
      throw new SQLException("RowSet generated from Multiple Tables does not support refreshRow.");
    if( insertmode )
      throw new SQLException("refreshRow() cannot be called when the cursor is on the insert row.");

    if( rowDeleted() )
      return;

    ResultSetMetaData meta = this.getMetaData();
    gweMysqlStringBuffer updatesql = new gweMysqlStringBuffer();
    updatesql.append("SELECT ");
    int colnum = meta.getColumnCount();
    for(int i = 1; i <= colnum; i++)
    {
      if( i != 1 )
        updatesql.append(",");
      updatesql.append(meta.getColumnName(i));
    }
    updatesql.append(" FROM ");
    updatesql.append(meta.getTableName(1));
    updatesql.append(" WHERE ");
    for(int i = 0; i < pk_cols.Count; i++)
    {
      if( i != 0 )
        updatesql.append(" AND ");
      string pkname = (string)pk_cols[i];
      updatesql.append(pkname);
      updatesql.append("=?");
    }
    updatesql.append(";");

    Connection conn = getConnectionForUpdate();
    PreparedStatement stmt = null;
    ResultSet rs = null;
    try {
      stmt = conn.prepareStatement(updatesql.ToString());
      int param_index = 1;
      for(int i = 0; i < pk_cols.Count; i++)
      {
        string pkname = (string)pk_cols[i];
        int targetSqlType = meta.getColumnType(findColumn(pkname));
        stmt.setObject(param_index, getObject(pkname), targetSqlType);
        param_index++;
      }
      rs = stmt.executeQuery();
      if( rs.next() )
      {
        ArrayList rowdata = (ArrayList)rows[index-1];
        for(int i = 1; i <= colnum; i++)
          rowdata[i-1] = rs.getObject(i);
      }
      else
      {
        throw new SQLException("Can not refresh Row.");
      }
    }
    finally {
      if( rs != null )
      {
        try { rs.close(); }
        catch(Exception ex) { }
        rs = null;
      }
      if( stmt != null )
      {
        try { stmt.close(); }
        catch(Exception ex) { }
        stmt = null;
      }
      clearUpdaters();
    }
  }
  private void clearUpdaters() {
    if( update_obj_array  != null &&
        update_type_array != null )
    {
      int i;
      for(i = 0; i < update_obj_array.Length; i++)
        update_obj_array[i]  = null;
      for(i = 0; i < update_type_array.Length; i++)
        update_type_array[i] = Types.NULL;
    }
  }
  public void cancelRowUpdates() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support cancelRowUpdates.");

    clearUpdaters();
  }
  public void moveToInsertRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support moveToInsertRow.");
    if( !onetable )
      throw new SQLException("RowSet generated from Multiple Tables does not support moveToInsertRow.");

    insertmode = true;
    clearUpdaters();
  }
  public void moveToCurrentRow() {
    if( getConcurrency() != (int)ResultSet_CONCUR.UPDATABLE )
      throw new SQLException("CONCUR_READ_ONLY RowSet does not support moveToCurrentRow.");

    insertmode = false;
    clearUpdaters();
  }
  public Statement getStatement() {
    return new gweMysqlRowSetStatement(getConnectionForUpdate());
  }
  public object getObject(int i, Hashtable map) {
    throw new SQLException("mysql does not support getObject(int, Map).");
  }
  public Ref getRef(int i) {
    throw new SQLException("mysql does not support getRef(int).");
  }
  public Blob getBlob(int i) {
    throw new SQLException("mysql does not support getBlob(int).");
  }
  public Clob getClob(int i) {
    throw new SQLException("mysql does not support getClob(int).");
  }
  public net.sql.Array getArray(int i) {
    throw new SQLException("mysql does not support getArray(int).");
  }
  public object getObject(string colName, Hashtable map) {
    throw new SQLException("mysql does not support getObject(String, Map).");
  }
  public Ref getRef(string colName) {
    throw new SQLException("mysql does not support getRef(String).");
  }
  public Blob getBlob(string colName) {
    throw new SQLException("mysql does not support getBlob(String).");
  }
  public Clob getClob(string colName) {
    throw new SQLException("mysql does not support getClob(String).");
  }
  public net.sql.Array getArray(string colName) {
    throw new SQLException("mysql does not support getArray(String).");
  }
  public System.DateTime getDate(int columnIndex, System.Globalization.Calendar cal) {
    return getDate(columnIndex);
  }
  public System.DateTime getDate(string columnName, System.Globalization.Calendar cal) {
    return getDate(findColumn(columnName));
  }
  public System.DateTime getTime(int columnIndex, System.Globalization.Calendar cal) {
    return getTime(columnIndex);
  }
  public System.DateTime getTime(string columnName, System.Globalization.Calendar cal) {
    return getTime(findColumn(columnName));
  }
  public System.DateTime getTimestamp(int columnIndex, System.Globalization.Calendar cal) {
    return getTimestamp(columnIndex);
  }
  public System.DateTime getTimestamp(string columnName, System.Globalization.Calendar cal) {
    return getTimestamp(findColumn(columnName));
  }

  //-------------------------- JDBC 3.0 ---------------------------------------
  public string getURL(int columnIndex) {
    throw new SQLException("mysql does not support getURL(int).");
  }
  public string getURL(string columnName) {
    throw new SQLException("mysql does not support getURL(String).");
  }
  public void updateRef(int columnIndex, Ref x) {
    throw new SQLException("mysql does not support updateRef(int, Ref).");
  }
  public void updateRef(string columnName, Ref x) {
    throw new SQLException("mysql does not support updateRef(String, Ref).");
  }
  public void updateBlob(int columnIndex, Blob x) {
    throw new SQLException("mysql does not support updateBlob(int, Blob).");
  }
  public void updateBlob(string columnName, Blob x) {
    throw new SQLException("mysql does not support updateBlob(String, Blob).");
  }
  public void updateClob(int columnIndex, Clob x) {
    throw new SQLException("mysql does not support updateClob(int, Blob).");
  }
  public void updateClob(string columnName, Clob x) {
    throw new SQLException("mysql does not support updateClob(String, Blob).");
  }
  public void updateArray(int columnIndex, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(int, Array).");
  }
  public void updateArray(string columnName, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(String, Array).");
  }

  private void notifyCursorMoved() {
    if( listeners.Count > 0 )
    {
      RowSetEvent rowsetevent = new RowSetEvent((RowSet)this);
      IEnumerator iterator = listeners.GetEnumerator();
      while( iterator.MoveNext() )
      {
        ((RowSetListener)iterator.Current).cursorMoved(rowsetevent);
      }
    }
  }
  private void notifyRowChanged() {
    if( listeners.Count > 0 )
    {
      RowSetEvent rowsetevent = new RowSetEvent((RowSet)this);
      IEnumerator iterator = listeners.GetEnumerator();
      while( iterator.MoveNext() )
      {
        ((RowSetListener)iterator.Current).rowChanged(rowsetevent);
      }
    }
  }
  private void notifyRowSetChanged() {
    if( listeners.Count > 0 )
    {
      RowSetEvent rowsetevent = new RowSetEvent((RowSet)this);
      IEnumerator iterator = listeners.GetEnumerator();
      while( iterator.MoveNext() )
      {
        ((RowSetListener)iterator.Current).rowSetChanged(rowsetevent);
      }
    }
  }

  //// for Serialization
  //private void writeObject(ObjectOutputStream oos) throws IOException {
  //  oos.defaultWriteObject();
  //}
  //private void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException {
  //  ois.defaultReadObject();
  //}
}

internal class gweMysqlRowSetStatement : Statement {
  private Connection connection = null;

  public gweMysqlRowSetStatement(Connection connection) {
    this.connection = connection;
  }
  public Connection getConnection() {
    return connection;
  }

  public ResultSet executeQuery(string sql) {
    throw new System.NotImplementedException("method executeQuery() is not implemented.");
  }
  public int executeUpdate(string sql) {
    throw new System.NotImplementedException("method executeUpdate() is not implemented.");
  }
  public void close() {
    throw new System.NotImplementedException("method close() is not implemented.");
  }
  public int getMaxFieldSize() {
    throw new System.NotImplementedException("method getMaxFieldSize() is not implemented.");
  }
  public void setMaxFieldSize(int max) {
    throw new System.NotImplementedException("method setMaxFieldSize() is not implemented.");
  }
  public int getMaxRows() {
    throw new System.NotImplementedException("method getMaxRows() is not implemented.");
  }
  public void setMaxRows(int max) {
    throw new System.NotImplementedException("method setMaxRows() is not implemented.");
  }
  public void setEscapeProcessing(bool enable) {
    throw new System.NotImplementedException("method setEscapeProcessing() is not implemented.");
  }
  public int getQueryTimeout() {
    throw new System.NotImplementedException("method getQueryTimeout() is not implemented.");
  }
  public void setQueryTimeout(int seconds) {
    throw new System.NotImplementedException("method setQueryTimeout() is not implemented.");
  }
  public void cancel() {
    throw new System.NotImplementedException("method cancel() is not implemented.");
  }
  public SQLWarning getWarnings() {
    throw new System.NotImplementedException("method getWarnings() is not implemented.");
  }
  public void clearWarnings() {
    throw new System.NotImplementedException("method clearWarnings() is not implemented.");
  }
  public void setCursorName(string name) {
    throw new System.NotImplementedException("method setCursorName() is not implemented.");
  }
  public bool execute(string sql) {
    throw new System.NotImplementedException("method execute() is not implemented.");
  }
  public ResultSet getResultSet() {
    throw new System.NotImplementedException("method getResultSet() is not implemented.");
  }
  public int getUpdateCount() {
    throw new System.NotImplementedException("method getUpdateCount() is not implemented.");
  }
  public bool getMoreResults() {
    throw new System.NotImplementedException("method getMoreResults() is not implemented.");
  }
  public void setFetchDirection(int direction) {
    throw new System.NotImplementedException("method setFetchDirection() is not implemented.");
  }
  public int getFetchDirection() {
    throw new System.NotImplementedException("method getFetchDirection() is not implemented.");
  }
  public void setFetchSize(int rows) {
    throw new System.NotImplementedException("method setFetchSize() is not implemented.");
  }
  public int getFetchSize() {
    throw new System.NotImplementedException("method getFetchSize() is not implemented.");
  }
  public int getResultSetConcurrency() {
    throw new System.NotImplementedException("method getResultSetConcurrency() is not implemented.");
  }
  public int getResultSetType() {
    throw new System.NotImplementedException("method getResultSetType() is not implemented.");
  }
  public void addBatch(string sql) {
    throw new System.NotImplementedException("method addBatch() is not implemented.");
  }
  public void clearBatch() {
    throw new System.NotImplementedException("method clearBatch() is not implemented.");
  }
  public int[] executeBatch() {
    throw new System.NotImplementedException("method executeBatch() is not implemented.");
  }
  //--------------------------JDBC 3.0-----------------------------
  bool hasAutoGeneratedKey() {
    return false;
  }
  public bool getMoreResults(int current) {
    throw new SQLException("RowSet does not support getMoreResults().");
  }
  public ResultSet getGeneratedKeys() {
    throw new SQLException("RowSet does not support getGeneratedKeys().");
  }
  public int executeUpdate(string sql, int autoGeneratedKeys) {
    throw new SQLException("RowSet does not support executeUpdate().");
  }
  public int executeUpdate(string sql, int[] columnIndexes) {
    throw new SQLException("RowSet does not support executeUpdate().");
  }
  public int executeUpdate(string sql, string[] columnNames) {
    throw new SQLException("RowSet does not support executeUpdate().");
  }
  public bool execute(string sql, int autoGeneratedKeys) {
    throw new SQLException("RowSet does not support execute().");
  }
  public bool execute(string sql, int[] columnIndexes) {
    throw new SQLException("RowSet does not support execute().");
  }
  public bool execute(string sql, string[] columnNames) {
    throw new SQLException("RowSet does not support execute().");
  }
  public int getResultSetHoldability() {
    throw new SQLException("RowSet does not support getResultSetHoldability().");
  }
}

}
