//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

namespace gwe.sql {

using System;
using System.IO;
using System.Collections;
using System.Globalization;
using net.sql;

public class gweMysqlVectorResultSet : ResultSet, ResultSetMetaData {

  private gweMysqlConnection conn;
  private gweMysqlColumn[] cols;
  private ArrayList rows;
  private int index;

  public gweMysqlVectorResultSet(gweMysqlConnection org_conn,
                               gweMysqlColumn[] org_cols, ArrayList org_rows) {
    conn = org_conn;
    cols = org_cols;
    rows = org_rows;
    index = 0;
  }

  /////////////// implements ResultSet //////////////////////////////////////
  public bool next() {
    int row_count = rows.Count;
    index++;
    if( index <= row_count )
      return true;
    return false;
  }

  public void close() {
  }

  public bool wasNull() {
    return false;
  }

  private int getColumnIndex(string columnName) {
    int retval = -1;
    for(int i = 0; i < cols.Length; i++)
    {
      gweMysqlColumn coldata = cols[i];
      if( coldata.getName().Equals(columnName) == true )
      {
        retval = i + 1;
        break;
      }
    }
    return retval;
  }

  public string getString(int columnIndex) {
    try {
      ArrayList rowdata = (ArrayList)rows[index-1];
      object data = rowdata[columnIndex-1];
      if( data == null )
        return "";
      return data.ToString();
    }
    catch(Exception e) {
      Console.WriteLine(e.Message);
      Console.WriteLine(e.StackTrace);
      throw new SQLException(e.Message);
    }
  }

  public bool getBoolean(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Boolean.Parse(s);
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  public sbyte getByte(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public short getShort(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return (short)Int32.Parse(s);
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  public int getInt(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Int32.Parse(s);
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  public long getLong(int columnIndex) {
    string s = getString(columnIndex);
    try {
      return Int64.Parse(s);
    }
    catch(Exception e) {
      throw new SQLException(e.Message);
    }
  }

  public float getFloat(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public double getDouble(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.Decimal getBigDecimal(int columnIndex, int scale)
                                                        {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public sbyte[] getSBytes(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }
  public byte[] getUBytes(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getDate(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTime(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTimestamp(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.BinaryReader getAsciiStream(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.BinaryReader getUnicodeStream(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.BinaryReader getBinaryStream(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public string getString(string columnName) {
    return getString(getColumnIndex(columnName));
  }

  public bool getBoolean(string columnName) {
    return getBoolean(getColumnIndex(columnName));
  }

  public sbyte getByte(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public short getShort(string columnName) {
    return getShort(getColumnIndex(columnName));
  }

  public int getInt(string columnName) {
    return getInt(getColumnIndex(columnName));
  }

  public long getLong(string columnName) {
    return getLong(getColumnIndex(columnName));
  }

  public float getFloat(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public double getDouble(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.Decimal getBigDecimal(string columnName, int scale)
                                                         {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public sbyte[] getSBytes(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }
  public byte[] getUBytes(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getDate(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTime(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTimestamp(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.BinaryReader getAsciiStream(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.BinaryReader getUnicodeStream(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.BinaryReader getBinaryStream(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public SQLWarning getWarnings() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void clearWarnings() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public string getCursorName() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public ResultSetMetaData getMetaData() {
    return this;
  }

  public object getObject(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public object getObject(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public int findColumn(string columnName) {
    return 0;
  }

  public System.IO.StreamReader getCharacterStream(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.IO.StreamReader getCharacterStream(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.Decimal getBigDecimal(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.Decimal getBigDecimal(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool isBeforeFirst() {
    return false;
  }

  public bool isAfterLast() {
    return false;
  }

  public bool isFirst() {
    return false;
  }

  public bool isLast() {
    return false;
  }

  public void beforeFirst() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void afterLast() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool first() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool last() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public int getRow() {
    return 0;
  }

  public bool absolute(int row) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool relative(int rows) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool previous() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void setFetchDirection(int direction) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public int getFetchDirection() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void setFetchSize(int rows) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public int getFetchSize() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public int getType() {
    return 0;
  }

  public int getConcurrency() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool rowUpdated() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool rowInserted() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public bool rowDeleted() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateNull(int columnIndex) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBoolean(int columnIndex, bool x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateByte(int columnIndex, sbyte x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateShort(int columnIndex, short x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateInt(int columnIndex, int x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateLong(int columnIndex, long x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateFloat(int columnIndex, float x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateDouble(int columnIndex, double x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBigDecimal(int columnIndex, System.Decimal x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateString(int columnIndex, string x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBytes(int columnIndex, sbyte[] x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateDate(int columnIndex, System.DateTime x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateTime(int columnIndex, System.DateTime x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateTimestamp(int columnIndex, System.DateTime x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateAsciiStream(int columnIndex, System.IO.BinaryReader x, int length) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }
  public void updateBinaryStream(int columnIndex, System.IO.BinaryReader x, int length) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateCharacterStream(int columnIndex, System.IO.StreamReader x, int length) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateObject(int columnIndex, object x, int scale) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateObject(int columnIndex, object x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateNull(string columnName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBoolean(string columnName, bool x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateByte(string columnName, sbyte x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateShort(string columnName, short x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateInt(string columnName, int x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateLong(string columnName, long x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateFloat(string columnName, float x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateDouble(string columnName, double x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBigDecimal(string columnName, System.Decimal x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateString(string columnName, string x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBytes(string columnName, sbyte[] x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateDate(string columnName, System.DateTime x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateTime(string columnName, System.DateTime x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateTimestamp(string columnName, System.DateTime x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateAsciiStream(string columnName, System.IO.BinaryReader x, int length) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateBinaryStream(string columnName, System.IO.BinaryReader x, int length) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateCharacterStream(string columnName, System.IO.StreamReader reader, int length) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateObject(string columnName, object x, int scale) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateObject(string columnName, object x) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void insertRow() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void updateRow() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void deleteRow() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void refreshRow() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void cancelRowUpdates() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void moveToInsertRow() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public void moveToCurrentRow() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Statement getStatement() {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public object getObject(int i, System.Collections.Hashtable map) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Ref getRef(int i) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Blob getBlob(int i) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Clob getClob(int i) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public net.sql.Array getArray(int i) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public object getObject(string colName, System.Collections.Hashtable map) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Ref getRef(string colName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Blob getBlob(string colName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public Clob getClob(string colName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public net.sql.Array getArray(string colName) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getDate(int columnIndex, System.Globalization.Calendar cal) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getDate(String columnName, System.Globalization.Calendar cal) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTime(int columnIndex, System.Globalization.Calendar cal) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTime(String columnName, System.Globalization.Calendar cal) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTimestamp(int columnIndex, System.Globalization.Calendar cal) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }

  public System.DateTime getTimestamp(String columnName, System.Globalization.Calendar cal) {
    throw new SQLException("gweMysqlVectorResultSet does not support this method.");
  }


  /////////////// implements ResultSetMetaData ////////////////////////////
  public int getColumnCount() {
    return cols.Length;
  }

  public bool isAutoIncrement(int column) {
    return false;
  }

  public bool isCaseSensitive(int column) {
    return true;
  }

  public bool isSearchable(int column) {
    return true;
  }

  public bool isCurrency(int column) {
    return false;
  }

  public int isNullable(int column) {
    return (int)ResultSetMetaData_column.NoNulls;
  }

  public bool isSigned(int column) {
    return false;
  }

  public int getColumnDisplaySize(int column) {
    int max_size = 1;
    for(int i = 0; i < rows.Count; i++)
    {
      ArrayList rowdata = (ArrayList)rows[i];
      object data = rowdata[column-1];
      if( data == null )
        continue;
      int temp_len = data.ToString().Length;
      if( temp_len > max_size )
        max_size = temp_len;
    }
    return max_size;
  }

  public string getColumnLabel(int column) {
    return cols[column-1].getName();
  }

  public string getColumnName(int column) {
    return cols[column-1].getName();
  }

  public string getSchemaName(int column) {
    return "";
  }

  public int getPrecision(int column) {
    return 0;
  }

  public int getScale(int column) {
    return 0;
  }

  public string getTableName(int column) {
    return cols[column-1].getTable();
  }

  public string getCatalogName(int column) {
    return "";
  }

  public int getColumnType(int column) {
    gweMysqlColumn col = cols[column-1];
    switch (col.getType()) {
      case gweMysqlMetaResult.FIELD_TYPE_DECIMAL     : return Types.DECIMAL;
      case gweMysqlMetaResult.FIELD_TYPE_TINY        : return Types.TINYINT;
      case gweMysqlMetaResult.FIELD_TYPE_SHORT       : return Types.SMALLINT;
      case gweMysqlMetaResult.FIELD_TYPE_LONG        : return Types.INTEGER;
      case gweMysqlMetaResult.FIELD_TYPE_FLOAT       : return Types.FLOAT;
      case gweMysqlMetaResult.FIELD_TYPE_DOUBLE      : return Types.DOUBLE;
      case gweMysqlMetaResult.FIELD_TYPE_NULL        : return Types.NULL;
      case gweMysqlMetaResult.FIELD_TYPE_TIMESTAMP   : return Types.TIMESTAMP;
      case gweMysqlMetaResult.FIELD_TYPE_LONGLONG    : return Types.BIGINT;
      case gweMysqlMetaResult.FIELD_TYPE_INT24       : return Types.INTEGER;
      case gweMysqlMetaResult.FIELD_TYPE_DATE        : return Types.DATE;
      case gweMysqlMetaResult.FIELD_TYPE_TIME        : return Types.TIME;
      case gweMysqlMetaResult.FIELD_TYPE_DATETIME    : return Types.TIMESTAMP;
      case gweMysqlMetaResult.FIELD_TYPE_NEWDATE     : return Types.DATE;
      case gweMysqlMetaResult.FIELD_TYPE_ENUM        : return Types.CHAR;
      case gweMysqlMetaResult.FIELD_TYPE_SET         : return Types.CHAR;
      case gweMysqlMetaResult.FIELD_TYPE_TINY_BLOB   : return Types.VARBINARY;
      case gweMysqlMetaResult.FIELD_TYPE_MEDIUM_BLOB : return Types.LONGVARBINARY;
      case gweMysqlMetaResult.FIELD_TYPE_LONG_BLOB   : return Types.LONGVARBINARY;
      case gweMysqlMetaResult.FIELD_TYPE_BLOB        : //return Types.LONGVARBINARY;
        // Thanks for INDEN-san
        if( col.isBinary() ) return Types.LONGVARBINARY;
        return Types.LONGVARCHAR;
      case gweMysqlMetaResult.FIELD_TYPE_VAR_STRING  : return Types.VARCHAR;
      case gweMysqlMetaResult.FIELD_TYPE_STRING      : return Types.CHAR;
      case gweMysqlMetaResult.FIELD_TYPE_YEAR        : return Types.SMALLINT; // YEAR support!!
      default : throw new SQLException("Unknown data type.");
    }
  }

  public string getColumnTypeName(int column) {
    gweMysqlColumn col = cols[column-1];
    switch (col.getType()) {
      case gweMysqlMetaResult.FIELD_TYPE_DECIMAL:
        return "DECIMAL";
      case gweMysqlMetaResult.FIELD_TYPE_TINY:
        return "TINY";
      case gweMysqlMetaResult.FIELD_TYPE_SHORT:
        return "SHORT";
      case gweMysqlMetaResult.FIELD_TYPE_LONG:
        return "LONG";
      case gweMysqlMetaResult.FIELD_TYPE_FLOAT:
        return "FLOAT";
      case gweMysqlMetaResult.FIELD_TYPE_DOUBLE:
        return "DOUBLE";
      case gweMysqlMetaResult.FIELD_TYPE_NULL:
        return "NULL";
      case gweMysqlMetaResult.FIELD_TYPE_TIMESTAMP:
        return "TIMESTAMP";
      case gweMysqlMetaResult.FIELD_TYPE_LONGLONG:
        return "LONGLONG";
      case gweMysqlMetaResult.FIELD_TYPE_INT24:
        return "INT";
      case gweMysqlMetaResult.FIELD_TYPE_DATE:
        return "DATE";
      case gweMysqlMetaResult.FIELD_TYPE_TIME:
        return "TIME";
      case gweMysqlMetaResult.FIELD_TYPE_DATETIME:
        return "DATETIME";
      case gweMysqlMetaResult.FIELD_TYPE_TINY_BLOB:
        return "TINYBLOB";
      case gweMysqlMetaResult.FIELD_TYPE_MEDIUM_BLOB:
        return "MEDIUMBLOB";
      case gweMysqlMetaResult.FIELD_TYPE_LONG_BLOB:
        return "LONGBLOB";
      case gweMysqlMetaResult.FIELD_TYPE_BLOB:
//        return "BLOB";
        // Thanks for INDEN-san
        if( col.isBinary() ) return "LONGBLOB";
        return "LONGTEXT";
      case gweMysqlMetaResult.FIELD_TYPE_VAR_STRING:
        return "VARCHAR";
      case gweMysqlMetaResult.FIELD_TYPE_STRING:
        return "CHAR";
      case gweMysqlMetaResult.FIELD_TYPE_ENUM:
        return "ENUM";
      case gweMysqlMetaResult.FIELD_TYPE_SET:
        return "SET";
      case gweMysqlMetaResult.FIELD_TYPE_YEAR:
        return "YEAR"; // YEAR support!!
      default:
        return "UNKNOWN";
    }
  }

  public bool isReadOnly(int column) {
    return false;
  }

  public bool isWritable(int column) {
    return true;
  }

  public bool isDefinitelyWritable(int column) {
    return true;
  }

  public string getColumnClassName(int column) {
    throw new SQLException("mysql does not support getColumnClassName(int).");
  }

  //------------------------- JDBC 3.0 -----------------------------------

  public string getURL(int columnIndex) {
    throw new SQLException("mysql does not support getURL(int).");
  }
  public string getURL(string columnName) {
    throw new SQLException("mysql does not support getURL(String).");
  }
  public void updateRef(int columnIndex, Ref x) {
    throw new SQLException("mysql does not support updateRef(int, Ref).");
  }
  public void updateRef(string columnName, Ref x) {
    throw new SQLException("mysql does not support updateRef(String, Ref).");
  }
  public void updateBlob(int columnIndex, Blob x) {
    throw new SQLException("mysql does not support updateBlob(int, Blob).");
  }
  public void updateBlob(string columnName, Blob x) {
    throw new SQLException("mysql does not support updateBlob(String, Blob).");
  }
  public void updateClob(int columnIndex, Clob x) {
    throw new SQLException("mysql does not support updateClob(int, Clob).");
  }
  public void updateClob(String columnName, Clob x) {
    throw new SQLException("mysql does not support updateClob(String, Clob).");
  }
  public void updateArray(int columnIndex, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(int, Array).");
  }
  public void updateArray(string columnName, net.sql.Array x) {
    throw new SQLException("mysql does not support updateArray(String, Array).");
  }
}

}
