//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 12.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import java.util.*;
import javax.sql.ConnectionEventListener;
import javax.sql.XAConnection;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;
import javax.transaction.xa.XAException;

public class gweMysqlXAConnection implements XAConnection, Connection, XAResource {

  private Connection org_conn = null;
  private gweMysqlXADataSource pool = null;
  private ConnectionEventListener listener = null;

  public gweMysqlXAConnection(gweMysqlXADataSource pool,
                              Connection org_conn) {
    this.pool = pool;
    this.org_conn = org_conn;
  }

  public int hashCode() {
    return org_conn.hashCode();
  }

  public boolean equals(Object obj) {
    if( obj != null && obj instanceof gweMysqlXAConnection )
    {
      gweMysqlXAConnection that = (gweMysqlXAConnection)obj;
      return (this.org_conn == that.org_conn);
    }
    return false;
  }

  public void close() throws java.sql.SQLException {
//System.out.println("close!!!");
    //org_conn.close();
    pool.storeConnection(this);
  }

  Connection getOrgConnection() {
//System.out.println("getOrgConnection!!!");
    return org_conn;
  }

  public Connection getConnection() throws java.sql.SQLException {
//System.out.println("getConnection!!!");
    return this;
  }

  public boolean isClosed() throws SQLException {
    //boolean retval = org_conn.isClosed();
    return org_conn == null;
  }

  public XAResource getXAResource() throws java.sql.SQLException {
//System.out.println("(XA)getXAResource!!!");
    return this;
  }

  public void addConnectionEventListener(ConnectionEventListener listener) {
//System.out.println("(XA)addConnectionEventListener");
    this.listener = listener;
  }

  public void removeConnectionEventListener(ConnectionEventListener listener) {
//System.out.println("(XA)removeConnectionEventListener");
    this.listener = null;
  }

  public Statement createStatement() throws SQLException {
//System.out.println("createStatement!!!");
    Statement retval = org_conn.createStatement();
    return retval;
  }
  public PreparedStatement prepareStatement(String sql) throws SQLException {
//System.out.println("prepareStatement!!!");
    PreparedStatement retval = org_conn.prepareStatement(sql);
    return retval;
  }
  public CallableStatement prepareCall(String sql) throws SQLException {
//System.out.println("prepareCall!!!");
    CallableStatement retval = org_conn.prepareCall(sql);
    return retval;
  }
  public String nativeSQL(String sql) throws SQLException {
//System.out.println("nativeSQL!!!");
    String retval = org_conn.nativeSQL(sql);
    return retval;
  }
  public void setAutoCommit(boolean autoCommit) throws SQLException {
//System.out.println("setAutoCommit!!!");
    org_conn.setAutoCommit(autoCommit);
  }
  public boolean getAutoCommit() throws SQLException {
//System.out.println("getAutoCommit!!!");
    boolean retval = org_conn.getAutoCommit();
    return retval;
  }
  public void commit() throws SQLException {
//System.out.println("commit!!!");
    org_conn.commit();
  }
  public void rollback() throws SQLException {
//System.out.println("rollback!!!");
    org_conn.rollback();
  }
  public DatabaseMetaData getMetaData() throws SQLException {
//System.out.println("getMetaData!!!");
    DatabaseMetaData retval = org_conn.getMetaData();
    return retval;
  }
  public void setReadOnly(boolean readOnly) throws SQLException {
//System.out.println("setReadOnly!!!");
    org_conn.setReadOnly(readOnly);
  }
  public boolean isReadOnly() throws SQLException {
//System.out.println("isReadOnly!!!");
    boolean retval = org_conn.isReadOnly();
    return retval;
  }
  public void setCatalog(String catalog) throws SQLException {
//System.out.println("setCatalog!!!");
    org_conn.setCatalog(catalog);
  }
  public String getCatalog() throws SQLException {
//System.out.println("getCatalog!!!");
    String retval = org_conn.getCatalog();
    return retval;
  }
  public void setTransactionIsolation(int level) throws SQLException {
//System.out.println("setTransactionIsolation!!!");
    org_conn.setTransactionIsolation(level);
  }
  public int getTransactionIsolation() throws SQLException {
//System.out.println("getTransactionIsolation!!!");
    int retval = org_conn.getTransactionIsolation();
    return retval;
  }
  public SQLWarning getWarnings() throws SQLException {
//System.out.println("getWarnings!!!");
    SQLWarning retval = org_conn.getWarnings();
    return retval;
  }
  public void clearWarnings() throws SQLException {
//System.out.println("clearWarnings!!!");
    org_conn.clearWarnings();
  }
  public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
//System.out.println("createStatement Ex!!!");
    Statement retval = org_conn.createStatement(resultSetType, resultSetConcurrency);
    return retval;
  }
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
//System.out.println("prepareStatement Ex!!!");
    PreparedStatement retval = org_conn.prepareStatement(sql, resultSetType, resultSetConcurrency);
    return retval;
  }
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
//System.out.println("prepareCall Ex!!!");
    CallableStatement retval = org_conn.prepareCall(sql, resultSetType, resultSetConcurrency);
    return retval;
  }
  public Map getTypeMap() throws SQLException {
//System.out.println("getTypeMap!!!");
    Map retval = org_conn.getTypeMap();
    return retval;
  }
  public void setTypeMap(Map map) throws SQLException {
//System.out.println("setTypeMap!!!");
    org_conn.setTypeMap(map);
  }

  //------------------------- JDBC 3.0 -----------------------------------

  public void setHoldability(int holdability) throws SQLException {
//System.out.println("setHoldability!!!");
    org_conn.setHoldability(holdability);
  }
  public int getHoldability() throws SQLException {
//System.out.println("getHoldability!!!");
    int retval = org_conn.getHoldability();
    return retval;
  }
  public Savepoint setSavepoint() throws SQLException {
//System.out.println("setSavepoint!!!");
    Savepoint retval = org_conn.setSavepoint();
    return retval;
  }
  public Savepoint setSavepoint(String name) throws SQLException {
//System.out.println("setSavepoint!!!");
    Savepoint retval = org_conn.setSavepoint(name);
    return retval;
  }
  public void rollback(Savepoint savepoint) throws SQLException {
//System.out.println("rollback!!!");
    org_conn.rollback(savepoint);
  }
  public void releaseSavepoint(Savepoint savepoint) throws SQLException {
//System.out.println("releaseSavepoint!!!");
    org_conn.releaseSavepoint(savepoint);
  }
  public Statement createStatement(int resultSetType, int resultSetConcurrency,
                                   int resultSetHoldability) throws SQLException {
//System.out.println("createStatement!!!");
    Statement retval = org_conn.createStatement(resultSetType,
                                   resultSetConcurrency, resultSetHoldability);
    return retval;
  }
  public PreparedStatement prepareStatement(String sql, int resultSetType,
                                      int resultSetConcurrency, int resultSetHoldability)
                                                              throws SQLException {
//System.out.println("prepareStatement!!!");
    PreparedStatement retval = org_conn.prepareStatement(sql, resultSetType,
                                   resultSetConcurrency, resultSetHoldability);
    return retval;
  }
  public CallableStatement prepareCall(String sql, int resultSetType,
                                      int resultSetConcurrency, int resultSetHoldability)
                                                              throws SQLException {
//System.out.println("prepareCall!!!");
    CallableStatement retval = org_conn.prepareCall(sql, resultSetType,
                                   resultSetConcurrency, resultSetHoldability);
    return retval;
  }
  public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys)
                                                              throws SQLException {
//System.out.println("prepareStatement!!!");
    PreparedStatement retval = org_conn.prepareStatement(sql, autoGeneratedKeys);
    return retval;
  }
  public PreparedStatement prepareStatement(String sql, int[] columnIndexes)
                                                              throws SQLException {
//System.out.println("prepareStatement!!!");
    PreparedStatement retval = org_conn.prepareStatement(sql, columnIndexes);
    return retval;
  }
  public PreparedStatement prepareStatement(String sql, String[] columnNames)
                                                              throws SQLException {
//System.out.println("prepareStatement!!!");
    PreparedStatement retval = org_conn.prepareStatement(sql, columnNames);
    return retval;
  }

  private final static int DEF_TIMEOUT = 60;
  private int timeout = gweMysqlXAConnection.DEF_TIMEOUT;
  private Xid xaid = null;
  public void start(Xid xid, int flags) throws XAException {
//System.out.println("(XA)start" + ":" + xid + ":" + flags);
    if( xid == null )
      throw new XAException(XAException.XAER_INVAL);

    if( flags == TMNOFLAGS )
    {
      if( xaid != null )
        throw new XAException(XAException.XAER_DUPID);
      xaid = xid;
      try {
        setAutoCommit(false);
      }
      catch(SQLException ex) {
        throw new XAException(XAException.XAER_RMERR);
      }
    }
    else if( flags == TMJOIN )
    {
      ;
    }
    else if( flags == TMRESUME )
    {
      ;
    }
    else
    {
      throw new XAException(XAException.XAER_INVAL);
    }
  }

  public void end(Xid xid, int flags) throws XAException {
//System.out.println("(XA)end" + ":" + xid + ":" + flags);
    if( xid == null )
      throw new XAException(XAException.XAER_INVAL);

    if( flags == TMSUSPEND )
    {
      if( xaid == null || !xaid.equals(xid) )
        throw new XAException(XAException.XAER_NOTA);
    }
    else if( flags == TMFAIL )
    {
      xaid = null;
    }
    else if( flags == TMSUCCESS )
    {
      xaid = null;
    }
    else
    {
      throw new XAException(XAException.XAER_INVAL);
    }
  }

  public int prepare(Xid xid) throws XAException {
//System.out.println("(XA)prepare" + ":" + xid);
    if( xid == null )
      throw new XAException(XAException.XAER_INVAL);

    return XA_OK;
  }

  public void commit(Xid xid, boolean onePhase) throws XAException {
//System.out.println("(XA)commit" + ":" + xid + ":" + onePhase);
    if( xid == null )
      throw new XAException(XAException.XAER_INVAL);

    if( onePhase )
    {
      try {
        commit();
      }
      catch(SQLException ex) {
        throw new XAException(XAException.XA_HEURRB);
      }
    }
    else
    {
      try {
        commit();
      }
      catch(SQLException ex) {
        throw new XAException(XAException.XA_HEURRB);
      }
    }
  }

  public void rollback(Xid xid) throws XAException {
//System.out.println("(XA)rollback" + ":" + xid);
    if( xid == null )
      throw new XAException(XAException.XAER_INVAL);

    try {
      rollback();
    }
    catch(SQLException ex) {
      throw new XAException(XAException.XA_RBROLLBACK);
    }
  }

  public Xid[] recover(int flag) throws XAException {
//System.out.println("(XA)recover" + ":" + flag);
    return new Xid[0];
  }

  public void forget(Xid xid) throws XAException {
//System.out.println("(XA)forget" + ":" + xid);
  }

  public int getTransactionTimeout() throws XAException {
//System.out.println("(XA)getTransactionTimeout");
    return timeout;
  }

  public boolean setTransactionTimeout(int seconds) throws XAException {
//System.out.println("(XA)setTransactionTimeout" + ":" + seconds);
    if( seconds < 0 )
      throw new XAException(XAException.XAER_INVAL);
    else if( seconds == 0 )
      timeout = gweMysqlXAConnection.DEF_TIMEOUT;
    else
      timeout = seconds;
    return true;
  }

  public boolean isSameRM(XAResource xares) throws XAException {
//System.out.println("(XA)isSameRM" + ":" + xares);
    if( xares != null && xares instanceof gweMysqlXAConnection )
    {
      gweMysqlXAConnection that = (gweMysqlXAConnection)xares;
      return (this.pool == that.pool);
    }
    return false;
  }
}
