namespace net.sql {

using System;
using System.Reflection;

public class DriverManager {

  private static System.Collections.ArrayList drivers =
                                         new System.Collections.ArrayList();
  private static int loginTimeout = 0;
  private static System.IO.TextWriter logWriter = null;

  private DriverManager(){
  }

  public static System.IO.TextWriter getLogWriter() {
    return logWriter;
  }

  public static void setLogWriter(System.IO.TextWriter out_) {
    lock(typeof(DriverManager)) {
      logWriter = out_;
    }
  }

  // throws SQLException
  public static Connection getConnection(string url,
                                         System.Collections.Hashtable info) {
    lock(typeof(DriverManager)) {
      SQLException ex = null;
      for(int i = 0; i < drivers.Count; i++)
      {
        DriverInfo d = (DriverInfo)drivers[i];
        try {
          Connection retval = d.driver.connect(url, info);
          if( retval != null )
            return retval;
        }
        catch(SQLException e) {
          if( ex == null )
            ex = e;
        }
      }
      if( ex != null )
        throw ex;
      throw new SQLException("There is no suitable driver.");
    }
  }

  // throws SQLException
  public static Connection getConnection(string url,
                                         string user, string password) {
    lock(typeof(DriverManager)) {
      System.Collections.Hashtable info = new System.Collections.Hashtable();
      if( user != null )
        info.Add("user", user);
      if( password != null )
        info.Add("password", password);
      return getConnection(url, info);
    }
  }

  // throws SQLException
  public static Connection getConnection(string url) {
    lock(typeof(DriverManager)) {
      System.Collections.Hashtable info = new System.Collections.Hashtable();
      return getConnection(url, info);
    }
  }

  // throws SQLException
  public static void registerDriver(string driverClassName) {
    lock(typeof(DriverManager)) {
      Type callerC = Type.GetType(driverClassName, true);
      ConstructorInfo ci = callerC.GetConstructor(new Type[0]);
      Driver driver = (Driver)ci.Invoke(new object[0]);
      registerDriver(driver);
    }
  }

  // throws SQLException
  public static void registerDriver(net.sql.Driver driver) {
    lock(typeof(DriverManager)) {
      DriverInfo d = new DriverInfo();
      d.driver          = driver;
      d.driverClass     = driver.GetType();
      d.driverClassName = d.driverClass.FullName;
      drivers.Add(d);
    }
  }

  public static void setLoginTimeout(int seconds) { 
    loginTimeout = seconds;
  }

  public static int getLoginTimeout() {
    return loginTimeout;
  }

  public static void println(string message) {
    lock(typeof(DriverManager)) {
      if( logWriter != null )
      {
        logWriter.WriteLine(message);
        logWriter.Flush();
      }
    }
  }
}

internal class DriverInfo {
  internal Driver driver;
  internal Type   driverClass;
  internal string driverClassName;

  public override string ToString() {
    return ("Driver[classFullName=" + driverClassName + "," + driver + "]");
  }
}

}
