namespace net.sql {

public enum ResultSet_FETCH {
  FORWARD = 1000,
  REVERSE = 1001,
  UNKNOWN = 1002,
}

public enum ResultSet_TYPE {
  FORWARD_ONLY       = 1003,
  SCROLL_INSENSITIVE = 1004,
  SCROLL_SENSITIVE   = 1005,
}

public enum ResultSet_CONCUR {
  READ_ONLY = 1007,
  UPDATABLE = 1008,
}

public enum ResultSet_CURSORS {
  HOLD_CURSORS_OVER_COMMIT = 1,
  CLOSE_CURSORS_AT_COMMIT  = 2,
}

public interface ResultSet {
  // throws SQLException
  bool next();

  // throws SQLException
  void close();

  // throws SQLException
  bool wasNull();

  // throws SQLException
  string getString(int columnIndex);

  // throws SQLException
  bool getBoolean(int columnIndex);

  // throws SQLException
  sbyte getByte(int columnIndex);

  // throws SQLException
  short getShort(int columnIndex);

  // throws SQLException
  int getInt(int columnIndex);

  // throws SQLException
  long getLong(int columnIndex);

  // throws SQLException
  float getFloat(int columnIndex);

  // throws SQLException
  double getDouble(int columnIndex);

  // throws SQLException
  System.Decimal getBigDecimal(int columnIndex, int scale);

  // throws SQLException
  sbyte[] getSBytes(int columnIndex);
  byte[] getUBytes(int columnIndex); // for only C#!!

  // throws SQLException
  System.DateTime getDate(int columnIndex);

  // throws SQLException
  System.DateTime getTime(int columnIndex);

  // throws SQLException
  System.DateTime getTimestamp(int columnIndex);

  // throws SQLException
  System.IO.BinaryReader getAsciiStream(int columnIndex);

  // throws SQLException
  System.IO.BinaryReader getUnicodeStream(int columnIndex);

  // throws SQLException
  System.IO.BinaryReader getBinaryStream(int columnIndex);

  // throws SQLException
  string getString(string columnName);

  // throws SQLException
  bool getBoolean(string columnName);

  // throws SQLException
  sbyte getByte(string columnName);

  // throws SQLException
  short getShort(string columnName);

  // throws SQLException
  int getInt(string columnName);

  // throws SQLException
  long getLong(string columnName);

  // throws SQLException
  float getFloat(string columnName);

  // throws SQLException
  double getDouble(string columnName);

  // throws SQLException
  System.Decimal getBigDecimal(string columnName, int scale);

  // throws SQLException
  sbyte[] getSBytes(string columnName);
  byte[] getUBytes(string columnName); // for only C#!!

  // throws SQLException
  System.DateTime getDate(string columnName);

  // throws SQLException
  System.DateTime getTime(string columnName);

  // throws SQLException
  System.DateTime getTimestamp(string columnName);

  // throws SQLException
  System.IO.BinaryReader getAsciiStream(string columnName);

  // throws SQLException
  System.IO.BinaryReader getUnicodeStream(string columnName);

  // throws SQLException
  System.IO.BinaryReader getBinaryStream(string columnName);

  // throws SQLException
  SQLWarning getWarnings();

  // throws SQLException
  void clearWarnings();

  // throws SQLException
  string getCursorName();

  // throws SQLException
  ResultSetMetaData getMetaData();

  // throws SQLException
  object getObject(int columnIndex);

  // throws SQLException
  object getObject(string columnName);

  // throws SQLException
  int findColumn(string columnName);

  // throws SQLException
  System.IO.StreamReader getCharacterStream(int columnIndex);

  // throws SQLException
  System.IO.StreamReader getCharacterStream(string columnName);

  // throws SQLException
  System.Decimal getBigDecimal(int columnIndex);

  // throws SQLException
  System.Decimal getBigDecimal(string columnName);

  // throws SQLException
  bool isBeforeFirst();

  // throws SQLException
  bool isAfterLast();

  // throws SQLException
  bool isFirst();

  // throws SQLException
  bool isLast();

  // throws SQLException
  void beforeFirst();

  // throws SQLException
  void afterLast();

  // throws SQLException
  bool first();

  // throws SQLException
  bool last();

  // throws SQLException
  int getRow();

  // throws SQLException
  bool absolute( int row );

  // throws SQLException
  bool relative( int rows );

  // throws SQLException
  bool previous();

  // throws SQLException
  void setFetchDirection(int direction);

  // throws SQLException
  int getFetchDirection();

  // throws SQLException
  void setFetchSize(int rows);

  // throws SQLException
  int getFetchSize();

  // throws SQLException
  int getType();

  // throws SQLException
  int getConcurrency();

  // throws SQLException
  bool rowUpdated();

  // throws SQLException
  bool rowInserted();

  // throws SQLException
  bool rowDeleted();

  // throws SQLException
  void updateNull(int columnIndex);

  // throws SQLException
  void updateBoolean(int columnIndex, bool x);

  // throws SQLException
  void updateByte(int columnIndex, sbyte x);

  // throws SQLException
  void updateShort(int columnIndex, short x);

  // throws SQLException
  void updateInt(int columnIndex, int x);

  // throws SQLException
  void updateLong(int columnIndex, long x);

  // throws SQLException
  void updateFloat(int columnIndex, float x);

  // throws SQLException
  void updateDouble(int columnIndex, double x);

  // throws SQLException
  void updateBigDecimal(int columnIndex, System.Decimal x);

  // throws SQLException
  void updateString(int columnIndex, string x);

  // throws SQLException
  void updateBytes(int columnIndex, sbyte[] x);

  // throws SQLException
  void updateDate(int columnIndex, System.DateTime x);

  // throws SQLException
  void updateTime(int columnIndex, System.DateTime x);

  // throws SQLException
  void updateTimestamp(int columnIndex, System.DateTime x);

  // throws SQLException
  void updateAsciiStream(int columnIndex,
                         System.IO.BinaryReader x,
                         int length);

  // throws SQLException
  void updateBinaryStream(int columnIndex,
                          System.IO.BinaryReader x,
                          int length);

  // throws SQLException
  void updateCharacterStream(int columnIndex,
                             System.IO.StreamReader x,
                             int length);

  // throws SQLException
  void updateObject(int columnIndex, object x, int scale);

  // throws SQLException
  void updateObject(int columnIndex, object x);

  // throws SQLException
  void updateNull(string columnName);

  // throws SQLException
  void updateBoolean(string columnName, bool x);

  // throws SQLException
  void updateByte(string columnName, sbyte x);

  // throws SQLException
  void updateShort(string columnName, short x);

  // throws SQLException
  void updateInt(string columnName, int x);

  // throws SQLException
  void updateLong(string columnName, long x);

  // throws SQLException
  void updateFloat(string columnName, float x);

  // throws SQLException
  void updateDouble(string columnName, double x);

  // throws SQLException
  void updateBigDecimal(string columnName, System.Decimal x);

  // throws SQLException
  void updateString(string columnName, string x);

  // throws SQLException
  void updateBytes(string columnName, sbyte[] x);

  // throws SQLException
  void updateDate(string columnName, System.DateTime x);

  // throws SQLException
  void updateTime(string columnName, System.DateTime x);

  // throws SQLException
  void updateTimestamp(string columnName, System.DateTime x);

  // throws SQLException
  void updateAsciiStream(string columnName,
                         System.IO.BinaryReader x,
                         int length);

  // throws SQLException
  void updateBinaryStream(string columnName,
                          System.IO.BinaryReader x,
                          int length);

  // throws SQLException
  void updateCharacterStream(string columnName,
                             System.IO.StreamReader reader,
                             int length);

  // throws SQLException
  void updateObject(string columnName, object x, int scale);

  // throws SQLException
  void updateObject(string columnName, object x);

  // throws SQLException
  void insertRow();

  // throws SQLException
  void updateRow();

  // throws SQLException
  void deleteRow();

  // throws SQLException
  void refreshRow();

  // throws SQLException
  void cancelRowUpdates();

  // throws SQLException
  void moveToInsertRow();

  // throws SQLException
  void moveToCurrentRow();

  // throws SQLException
  Statement getStatement();

  // throws SQLException
  object getObject(int i, System.Collections.Hashtable map);

  // throws SQLException
  Ref getRef(int i);

  // throws SQLException
  Blob getBlob(int i);

  // throws SQLException
  Clob getClob(int i);

  // throws SQLException
  Array getArray(int i);

  // throws SQLException
  object getObject(string colName, System.Collections.Hashtable map);

  // throws SQLException
  Ref getRef(string colName);

  // throws SQLException
  Blob getBlob(string colName);

  // throws SQLException
  Clob getClob(string colName);

  // throws SQLException
  Array getArray(string colName);

  // throws SQLException
  System.DateTime getDate(int columnIndex, System.Globalization.Calendar cal);

  // throws SQLException
  System.DateTime getDate(string columnName, System.Globalization.Calendar cal);

  // throws SQLException
  System.DateTime getTime(int columnIndex, System.Globalization.Calendar cal);

  // throws SQLException
  System.DateTime getTime(string columnName, System.Globalization.Calendar cal);

  // throws SQLException
  System.DateTime getTimestamp(int columnIndex, System.Globalization.Calendar cal);

  // throws SQLException
  System.DateTime getTimestamp(string columnName, System.Globalization.Calendar cal);

  // throws SQLException
  string getURL(int columnIndex);

  // throws SQLException
  string getURL(string columnName);

  // throws SQLException
  void updateRef(int columnIndex, Ref x);

  // throws SQLException
  void updateRef(string columnName, Ref x);

  // throws SQLException
  void updateBlob(int columnIndex, Blob x);

  // throws SQLException
  void updateBlob(string columnName, Blob x);

  // throws SQLException
  void updateClob(int columnIndex, Clob x);

  // throws SQLException
  void updateClob(string columnName, Clob x);

  // throws SQLException
  void updateArray(int columnIndex, Array x);

  // throws SQLException
  void updateArray(string columnName, Array x);
}

}
